/*
 * ProfilerCore.java
 *
 * Version:
 *    $Id: ProfilerCore.java,v 1.5 2005/06/19 21:41:14 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ProfilerCore.java,v $
 *    Revision 1.5  2005/06/19 21:41:14  gap7472
 *    -Name changed from GreedyCore.java -> ProfilerCore.java
 *    -All console output moved to the more elegant GUI interface.
 *    -Added progress manager calls to inform user of status while repeats
 *     are added and data is broken into fragments.
 *    -Moved runtime monitoring into a custom implementation of progress
 *     manager.
 *    -Added a method for clearing memory via the garbage collector.
 *    -Added calls for allowing the GUI to obtain information to display
 *     on summary screen.
 *    -Added ability to save results to a text file or copy them to the
 *     clipboard.
 *    -Modified the way the GUI passes parameters into the core by.
 *     encapsulating them into a new class called ParamInfo.
 *    -Modified repeat generation to allow for short, medium, and large
 *     repeat sizes.
 *    -Added capability to use either generated data or user-supplied data.
 *
 *    Revision 1.4  2005/06/08 02:50:53  gap7472
 *    Minor changes.
 *
 *    Revision 1.3  2005/06/08 02:47:59  gap7472
 *    -Rewrote javadocs and commenting to be much clearer and more accurate.
 *    -Refomatted code lines to be no longer than 70-80 characters,
 *     as per the Sun Coding standard.
 *    -Modified generation of fragments so that it no longer uses an
 *     average size, but instead takes a min and max size.
 *    -Modified repeat generation in similar way to fragment generation.
 *    -Formatted output to console to be nicer.
 *    -Added the ability for the user to stop the algorithm while it is
 *    -executing.
 *    -Modified contig output to display all contigs, not just the longest one.
 *    -Moved statistics output for the end of the algorithm run so that it is
 *     now in a separate method called printStats().
 *    -Added algorithm runtime timing.
 *    -Began work on string comparison algorithm.
 *
 *    Revision 1.2  2005/05/09 07:24:13  gap7472
 *    Modified to allow user to choose average piece size and average
 *    repeat size
 *
 *    Revision 1.1  2005/05/04 17:36:52  gap7472
 *    Initial revision
 *
 */

import alg.greedy.*;
import util.*;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.datatransfer.*;

/**
 * This class is the core of the program. The GUI calls the methods
 * in this class to perform the program tasks.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ProfilerCore
{
    private static ProfilerProgress progress;
    private static GreedyAlg        greedy          = null;
    private static StringBuffer     data;
    private static String[]         contigs;

    public static void run(ParamInfo params,
                           ProfilerProgress progressManager)
    {
        LinkedList<String>  fragments;

        // Clear contigs, just in case
        contigs             = new String[0];

        // Sync progress manager
        progress            = progressManager;

        switch (params.getMode())
        {
            case IM_GENERATE_DATA:
                // Prepare the initial random data
                data    = generateData(params.getNucleotideInfo(),
                                       params.getDataLength());
                break;

            case IM_SUPPLIED_DATA:
                data    = params.getSuppliedData();
                break;
        }

        // Add repeats
        data                = generateRepeats(data, params.getRepeatInfo());

        // Split the data into fragments
        fragments           = generateFragments(data,
                                                params.getFragmentInfo());

        // Run the algorithm
        runAlg(fragments);
    }

    /**
     * Method called by the GUI to stop the execution of the greedy
     * algorithm.
     */
    public static void stop()
    {
        (new Thread()
        {
            public void run()
            {
                // Wait until the algorithm has been initialized before
                // stopping it
                while (greedy == null)
                {
                    // Do nothing
                }

                greedy.stop();
            }
        }).start();
    }

    /**
     * This method will generate random data that consists of the
     * letters A, C, G, T, representing each of the four nucleotide
     * bases that make up DNA. The data will be generated according
     * to the specified nucleotide distribution and will be of the
     * requested length.
     *
     * @param   distInfo    An instance of NucleotideDistInfo,
     *                      describing how the various nucleotide
     *                      bases are to be distributed throughout
     *                      the generated data
     * @param   length      The desired length of the generated data
     */
    private static StringBuffer generateData(NucleotideDistInfo distInfo,
                                             int length)
    {
        int             spinner;
        StringBuffer    data    = new StringBuffer();

        progress.setStatusString("Generating data...");

        // Keep generating data until we have the desired length
        for (int i = 0; i < length; i++)
        {
            spinner = (int)(Math.random() * 100);

	        if (spinner < distInfo.getMaxA())
                data.append('A');
            
            else if (spinner < distInfo.getMaxC())
                data.append('C');
            
            else if (spinner < distInfo.getMaxG())
                data.append('G');

            else
                data.append('T');
        }

        return (data);
    }

    /**
     * This method will generate repeats from initial data, insert
     * the repeats in random locations throughout the data, and
     * return the result.
     *
     * @param   data    The data from which repeats will be generated
     * @param   repInfo An 3-element array of ValueInfo, with each
     *                  element describing the number and min and max
     *                  length of short, medium, and large repeats
     *                  respectively.
     */
    private static StringBuffer generateRepeats(StringBuffer data,
                                                ValueInfo[] repInfo)
    {
        progress.setStatusString("Generating repeats...");

        // Inform the progress manager of the total number of repeats
        progress.setNumRepeatsNeeded(repInfo[0].getHowMany() +
                                     repInfo[1].getHowMany() +
                                     repInfo[2].getHowMany());

        // Run through, generating repeats of each size
        for (int i = 0; i < 3; i++)
        {
            data    = generateRepeats(data,
                                      repInfo[i].getHowMany(),
                                      repInfo[i].getMinLength(),
                                      repInfo[i].getMaxLength());
        }

        return (data);
    }

    /**
     * This method generates sections of repeat data by copying random
     * sections of varying size and reinserting them at random positions.
     *
     * @param   data            The data from which to generate repeats.
     * @param   numRepeats      The number of repeats to generate
     * @param   minSize         The minimum size of each repeat section
     * @param   maxSize         The maximum size of each repeat section
     */
    private static StringBuffer generateRepeats(StringBuffer data,
                                                int numRepeats,
                                                int minLen, int maxLen)
    {
        StringBuffer    retData;
        int             length,
                        startPos,
                        endPos,
                        insertPos;

        // Nothing to do if we do not need to repeat
        if (numRepeats == 0)
            return (data);

        // Make a copy of the initial data to prevent changing it
        retData = new StringBuffer(data.toString());

        // Don't allow a repeat to be larger than the data itself
        //
        // FIXME:   Shouldn't the GUI prevent this from happening?
        if (maxLen > data.length())
            maxLen  = data.length();

        // Keep generating repeats until we have the desired number
        for (int i = 0; i < numRepeats; i++)
        {
            length      = (int)(Math.random() * (maxLen - minLen)) + minLen;
            startPos    = (int)(Math.random() * (data.length() - length));
            endPos      = startPos + length;
            insertPos   = (int)(Math.random() * (retData.length()));

            retData.insert(insertPos, data.substring(startPos, endPos));

            progress.repeatGenerated();
        }

        return (retData);
    }

    /**
     * This method generates fragments of varying size from the data.
     *
     * @param   data            The data to split into fragments
     * @param   numPieces       The number of fragments to create
     * @param   minLen          The minimum size of each fragment
     * @param   maxLen          The maximum size of each fragment
     * @return                  A LinkedList containing the resulting
     *                          fragments
     */
    private static LinkedList<String> generateFragments(StringBuffer data,
                                                        ValueInfo fragInfo)
    {
        LinkedList<String>      fragments;
        int                     numFrags    = fragInfo.getHowMany(),
                                minLen      = fragInfo.getMinLength(),
                                maxLen      = fragInfo.getMaxLength(),
                                start,
                                length;

        fragments   = new LinkedList<String>();

        progress.setStatusString("Generating fragments...");

        // Don't allow a fragment to be larger than the data itself
        //
        // FIXME:   Shouldn't the GUI prevent this from happening?
        if (maxLen > data.length())
            maxLen  = data.length();

        // Keep generating fragments until we have met the number
        // requested by the user
        for (int i = 0; i < numFrags; i++)
        {
            length  = (int)(Math.random() * (maxLen - minLen)) + minLen;
            start   = (int)(Math.random() * (data.length() - length));

            fragments.add(data.substring(start, start + length));
        }

        return (fragments);
    }

    /**
     * This method simply runs the greedy algorithm with the
     * generated fragments.
     *
     * @param   fragments   The generated fragments
     */
    private static void runAlg(LinkedList<String> fragments)
    {
        greedy      = new GreedyAlg(fragments.size());
        
        greedy.setProgressManager(progress);
        
        // Indicate we've completed all initial steps
        progress.initialStepsComplete();

        // Clear memory; the algorithm will likely use a lot
        // of memory
        clearMemory();

        // Start monitoring memory and runtime
        progress.startMonitors();

        // Run the algorithm with the fragments we've created
        contigs = greedy.run(fragments.toArray(new String[0]));

        // Stop monitoring memory and runtime
        progress.stopMonitors();

        // Clear memory; the algorithm likely used a lot of memory
        clearMemory();
    }

    /**
     * This method calls the garbage collector to attempt to free
     * as much memory as possible.
     *
     * The status text is updated before collection proceeds to
     * inform the user (via the progress dialog box) that garbage
     * collection is underway.
     */
    private static void clearMemory()
    {
        Thread  cleanThread = new SleepThread()
        {
            public void run()
            {
                progress.setStatusString("Clearing memory...");

                // Give the progress dialog box ample time to display
                // what is going on before the garbage collector
                // pauses the program
                sleep(500);

                System.gc();
                System.runFinalization();
            }
        };

        cleanThread.start();

        // Wait for garbage collection to finish
        while (cleanThread.isAlive())
        {
            // Do nothing
        }
    }

    /**
     * This method provides access to the data that was used to
     * construct fragments for the greedy algorithm.
     *
     * @return  The initial data that was split into fragments
     */
    public static StringBuffer getData()
    {
        return (data);
    }

    /**
     * This method provides access to the contigs generated
     * by the greedy algorithm.
     *
     * @return  The contigs generated by the greedy algorithm
     */
    public static String[] getContigs()
    {
        return (contigs);
    }

    /**
     * This method generates the output that is used for saving
     * results or copying them to the clipboard.
     *
     * @param   out     The PrintWriter to which the output will
     *                  be written.
     * @param   stats   A string array of the statistics that are
     *                  to be written out following the input and
     *                  output data of the greedy algorithm
     */
    private static void writeResults(PrintWriter out, String stats[])
    {
        // Header & Timestamp
        out.println("Greedy Algorithm Run - " + progress.getStartTime());
        out.println("===================================================");
        out.println();

        // Input Data
        out.println("Input Data");
        out.println("----------");
        out.println(data);
        
        // Output Data (Contigs)
        if (contigs.length > 0)
        {
            for (int i = 0; i < contigs.length; i++)
            {
                out.println();
                out.println("Contig " + (i + 1));
                out.println("----------");
                out.println(contigs[i]);
            }
        }

        // Statistics
        out.println();
        out.println("Execution Statistics");
        out.println("--------------------");

        for (String statLine : stats)
            out.println(statLine);

        // If the algorithm ended prematurely, make sure that this
        // is recorded.
        if (contigs.length == 0)
        {
            out.println();
            out.println("The algorithm did not run to completion.");
        }
    }

    /**
     * This method will save the results of an execution of the
     * greedy algorithm to a file.
     *
     * @param   outputFile  The file to which the results will be
     *                      saved
     * @param   stats       A string array of the statistics that are
     *                      to be written out following the input and
     *                      output data of the greedy algorithm
     */
    public static void saveResults(File outputFile, String stats[])
    throws IOException
    {
        PrintWriter out = new PrintWriter(
                             new BufferedWriter(
                                new FileWriter(outputFile)), true);

        writeResults(out, stats);

        out.close();
    }

    /**
     * This method will copy the results of an execution of the
     * greedy algorithm to the clipboard.
     *
     * @param   stats       A string array of the statistics that are
     *                      to be written out following the input and
     *                      output data of the greedy algorithm
     */
    public static void copyResults(String stats[])
    {
        ByteArrayOutputStream   resultsStream;
        String                  resultsText;
        Transferable            resultsTransfer;
        Clipboard               clipboard;

        resultsStream   = new ByteArrayOutputStream();

        writeResults(new PrintWriter(resultsStream, true), stats);

        clipboard       = Toolkit.getDefaultToolkit().getSystemClipboard();
        resultsText     = resultsStream.toString();
        resultsTransfer = new StringSelection(resultsText);

        clipboard.setContents(resultsTransfer, null);
    }
}

