/*
 * ProgressDialog.java
 *
 * Version:
 *    $Id: ProgressDialog.java,v 1.1 2005/06/20 00:59:26 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ProgressDialog.java,v $
 *    Revision 1.1  2005/06/20 00:59:26  gap7472
 *    Initial revision
 *
 */

import alg.greedy.*;
import gui.*;
import util.*;

import java.text.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

/**
 * This dialog box is displayed to the user while the profiler is
 * preparing for, and running, the greedy algorithm. It displays
 * the status and overall progress of the run, as well as the
 * elapsed runtime and memory usage.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ProgressDialog extends JDialog
{
    /**
     * The "stage director" for this frame -- it ensures that
     * the right methods are called when user-triggered
     * events occur (button clicks, etc).
     */
    EventDispatcher     eventManager;

    /**
     * The progress manager that we will use to obtain the
     * information we show to the user.
     */
    ProfilerProgress    progress;

    /**
     * This allows us to obtain the memory usage value we show
     * to the user while the algorithm runs.
     */
    MemoryWatcher       memWatch;

    /*
     * The elements that make up this dialog.
     */
    JPanel              topPanel,
                        infoPanel,
                        bottomPanel;
    JProgressBar        progressBar;
    JLabel              statusLabel,
                        timerLabel,
                        memUsageLabel;
    JButton             stopBtn;

    /**
     * Constructor for ProgressDialog.
     *
     * The dialog is created as a modal child of the specified
     * parent and will use the specified progress manager to
     * obtain the information that it displays to the user.
     *
     * @param   parent          The main window for which this dialog
     *                          will be a child
     * @param   progressManager The progress manager that the dialog
     *                          will use to obtain information about
     *                          the running task
     */
    public ProgressDialog(JFrame parent,
                          ProfilerProgress progressManager)
    {
        super(parent, true);

        // Setup our event manager
        eventManager    = new EventDispatcher(this);

        progress        = progressManager;
        memWatch        = progress.getMemWatcher();

        setTitle("Algorithm Running");
        setPreferredSize(new Dimension(300, 135));
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setLayout(new BorderLayout());
        setResizable(false);

        // Capture the close box on the dialog so that
        // we can stop the profiler before we close.
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                // Stop the current task
                ProfilerCore.stop();

                // Close the dialog
                dispose();
            }
        });

        addComponents();

        // Center on our parent
        setLocationRelativeTo(parent);
    }

    /**
     * This adds the components to the form of the dialog.
     */
    private void addComponents()
    {
        // TOP
        topPanel        = new JPanel();
        statusLabel     = new JLabel();
        progressBar     = new JProgressBar(0, 100);

        statusLabel.setPreferredSize(new Dimension(250, 20));
        statusLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

        progressBar.setValue(0);
        progressBar.setStringPainted(true);

        topPanel.setLayout(new BoxLayout(topPanel, BoxLayout.Y_AXIS));
        topPanel.setBorder(new EmptyBorder(5, 5, 0, 5));
        topPanel.add(statusLabel);
        topPanel.add(Box.createVerticalStrut(5));
        topPanel.add(progressBar);

        // MIDDLE (INFO)
        infoPanel       = new JPanel();
        timerLabel      = new JLabel();
        memUsageLabel   = new JLabel();

        infoPanel.setLayout(new BoxLayout(infoPanel, BoxLayout.X_AXIS));
        infoPanel.setBorder(new EmptyBorder(2, 8, 2, 8));
        infoPanel.add(timerLabel);
        infoPanel.add(Box.createHorizontalGlue());
        infoPanel.add(memUsageLabel);

        // BOTTOM
        bottomPanel     = new JPanel();
        stopBtn         = new JButton("Stop");

        eventManager.addActionHandler(stopBtn,  "stopBtnClick");

        bottomPanel.setLayout(new FlowLayout());
        bottomPanel.setBorder(new EmptyBorder(0, 5, 0, 5));
        bottomPanel.add(stopBtn);

        add(topPanel,      BorderLayout.NORTH);
        add(infoPanel,     BorderLayout.CENTER);
        add(bottomPanel,   BorderLayout.SOUTH);

        pack();
    }

    /**
     * Method to show the progress dialog and start updating it
     * at regular intervals.
     */
    public void showProgress()
    {
        Thread showThread   = new SleepThread()
        {
             public void run()
             {
                setVisible(true);
             }
        };

        Thread updateThread = new SleepThread()
        {
            public void run()
            {
                // Wait until the dialog is visible
                while (!isVisible())
                {
                    sleep(25);
                }

                // Keep updating until the dialog is closed
                while (isVisible())
                {
                    updateInfo();

                    // Sleep a bit to prevent excessive updating
                    sleep(25);
                }
            }
        };

        // We run the code to show the dialog in a separate thread
        // because the dialog is modal and will block on setVisible()
        showThread.start();

        // Start updating the dialog
        updateThread.start();
    }

    /**
     * This method is called by the EventHandler when
     * the Stop button is clicked.
     *
     * @param   e   The action event that caused this method
     *              to be invoked.
     */
    public void stopBtnClick(ActionEvent e)
    {
        // Keep the user from clicking twice
        stopBtn.setEnabled(false);
        stopBtn.setText("Stopping...");

        // Signal to stop
        ProfilerCore.stop();
    }

    /**
     * This method is called by the update thread to update the
     * information on the progress dialog.
     */
    public void updateInfo()
    {
        String          timeStr,
                        memStr;
        NumberFormat    fmt         = NumberFormat.getInstance();

        ((DecimalFormat)fmt).setGroupingSize(3);

        // Update status label and progress bar
        statusLabel.setText(progress.getStatusString());
        progressBar.setValue((int)(progress.getPercentComplete()));

        // Only update the time and memory usage if we have
        // that information
        if (progress.isMonitoring())
        {
            timeStr = ProfilerProgress.timeToStr(progress.getTotalTime());
            memStr  = fmt.format(memWatch.getCurUsage() / 1024);

            timerLabel.setText(timeStr);
            memUsageLabel.setText(memStr + " KB");
        }
    }
}