/*
 * GreedyAlg.java
 *
 * Version:
 *    $Id: GreedyAlg.java,v 1.8 2005/06/20 01:32:02 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: GreedyAlg.java,v $
 *    Revision 1.8  2005/06/20 01:32:02  gap7472
 *    -Package changed from GreedyAlg -> alg.greedy.
 *    -Added code to support progress manager and status/progress updates
 *     so that callers can find out how the greedy algorithm is running,
 *     if they so wish.
 *    -Moved counter for contained nodes into progress manager.
 *    -Updated code to reflect minor changes in Heap implementation.
 *
 *    Revision 1.7  2005/06/08 03:07:47  gap7472
 *    -Revised/added many javadoc comments for clarity
 *    -Reformatted code lines to be no longer than 70-80 characters,
 *     as per the Sun Coding standard.
 *    -Added counter and corresponding counter accessor for the number
 *     of contained fragments.
 *    -Added stop() method to stop the algorithm while it is running.
 *    -Added conditions to all loops to allow immediate breaks when stop()
 *     is called.
 *    -Added wasStopped() method to allow callers to determine if the
 *     algorithm was stopped by the stop() method.
 *    -Modified the way that the LinkManager is instantiated so that it is
 *     informed about the number of fragments it should expect, thus
 *     allowing it to perform better.
 *    -Added alternate constructor that can take an array of fragments
 *     and setup the algorithm to handle them.
 *    -Added getNumNodes() to allow callers to determine how many
 *     nodes (fragments) have been added to the algorithm.
 *    -Added addAll() method to allow an array of fragments to be added
 *     quickly.
 *    -Modified addNode() method to throw a NodeOverflowException if a
 *     caller attempts to add too many fragments
 *    -Modified run() to return a Heap of contigs rather than just one
 *    -Added alternate run() which can take an array of fragments and
 *     process them immediately.
 *
 *    Revision 1.5  2005/04/13 06:17:36  gap7472
 *    Normalized line endings.
 *
 *    Revision 1.4  2005/04/13 06:07:53  gap7472
 *    Modified to prevent looping.
 *
 *    Revision 1.1  2005/04/06 05:54:17  gap7472
 *    Initial revision
 *
 */
package alg.greedy;

import util.*;
import util.heap.*;
import util.suffixtree.*;

import java.io.*;
import java.util.*;

/**
 * A greedy algorithm for reconstructing data from fragments
 * of the data.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class GreedyAlg
{
    /**
     * This stores the nodes that the algorithm will attempt to
     * piece together.
     */
    private LinkedList<GraphNode>   nodeList;

    /**
     * This stores and sorts the Arc candidates as they are generated
     * by addNodes().
     */
    private Heap<GraphArc>          candidateHeap;

    /**
     * This keeps track of the connections between nodes so that
     * potential loops are detected before they are formed.
     */
    private LinkManager<GraphNode>  nodeLinks;

    /**
     * The number of nodes (fragments) that the algorithm has been
     * initialized to handle.
     */
    private int                     numNodes;


    /**
     * This keeps track of the progress of the algorithm. It is
     * not initialized by default; a caller wishing to use the
     * progress manager must create an instance of ProgressManager
     * and set this value by a call to setProgressManager().
     */
    private ProgressManager         progressManager;

    /**
     * Used by the stop() method to end the algorithm in the middle
     * of processing.
     */
    private boolean                 stopRunning;

    /**
     * Constructor for GreedyAlg that initializes the algorithm to
     * handle the specified number of fragments.
     *
     * @param   numNodes    The number of fragments that the
     *                      algorithm will be processing.
     */
    public GreedyAlg(int numFragments)
    {
        // Each fragment becomes a node
        numNodes        = numFragments;

        // Allocate enough space to hold up to n-1 arcs for every
        // node
        candidateHeap   = new ArrayHeap<GraphArc>(numNodes * (numNodes - 1));
        //candidateHeap   = new LinkedHeap<GraphArc>();

        nodeList        = new LinkedList<GraphNode>();
        nodeLinks       = new LinkManager<GraphNode>(numFragments);
        progressManager = null;
        stopRunning     = false;
    }

    /**
     * Constructor for GreedyAlg that initializes the algorithm
     * with the specified fragments.
     *
     * @param   fragments   The fragments that the algorithm will
     *                      process.
     */
    public GreedyAlg(String fragments[])
    {
        this(fragments.length);

        addAll(fragments);
    }

    /**
     * This method is used to supply an instance of ProgressManager
     * that will be used by the caller to track the progress of the
     * algorithm as it runs.
     *
     * @param   manager The process manager instance to use for
     *                  progress updates.
     */
    public void setProgressManager(ProgressManager manager)
    {
        progressManager = manager;

        // Inform the progress manager how many fragments we will
        // be handling.
        progressManager.setNumFragments(numNodes);
    }

    /**
     * Returns the number of nodes that have been added to the
     * algorithm.
     *
     * @return  The number of nodes in the algorithm
     */
    public int getNumNodes()
    {
        return (nodeList.size());
    }

    /**
     * Used to check if the algorithm was stopped by a previous
     * call to stop().
     *
     * @return  True if stop() was previously called, false otherwise
     */
    public boolean wasStopped()
    {
        return (stopRunning);
    }

    /**
     * Adds all of the specified fragments to the algorithm as
     * new nodes.
     *
     * @param   fragments   The fragments to be added as nodes
     */
    public void addAll(String fragments[])
    {
        // Convert each fragment into a node
        for (String curFragment : fragments)
        {
            // Stop processing if stop() was called
            if (stopRunning)
                break;

            addNode(curFragment);
        }
    }

    /**
     * Generates a new node from a fragment. The method will look for
     * overlap between the new node and pre-existing nodes, and store
     * any overlaps it finds as arc candidates in the heap.
     *
     * @param   fragment    The fragment to become a new node.
     */
    public void addNode(String fragment)
    throws NodeOverflowException
    {
        GraphNode   newNode = new GraphNode(fragment);
        GraphArc    newArc;

        // Catch trying to add more nodes than we were told to expect
        if (nodeList.size() == numNodes)
            throw new NodeOverflowException(
                                "Attempted to add more nodes than "+
                                "initialized to handle");

        // Loop through all existing nodes looking for overlaps
        for (GraphNode curNode : nodeList)
        {
            // Stop processing if stop() was called
            if (stopRunning)
                break;

            // Look for overlap between the current node we're looking at
            // and the node we've created
            newArc  = findOverlap(curNode, newNode);

            if (newArc != null)
            {
                // Add the arc candidate describing the overlap to the heap
                candidateHeap.push(newArc);
            }

            // Look for overlap between the node we've created and the node
            // we're looking at
            newArc  = findOverlap(newNode, curNode);

            if (newArc != null)
            {
                // Add the arc candidate describing the overlap to the heap
                candidateHeap.push(newArc);
            }
        }

        // Now add the new node to the list of nodes
        nodeList.add(newNode);

        // Update status if we have a progress manager
        if (progressManager != null)
            progressManager.completeFragment();
    }

    /**
     * This method attempts to find an overlap between the ending of one node
     * and the beginning of another.
     *
     * @param   firstNode   The node where an overlap might begin
     * @param   secondNode  The node where an overlap might end
     * @return              An arc representing the overlap between the two
     *                      nodes, or null if no overlap exists.
     */
    private GraphArc findOverlap(GraphNode firstNode, GraphNode secondNode)
    {
        OverlapInfo overlap;

        try
        {
            // Look for an overlap between firstNode and secondNode
            overlap = firstNode.findOverlap(secondNode.getLabel());
        }

        // If secondNode was contained within firstNode,
        // disqualify it because it offers no new information
        catch (ContainmentException e)
        {
            secondNode.setDisqualified(true);

            if (progressManager != null)
                progressManager.fragmentContained();

            return (null);
        }

        if (overlap != null)
        {
            // If we discover overlap, generate an arc to represent it.
            return (new GraphArc(overlap, secondNode, firstNode));
        }

        else
            return (null);
    }

    /**
     * Method to run the greedy algorithm with previously supplied
     * fragments and return the resulting contig(s), if any.
     */
    public String[] run()
    {
        // Process the arc candidates generated when nodes were added
        makeNodeLinks();

        // Return all the contigs that the algorithm can generate,
        // sorted longest length first
        return (buildContigs());
    }

    /**
     * Method to run the greedy algorithm with supplied fragments
     * and return the resulting contig(s), if any.
     *
     * @param   fragments   The fragments to process
     * @return              The contig(s) generated by the algorithm,
     *                      sorted longest first
     */
    public String[] run(String fragments[])
    {
        addAll(fragments);

        return (run());
    }

    /**
     * Stop the algorithm if it is running.
     */
    public void stop()
    {
        stopRunning = true;
    }

    /**
     * This method goes through the heap of arc candidates,
     * largest candidates first, and links nodes together
     * such that:
     *      1) Each node has at most only one arc coming in
     *      2) Each node has at most only one arc going out
     *      3) No connection is made to or from disqualified nodes
     *      4) No loops are formed
     */
    private void makeNodeLinks()
    {
        GraphArc    curArc;
        GraphNode   curHead,
                    curTail;

        // Update status if we have a progress manager
        if (progressManager != null)
            progressManager.setNumArcCandidates(candidateHeap.size());

        // Keep grabbing arcs off the top of the candidate heap until
        // we have no more to grab
        while ((curArc = candidateHeap.pop()) != null)
        {
            // Stop processing if stop() was called
            if (stopRunning)
                break;

            if (progressManager != null)
                progressManager.completeArcCandidate();

            curHead = curArc.getHead();
            curTail = curArc.getTail();

            // If any of the nodes that this arc points to have been
            // disqualified, ignore this arc candidate
            if (curHead.isDisqualified() || curTail.isDisqualified())
                continue;

            // If the tail of this arc already has an arc coming out of it,
            // or the head of this arc already has an arc going into it,
            // ignore this arc candidate
            if ((curHead.getIn() != null) || (curTail.getOut() != null))
                continue;

            // Avoid loops
            if (nodeLinks.areConnected(curHead, curTail))
                continue;

            // Mark the two as linked
            nodeLinks.addLinked(curHead, curTail);

            // Tell the two nodes about each other
            curHead.setIn(curArc);
            curTail.setOut(curArc);
        }
    }

    /**
     * This method returns the contigs that it has from the fragments
     * provided earlier.
     *
     * It does so by locating all of the nodes that have no arcs
     * coming in and calling buildContig() to construct contigs
     * starting from each one.
     *
     * The resulting contigs are inserted into a Heap that sorts
     * them, longest first. The Heap is then returned.
     */
    private String[] buildContigs()
    {
        Heap<WeightedString>    contigHeap;
        String                  contigs[];
        
        contigHeap  = new ArrayHeap<WeightedString>(nodeList.size());
        //contigHeap  = new LinkedHeap<WeightedString>();
        
        // Look through all the nodes, looking for ones that have
        // no incoming arcs (i.e. look for the beginnings of contigs)
        for (GraphNode curNode : nodeList)
        {
            // Stop processing if stop() was called
            if (stopRunning)
                break;

            // Update status if we have a progress manager
            if (progressManager != null)
                progressManager.completeContigCandidate();

            // We don't want disqualified nodes in our contigs
            if (curNode.isDisqualified())
                continue;

            // Once we locate the starting node of a contig, we call
            // buildContig() to generate the contig recursively,
            // starting at that node
            if (curNode.getIn() == null)
                contigHeap.push(new WeightedString(buildContig(curNode)));
        }

        contigs = new String[contigHeap.size()];

        for (int i = 0; i < contigs.length; i++)
        {
            contigs[i] = contigHeap.pop().getValue();
        }

        return (contigs);
    }

    /**
     * This method recursively builds a contig that starts with the
     * specified node.
     *
     * @param   curNode     The node from which to start building
     *                      the contig.
     */
    private String buildContig(GraphNode curNode)
    {
        OverlapInfo overlapInfo;
        GraphArc    outArc              = curNode.getOut();
        String      curLabel            = curNode.getLabel(),
                    firstPart,
                    lastPart;

        // Stop processing if stop() was called
        if (stopRunning)
            return (null);

        // If we hit the end of the nodes, return the label
        // of the current node
        if (outArc == null)
            return (curLabel);

        // Get information about the overlap between this node
        // and the next node
        overlapInfo     = outArc.getOverlapInfo();

        // Cut the first string at the point where the overlap starts
        firstPart       = curLabel.substring(0, overlapInfo.getStart());

        // Get the second string from the splicing of the next one
        lastPart        = buildContig(outArc.getHead());

        // Now, cut off the beginning of the second string that
        // contains the rest of the overlap
        lastPart        = lastPart.substring(curLabel.length() -
                                             overlapInfo.getStart());

        return (firstPart + overlapInfo.getOverlap() + lastPart);
    }
}

/**
 * Exception thrown when trying to add more nodes than the GreedyAlg
 * instance was initialized to hold.
 */
class NodeOverflowException extends RuntimeException
{
    public NodeOverflowException(String msg)
    {
        super(msg);
    }
}