/*
 * ProgressManager.java
 *
 * Version:
 *    $Id: ProgressManager.java,v 1.1 2005/06/20 01:32:02 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ProgressManager.java,v $
 *    Revision 1.1  2005/06/20 01:32:02  gap7472
 *    Initial revision
 *
 *    
 */

package alg.greedy;

/**
 * This class is used by callers to track the progress of the greedy
 * algorithm as it runs.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ProgressManager
{
    /**
     * An enumerated type used for calls to updateProgress() to
     * specify what stage progress is being updated for.
     */
    public enum UpdateType
    {
        /**
         * Status updates for initial progress (i.e. loading data
         * for the algorithm, etc)
         */
        UT_INITIAL,

        /**
         * Status updates when adding fragments
         */
        UT_FRAGS,

        /**
         * Status updates when processing arc candidates
         */
        UT_ARCS,

        /**
         * Status updates when building contigs
         */
        UT_CONTIGS;
    };

    /**
     * These constants control how much "weight" each part of the
     * algorithm's execution holds in the total execution progress.
     *
     * WEIGHT_INITIAL   -   The amount of the total progress
     *                      reserved for initialization or any
     *                      special code that needs to run before
     *                      the actual greedy algorithm can begin
     *                      its work.
     *
     * WEIGHT_FRAGMENTS -   The amount of the total progress
     *                      reserved for the algorithm to process
     *                      and add each individual fragment.
     *
     * WEIGHT_ARCS      -   The amount of the total progress
     *                      reserved for the algorithm to process
     *                      the arc candidates generated when the
     *                      fragments were added.
     *
     * WEIGHT_CONTIGS   -   The amount of the total progress
     *                      reserved for the algorithm to locate
     *                      and build contigs.
     */
    private static final float      WEIGHT_INITIAL      = 10.0f,
                                    WEIGHT_FRAGMENTS    = 30.0f,
                                    WEIGHT_ARCS         = 55.0f,
                                    WEIGHT_CONTIGS      = 5.0f;

    /**
     * The current status string. This is what will likely be
     * displayed to the user by callers and should reflect what
     * the algorithm is currently doing.
     */
    private String                  status;

    /**
     * The number of fragments that the algorithm has processed.
     * This is automatically incremented by calls to
     * completeFragment().
     */
    private int                     numFragsProcessed,

    /**
     * The number of fragments that the algorithm found to be
     * contained within other fragments while in the process of
     * generating arc candidates.
     */
                                    numFragsContained,
    /**
     * The total number of fragments that the algorithm will process.
     * This is set by a call to setNumFragments().
     */
                                    numFragsTotal,

    /**
     * The number of arc candidates that the algorithm has processed.
     * This is automatically incremented by calls to
     * completeArcCandidate().
     */
                                    numArcsProcessed,

    /**
     * The total number of arc candidates that the algorithm will
     * process. This is set by a call to setNumArcCandidates().
     */
                                    numArcsTotal,
    /**
     * The number of contig candidates that the algorithm has
     * processed. This is automatically incremented by calls to
     * completeContigCandidate().
     */
                                    numContigsProcessed;

    /**
     * The overall percentage that the algorithm is complete at any
     * given moment. The aforementioned weight constants control
     * how much each step contributes to this total progress.
     */
    private float                   percentComplete;

    /**
     * Constructor for ProgressManager.
     *
     * Initializes total progress and all counters to zero.
     */
    public ProgressManager()
    {
        numFragsTotal       = 0;
        numArcsTotal        = 0;

        resetProgress();
    }

    /**
     * Modifies the status string that is shown to the user.
     *
     * @param   statusString    The string to display to the user
     *                          describing what is currently
     *                          being done.
     */
    public void setStatusString(String statusString)
    {
        this.status = statusString;
    }

    /**
     * Sets the number of total fragments that the greedy algorithm
     * will be processing. This is used in conjunction with calls to
     * completeFragment() to properly display progress while the
     * greedy algorithm is adding fragments.
     */
    public void setNumFragments(int numFragments)
    {
        numFragsTotal   = numFragments;
    }

    /**
     * Sets the number of total arc candidates that the greedy
     * algorithm will be processing. This is used in conjunction with
     * calls to completeArcCandidate() to properly display progress
     * while the greedy algorithm is processing arc candidates that
     * were generated when each fragment was added.
     */
    public void setNumArcCandidates(int numArcs)
    {
        numArcsTotal    = numArcs;
    }

    public int getNumFragmentsContained()
    {
        return (numFragsContained);
    }

    /**
     * Accessor for the status string that describes what the greedy
     * algorithm is currently doing.
     *
     * @return  A status string describing the execution of the
     *          greedy algorithm.
     */
    public String getStatusString()
    {
        return (status);
    }

    /**
     * Accessor for the overall progress percentage.
     *
     * @return  The progress, from 0 to 100, of the greedy algorithm
     *          at the time the method is invoked.
     */
    public float getPercentComplete()
    {
        return (percentComplete);
    }

    /**
     * Used to update the progress percentage for a processing step.
     *
     * @param   type        The step whose progress to update.
     * @param   percentage  The progress percentage value to use for
     *                      the specified step.
     */
    public void updatePercentage(UpdateType type, float percentage)
    {
        switch (type)
        {
            case UT_INITIAL:
                percentComplete = percentage * (WEIGHT_INITIAL / 100);
                break;

            case UT_FRAGS:
                percentComplete = WEIGHT_INITIAL +
                                  (percentage * (WEIGHT_FRAGMENTS / 100));
                break;

            case UT_ARCS:
                percentComplete = WEIGHT_INITIAL +
                                  WEIGHT_FRAGMENTS +
                                  (percentage * (WEIGHT_ARCS / 100));
                break;

            case UT_CONTIGS:
                percentComplete = WEIGHT_INITIAL +
                                  WEIGHT_FRAGMENTS +
                                  WEIGHT_ARCS +
                                  (percentage * (WEIGHT_CONTIGS / 100));
                break;
        }
    }

    /**
     * Used by the greedy algorithm to signal the completion of a
     * fragment. This automatically increments the counter of
     * processed fragments and updates the status string and percent
     * complee values accordingly.
     */
    public void completeFragment()
    {
        numFragsProcessed++;

        setStatusString(String.format("Processing fragments (%d/%d)...",
                                      numFragsProcessed, numFragsTotal));

        updatePercentage(UpdateType.UT_FRAGS,
                         (numFragsProcessed/(float)numFragsTotal) * 100);
    }

    public void fragmentContained()
    {
        numFragsContained++;
    }

    /**
     * Used by the greedy algorithm to signal the completion of an
     * arc candidate. This automatically increments the counter of
     * processed arc candidates and updates the status string and
     * percent complee values accordingly.
     */
    public void completeArcCandidate()
    {
        numArcsProcessed++;

        setStatusString(String.format("Processing arc candidates (%d/%d)...",
                                      numArcsProcessed, numArcsTotal));

        updatePercentage(UpdateType.UT_ARCS,
                         (numArcsProcessed/(float)numArcsTotal) * 100);
    }

    /**
     * Used by the greedy algorithm to signal the completion of a
     * contig candidate. This automatically increments the counter of
     * processed contig candidates and updates the status string and
     * percent complee values accordingly.
     */
    public void completeContigCandidate()
    {
        numContigsProcessed++;

        setStatusString(String.format("Building contigs (%d/%d)...",
                                      numContigsProcessed, numFragsTotal));

        updatePercentage(UpdateType.UT_CONTIGS,
                         (numContigsProcessed/(float)numFragsTotal) * 100);
    }

    /**
     * Resets all counters to zero without touching the total number
     * of fragments or arc candidates.
     */
    public void resetProgress()
    {
        percentComplete     = 0;
        numFragsProcessed   = 0;
        numFragsContained   = 0;
        numArcsProcessed    = 0;
        numContigsProcessed = 0;
    }
}