/*
 * EventDispatcher.java
 *
 * Version:
 *    $Id: EventDispatcher.java,v 1.3 2005/06/20 01:06:17 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: EventDispatcher.java,v $
 *    Revision 1.3  2005/06/20 01:06:17  gap7472
 *    No changes.
 *
 *    Revision 1.2  2005/06/08 02:51:12  gap7472
 *    Minor changes.
 *
 *    Revision 1.1  2005/05/04 17:36:52  gap7472
 *    Initial revision
 *
 */

package gui;

import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.event.*;

/**
 * This class implements an EventDispatcher which handles dispatching
 * component events to their appropriate methods.
 *
 * @author  Guy Paddock <gap7472@cs.rit.edu>
 */
public class EventDispatcher implements ActionListener, ChangeListener
{
    /**
     * Set to true to enable debug mode.
     */
    private static final boolean        DEBUG_ENABLED   = true;

    /**
     * The object in which the event handler methods are found
     * and on which they will be invoked.
     */
    private Object                      parent;

    /**
     * The map of component to event handler method.
     */
    private HashMap<JComponent, Method> handlers;

    /**
     * Constructor for EventDispatcher.
     *
     * @param   parent  the parent object in which the
     *                  methods that this class will call exist
     */
    public EventDispatcher(Object parent)
    {
        this.parent     = parent;
        this.handlers   = new HashMap<JComponent, Method>();
    }

    public void dispatchEvent(EventObject e)
    {
        // Try to locate an assigned handler
        Method  handler   = this.handlers.get(e.getSource());

        // Don't do anything if there isn't one
        if (handler == null)
            return;

        try
        {
            // Invoke the handler method; all handlers should take some type of
            // EventObject parameter.

            if (e instanceof ActionEvent)
                handler.invoke(parent, (ActionEvent)e);

            else
                handler.invoke(parent, (ChangeEvent)e);
        }

        catch (Exception invokeExcep)
        {
            // Don't really do anything if the invocation fails

            if (DEBUG_ENABLED)
            {
                // Print a stack trace in case it was something in the code
                invokeExcep.printStackTrace();
            }
        }
    }

    /**
     * Method triggered by a registered component that fires actions.
     * The event is dispatched to its registered method, if any.
     *
     * @param   e   the action event for this action
     */
    public void actionPerformed(ActionEvent e)
    {
        dispatchEvent(e);
    }

    /**
     * Method triggered by a registered component that fires change
     * events. The event is dispatched to its registered method, if
     * any.
     *
     * @param   e   the action event for this action
     */
    public void stateChanged(ChangeEvent e)
    {
        dispatchEvent(e);
    }

    /**
     * Method to register an action handler for a specific component.
     *
     * @param   theComponent    the component for which an action handler is being added
     * @param   methodName      the name of the method that will act as the action handler
     */
    public void addActionHandler(JComponent theComponent, String methodName)
    {
        Method  addActionListenerMethod;
        
        try
        {
            addActionListenerMethod = theComponent.getClass().getMethod(
                                            "addActionListener",
                                            Class.forName("java.awt.event.ActionListener"));

            addActionListenerMethod.invoke(theComponent, this);

            handlers.put(theComponent, parent.getClass().getDeclaredMethod(methodName,
                         Class.forName("java.awt.event.ActionEvent")));
        }

        catch (Exception e)
        {
            if (DEBUG_ENABLED)
            {
                // Print a stack trace in case it was something in the code
                e.printStackTrace();
            }
        }
    }

    /**
     * Method to register a state change handler for a specific component.
     *
     * @param   theComponent    the component for which a change handler is being added
     * @param   methodName      the name of the method that will act as the change handler
     */
    public void addChangeHandler(JComponent theComponent, String methodName)
    {
        Method  addChangeListenerMethod;
        
        try
        {
            addChangeListenerMethod = theComponent.getClass().getMethod(
                                            "addChangeListener",
                                            Class.forName("javax.swing.event.ChangeListener"));

            addChangeListenerMethod.invoke(theComponent, this);

            handlers.put(theComponent, parent.getClass().getDeclaredMethod(methodName,
                         Class.forName("javax.swing.event.ChangeEvent")));
        }

        catch (Exception e)
        {
            if (DEBUG_ENABLED)
            {
                // Print a stack trace in case it was something in the code
                e.printStackTrace();
            }
        }
    }
}
