/*
 * GroupBox.java
 *
 * Version:
 *    $Id: GroupBox.java,v 1.1 2005/06/20 01:06:17 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: GroupBox.java,v $
 *    Revision 1.1  2005/06/20 01:06:17  gap7472
 *    Initial revision
 *
 */

package gui;

import javax.swing.*;
import javax.swing.border.*;

/**
 * A descendent of JPanel that emulates a "GroupBox" from the
 * Windows GUI.
 *
 * A GroupBox is a container with an etched inset border that is
 * used to group together similar controls. It usually has a
 * title in the upper-left hand corner to label the group, which
 * provides a visual hint to the user about the function of the
 * controls.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class GroupBox extends JPanel
{
    String  title;
    Border  innerBorder,
            outerBorder;

    /**
     * Constructs a new GroupBox without a title.
     */
    public GroupBox()
    {
        this(null);
    }

    /**
     * Constructs a new GroupBox with the specified title.
     *
     * @see     GroupBox#setText
     * @param   title   The title to display for the new GroupBox.
     */
    public GroupBox(String title)
    {
        super();

        setText(title);
    }

    /**
     * This method sets the title that is displayed in the
     * upper-left-hand corner of the GroupBox.
     *
     * @param   text    The new title for the group box.
     */
    public void setText(String text)
    {
        title   = text;

        setBorders(innerBorder, outerBorder);
    }

    /**
     * This method returns the title of the GroupBox, as set by
     * the constructor or a call to setText().
     *
     * @see     GroupBox#GroupBox(String)
     * @see     GroupBox#setText
     * @return  The title of the GroupBox instance.
     */
    public String getText()
    {
        return (title);
    }

    /**
     * This method is unsupported for the GroupBox. You must call
     * either setInnerBorder(), setOuterBorder(), or setBorders()
     * to set the inner and outer borders of the group box.
     *
     * @see         GroupBox#setInnerBorder
     * @see         GroupBox#setOuterBorder
     * @see         GroupBox#setBorders
     * @exception   UnsupportedOperationException   Thrown if this
     *                                              method is called.
     */
    public void setBorder(Border border)
    throws UnsupportedOperationException
    {
        /*
         * Don't throw the exception if we weren't actually setting a
         * border. Our parent JPanel may call this method on
         * initialization to set a null border.
         */
        if (border != null)
        {
            throw new UnsupportedOperationException(
                "Unsupported method; use setBorders(), " +
                "setInnerBorder(), or setOuterBorder().");
        }
    }

    /**
     * This method sets the border to use on the outside of the
     * inset frame of the GroupBox.
     *
     * @param   outerBorder The border to use on the outside of the
     *                      GroupBox frame.
     *                      
     */
    public void setOuterBorder(Border outerBorder)
    {
        setBorders(outerBorder, this.innerBorder);
    }

    /**
     * This method sets the border to use on the inside of the
     * inset frame of the GroupBox.
     *
     * @param   innerBorder The border to use on the inside of the
     *                      GroupBox frame.
     *                      
     */
    public void setInnerBorder(Border innerBorder)
    {
        setBorders(this.outerBorder, innerBorder);
    }

    /**
     * This method sets both the inner and outer borders at the same
     * time.
     *
     * @see         GroupBox#setInnerBorder
     * @see         GroupBox#setOuterBorder
     * @param   outerBorder The border to use on the outside of the
     *                      GroupBox frame.
     * @param   innerBorder The border to use on the inside of the
     *                      GroupBox frame.
     */
    public void setBorders(Border outerBorder, Border innerBorder)
    {
        // At the very least, a group box has an etched border
        Border  groupBoxBorder  = new EtchedBorder();

        this.innerBorder    = innerBorder;
        this.outerBorder    = outerBorder;

        /*
         * If we have a title, we need a titled border that utilizes
         * our existing group box border
         */
        if (title != null)
            groupBoxBorder = new TitledBorder(groupBoxBorder, title);

        /*
         * The group box border is part of the inside border of the
         * JPanel. Thus, if we have an inner border, it needs to be
         * coupled with the group box border in a compound border
         * so that the two borders are displayed properly.
         */
        if (innerBorder != null)
            innerBorder = new CompoundBorder(groupBoxBorder,
                                             innerBorder);

        // Otherwise, the only inner border we have is the group
        // box border.
        else
            innerBorder = groupBoxBorder;

        /*
         * If we have an outer border, we'll need to couple the inner
         * border and outer border in a compound border and then set
         * that as the border of the panel.
         */
        if (outerBorder != null)
            super.setBorder(new CompoundBorder(outerBorder,
                                               innerBorder));

        /*
         * Otherwise, we just set the border of the panel to the
         * inner border.
         */
        else
            super.setBorder(innerBorder);
    }
}