/*
 * ArrayHeap.java
 *
 * Version:
 *    $Id: ArrayHeap.java,v 1.5 2005/06/20 01:46:03 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ArrayHeap.java,v $
 *    Revision 1.5  2005/06/20 01:46:03  gap7472
 *    -Package changed from GreedyAlg -> util.heap
 *    -Name changed from Heap -> ArrayHeap to reflect type of heap
 *     implementation.
 *    -Converted to being a child of Heap.
 *    -Renamed insert() -> push().
 *    -Renamed remove() -> pop().
 *
 *    Revision 1.4  2005/06/08 03:07:47  gap7472
 *    -Revised/added many javadoc comments for clarity
 *    -Refomatted code lines to be no longer than 70-80 characters, as per
 *     the Sun Coding standard.
 *    -Added comments regarding warning generated at compile time.
 *    -Added special tag to supress compile time warning for compilers
 *     after J2SE 5.0
 *    -Made getParent() -> parentOf(), getLeftChild() -> leftChildOf(),
 *     and getRightChild() -> rightChildOf(), so that names are more
 *     meaningful
 *    -Added new exception for insert, HeapOverflowException, for when
 *     a caller attempts to add too many elements to the heap.
 *
 *    Revision 1.3  2005/04/18 06:19:08  gap7472
 *    Modified for generics. Can now accept anything
 *    that implements the WeightedObject interface.
 *
 *    Revision 1.2  2005/04/06 05:55:37  gap7472
 *    *** empty log message ***
 *
 *    Revision 1.1  2005/04/06 05:54:17  gap7472
 *    Initial revision
 *
 *
 */

package util.heap;

import util.*;

/**
 * An implementation of an array-based Heap, based on code
 * by Rhys Price Jones.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ArrayHeap<T extends WeightedObject> extends Heap<T>
{
    private static final int    TOP_INDEX = 1;
    private T                   theHeap[];
    private int                 numElements;

    /**
     * Constructor for ArrayHeap.
     *
     * The internal array is initialized to hold the specified number of
     * elements.
     *
     * @param   size    The desired number of elements for the Heap to hold
     */
    @SuppressWarnings("unchecked")    // Attempt to suppress an unnecessary
                                      // warning (not implemented in J2SE 5.0;
                                      // see below)
    public ArrayHeap(int size)
    {
        // The size is one more than requested because we waste
        // the first element
        //
        // NOTE: This causes a compiler warning under J2SE 5.0 because there
        //       is currently no workaround that "safely" creates an array
        //       of generic types without relying on the following typecast
	    theHeap     = (T[])new WeightedObject[size + 1];

    	numElements = 0;

        // We waste the first array index so that we can
        // start at 1, simplifying the math involved
        theHeap[0]  = null;
    }

    /**
     * Returns how many items are stored in the heap.
     *
     * @return  The number of items stored in the heap
     */
    public int size()
    {
        return numElements;
    }

    /**
     * This is used to detect if the heap is empty.
     *
     * @return  True if the heap is empty, false otherwise.
     */
    public boolean isEmpty()
    {
        return (numElements == 0);
    }

    /**
     * This is used to detect if the heap is full.
     *
     * @return  True if the heap is at capacity, false otherwise.
     */
    public boolean isFull()
    {
        return (numElements == (theHeap.length - 1));
    }

    /**
     * This is used to examine the item at the top of the heap
     * without removing it.
     *
     * @return  The item at the top of the heap.
     */
    public T peek()
    {
        return (theHeap[TOP_INDEX]);
    }

    /**
     * This inserts a new element at the bottom of the heap, and
     * then calls swapUp() to position it properly.
     *
     * @param       newElement              The element to insert
     *                                      into the heap.
     * @exception   HeapOverflowException   Thrown if the heap is full
     */
    public void push(T newElement)
    throws HeapOverflowException
    {
        if (isFull())
            throw new HeapOverflowException("Heap is full");

        // Add the element to the bottom of the heap
        theHeap[++numElements]  = newElement;

        // Position the element appropriately
        swapUp(newElement, numElements);
    }

    /**
     * This removes the topmost element and then sorts the heap
     * accordingly.
     *
     * @return  the top-most element of the heap, or null if the
     *          heap is empty.
     */
    public T pop()
    {
        T   retVal,
            bottom;

        // Return null if the heap is empty
        if (isEmpty())
            return (null);

        retVal                  = theHeap[TOP_INDEX];
        bottom                  = theHeap[numElements];

        // Move the bottom element to the top
        theHeap[TOP_INDEX]      = bottom;

        // Remove a reference to the old bottom so that it can be
        // garbage collected
        theHeap[numElements--] = null;

        // If we only have one element, we're done
        if (numElements <= 1)
            return retVal;

        // Position the element appropriately
        swapDown(bottom, TOP_INDEX);

        // Return the original top element
        return retVal;
    }

    /**
     * This swaps two items in the heap.
     *
     * @param   firstIndex  The index of the first item
     * @param   secondIndex The index of the second item
     */
    private void swap(int firstIndex, int secondIndex)
    {
        T   temp    = theHeap[firstIndex];

        theHeap[firstIndex]     = theHeap[secondIndex];
        theHeap[secondIndex]    = temp;
    }

    /**
     * This method uses recursion to place an item that starts
     * at the bottom of the heap into its proper position.
     *
     * @param   theElement  The element being positioned in the heap
     * @param   curPos      The current position of the element
     *                      in the heap.
     */
    private void swapUp(T theElement, int curPos)
    {
        int parentPos;

        if (curPos == TOP_INDEX)
            return;
            
        parentPos  = parentOf(curPos);

        // Stop if the element has been placed properly (i.e. the
        // element has a lower weight than its parent)
        if (theElement.getWeight() < theHeap[parentPos].getWeight())
            return;

        // Otherwise, swap the element with its parent
        else
            swap(curPos, parentPos);

        // Check the new position
        swapUp(theElement, parentPos);
    }

    /**
     * This method uses recursion to place an item that starts
     * at the top of the heap into its proper position.
     *
     * @param   theElement  The element being positioned in the heap
     * @param   curPos      The current position of the element
     *                      in the heap.
     */
    private void swapDown(T theElement, int curPos)
    {
        int leftPos     = leftChildOf(curPos),
            rightPos    = rightChildOf(curPos),
            curWeight   = theElement.getWeight(),
            leftWeight,
            rightWeight;

        // Stop if the bottom of the heap has been reached
        if (rightPos > numElements)
            return;

        leftWeight      = theHeap[leftPos].getWeight();
        rightWeight     = theHeap[rightPos].getWeight();

        // Stop sorting if the element has been properly placed
        // (i.e. the element has a higher weight than both of
        //  its two children)
        if ((curWeight > leftWeight) && (curWeight > rightWeight))
            return;

        // Otherwise, more placing needs to be done. If the right
        // child is the larger of the two, swap with it
        else if (rightWeight >= leftWeight)
        {
            swap(curPos, rightPos);
            swapDown(theElement, rightPos); // Check the new position
        }

        // Otherwise, swap with the left child
        else
        {
            swap(curPos, leftPos);
            swapDown(theElement, leftPos);  // Check the new position
        }
    }

    /**
     * Returns the index of the parent for the item at the specified
     * index.
     *
     * @param   curIndex    The index whose parent index will be
     *                      returned
     * @return              The index of the parent for the item at
     *                      curIndex
     */
    private static int parentOf(int curIndex)
    {
        return (curIndex / 2);
    }

    /**
     * Returns the index of the left child for the item at the
     * specified index.
     *
     * @param   curIndex    The index whose left child index will be
     *                      returned
     * @return              The index of the left child for the item
     *                      at curIndex
     */
    private static int leftChildOf(int curIndex)
    {
        return (2 * curIndex);
    }

    /**
     * Returns the index of the right child for the item at the
     * specified index.
     *
     * @param   curIndex    The index whose right child index will be
     *                      returned
     * @return              The index of the right child for the item
     *                      at curIndex
     */
    private static int rightChildOf(int curIndex)
    {
        return ((2 * curIndex) + 1);
    }

    /**
     * This is used to obtain the index of the lowest item in the
     * heap.
     *
     * @return  The index of the lowest item in the heap
     */
    private int getBottomIndex()
    {
        return (numElements);
    }
}

/**
 * Exception thrown when trying to add more elements to the heap
 * than the Heap instance was initialized to hold.
 */
class HeapOverflowException extends RuntimeException
{
    public HeapOverflowException(String msg)
    {
        super(msg);
    }
}