/*
 * LinkedHeap.java
 *
 * Version:
 *    $Id: LinkedHeap.java,v 1.1 2005/06/20 01:46:03 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: LinkedHeap.java,v $
 *    Revision 1.1  2005/06/20 01:46:03  gap7472
 *    Initial revision
 *
 *
 */
package util.heap;

import util.*;

/**
 * A linked implementation of a heap.
 *
 * @author  Guy Paddock     (gap7472@rit.edu)
 */
public class LinkedHeap<T extends WeightedObject> extends Heap<T>
{
    /**
     * The top node is the highest node of the heap. It will be
     * the first off the heap when pop() is called.
     */
    private HeapNode<T> topNode,

    /**
     * The insertion node is the node who will be the parent of any
     * new nodes added to the heap. Once the insertion node has
     * both a left and right child, a new insertion node is chosen.
     */
                        insertionNode,

    /**
     * The bottom node is the lowest node of the heap, and is where
     * newly-added items start out.
     */
                        bottomNode;
    /**
     * The number of nodes in the heap. This is stored here so that
     * the size() method can run in constant time.
     */
    private int         numNodes;

    /**
     * Constructor for LinkedHeap.
     *
     * Initializes an empty heap without any nodes.
     */
    public LinkedHeap()
    {
        topNode         = null;
        insertionNode   = null;
        bottomNode      = null;
        numNodes        = 0;
    }

    /**
     * Returns how many items are stored in the heap.
     *
     * @return  The number of items stored in the heap
     */
    public int size()
    {
        return (numNodes);
    }

    /**
     * This is used to detect if the heap is empty.
     *
     * @return  True if the heap is empty, false otherwise.
     */
    public boolean isEmpty()
    {
        return (numNodes == 0);
    }

    /**
     * This is used to detect if the heap is full.
     *
     * @return  True if the heap is at capacity, false otherwise.
     */
    public boolean isFull()
    {
        // Linked heaps are never full
        return false;
    }

    /**
     * This is used to examine the item at the top of the heap
     * without removing it.
     *
     * @return  The item at the top of the heap.
     */
    public T peek()
    {
        return (topNode.getData());
    }

    /**
     * This inserts a new element at the bottom of the heap, and
     * then calls swapUp() to position it properly.
     *
     * @param   newElement  The element to insert into the heap.
     */
    public void push(T newData)
    {
        HeapNode<T> newNode = new HeapNode<T>(newData,
                                              insertionNode);

        numNodes++;

        // The new node becomes the next link for the
        // old bottom node
        newNode.inheritLinks(null, bottomNode);

        // The new node becomes the bottom of the Heap
        bottomNode  = newNode;

        // Create the top and insertion nodes if this is
        // the first node of the heap
        if (topNode == null)
        {
            topNode         = newNode;
            insertionNode   = newNode;
        }

        // Otherwise, add to existing heap
        else
        {
            // Tell the insertion node to replace its first null
            // child with the new node
            insertionNode.swapChild(null, newNode);

            // If we just filled the right child of the insertion
            // node, we need a new insertion node.
            if (insertionNode.getRightChild() != null)
                insertionNode   = insertionNode.getNextNode();

            // Position the element appropriately
            swapUp(newNode);
        }
    }

    /**
     * This swaps a parent and child node in the heap. The specified
     * child will become the new parent and the specified parent will
     * become the new parent.
     *
     * The specified parent must be the parent node of the specified
     * child before calling this method.
     *
     * @param   parent      The parent node to become the new child
     * @param   child       The child node to become the new parent
     */
    private void swap(HeapNode<T> parent, HeapNode<T> child)
    {
        HeapNode<T> parentParent    = parent.getParent(),
                    parentLeft      = parent.getLeftChild(),
                    parentRight     = parent.getRightChild(),
                    parentNext      = parent.getNextNode(),
                    parentPrev      = parent.getPrevNode(),
                    childLeft       = child.getLeftChild(),
                    childRight      = child.getRightChild(),
                    childNext       = child.getNextNode(),
                    childPrev       = child.getPrevNode();

        /*
         * In the code below, the old child becomes the new parent
         * by inheriting the children and links of the old parent
         * and replacing the old parent as child of the old parent's
         * parent node.
         *
         * Because one of the children of the old parent was the old
         * child, this means that the new parent will have itself as
         * one of its children. This is fixed in the next block (see
         * below).
         */
        child.inheritParent(parentParent, parent);
        child.inheritChildren(parentLeft, parentRight);
        child.inheritLinks(parentNext, parentPrev);

        /*
         * If the next link for the old parent was the old child,
         * the new parent should have the new parent as its next
         * link.
         */
        if (parentNext == child)
            child.setNextNode(parent);

        /**
         * Here, the old parent becomes the new child by inheriting
         * the children and links of the old child. The new child
         * also replaces the old child as a child of itself,
         * thereby fixing the problem of the new child having itself
         * as a child.
         */
        parent.inheritParent(child, child);
        parent.inheritChildren(childLeft, childRight);
        parent.inheritLinks(childNext, childPrev);

        /*
         * If the previous link for the old child was the old parent,
         * the new child should have the new parent as its previous
         * link.
         */
        if (childPrev == parent)
            parent.setPrevNode(child);

        // If the old parent was the top node, the new child becomes
        // the new top node
        if (parentParent == null)
            topNode         = child;

        /* Keep the insertion pointer at the same place in the heap */
        if (insertionNode == child)
            insertionNode = parent;

        else if (insertionNode == parent)
            insertionNode   = child;

        /* Keep the bottom pointer at the same place in the heap */
        if (bottomNode == child)
            bottomNode      = parent;

        else if (bottomNode == parent)
            bottomNode      = child;
    }

    /**
     * This method swaps the bottom node to the top of the heap.
     */
    private void swapBottomToTop()
    {
        HeapNode<T> bottomParent    = bottomNode.getParent(),
                    bottomPrevious  = bottomNode.getPrevNode();
       
        /*
         * The bottom node becomes the new top node and inherits
         * the children and links of the old top node.
         */
        bottomNode.setParent(null);
        bottomNode.inheritChildren(topNode.getLeftChild(),
                                   topNode.getRightChild());
        bottomNode.inheritLinks(topNode.getNextNode(),
                                topNode.getPrevNode());

        // In case the top node was the parent of the bottom node,
        // we want to ensure we don't have ourselves as a child
        bottomNode.swapChild(bottomNode, null);

        // Make the parent of the bottom node lose it as a child
        bottomParent.swapChild(bottomNode, null);

        // Tell the bottom's previous node to drop it as a next link
        bottomPrevious.setNextNode(null);
       
        /*
         * If the insertion node is currently the node after the
         * bottom's parent, then removing the bottom node should
         * make the bottom's parent eligible to be the insertion
         * node.
         */
        if (insertionNode == bottomParent.getNextNode())
            insertionNode   = insertionNode.getPrevNode();

        topNode     = bottomNode;
        bottomNode  = bottomPrevious;

        // Handle the special case where there is only one node left
        if (numNodes == 1)
        {
            bottomNode      = topNode;
            insertionNode   = topNode;
        }
    }

    /**
     * This method uses recursion to place an item that starts
     * at the bottom of the heap into its proper position.
     *
     * @param   node    The node that is to be placed.
     */
    private void swapUp(HeapNode<T> node)
    {
        HeapNode<T> parent  = node.getParent();

        // Stop if we've hit the top of the heap
        if (topNode == node)
            return;

        // Stop if the element has been placed properly (i.e. the
        // element has a lower weight than its parent)
        if (node.getWeight() < parent.getWeight())
            return;

        // Otherwise, swap the element with its parent
        else
            swap(parent, node);

        // Check the new position
        swapUp(node);
    }

    /**
     * This method uses recursion to place an item that starts
     * at the top of the heap into its proper position.
     *
     * @param   node    The node that is to be placed.
     */
    public void swapDown(HeapNode<T> node)
    {
        HeapNode<T>     left        = node.getLeftChild(),
                        right       = node.getRightChild();
        int             curWeight   = node.getWeight(),
                        leftWeight  = (left  != null) ? left.getWeight()  : 0,
                        rightWeight = (right != null) ? right.getWeight() : 0;

        // Stop if we've hit the bottom of the heap
        if (left == null)
            return;

        // Stop sorting if the element has been properly placed
        // (i.e. the element has a higher weight than both of
        //  its two children)
        if ((curWeight > leftWeight) && (curWeight > rightWeight))
            return;

        // Otherwise, more placing needs to be done. Swap with the
        // largest child of the node.
        swap(node, (rightWeight >= leftWeight) ? right : left);

        // Check the new position
        swapDown(node);
    }

    /**
     * Remove the topmost element and sort the heap accordingly.
     *
     * @return  the top-most element of the heap, or null if the
     *          heap is empty.
     */
    public T pop()
    {
        T   retVal;

        // Return null if the heap is empty
        if (isEmpty())
            return (null);

        numNodes--;

        retVal  = topNode.getData();

        if (numNodes > 0)
        {
            // Move the bottom element to the top
            // and position it properly
            swapBottomToTop();
            swapDown(topNode);
        }

        else
        {
            // The last node has been removed. Clear the heap.
            topNode         = null;
            insertionNode   = null;
            bottomNode      = null;
        }

        // Return the data of the original top element
        return retVal;
    }
}

class HeapNode<T extends WeightedObject>
{
    T               data;
    HeapNode<T>     parent,
                    leftChild,
                    rightChild,
                    next,
                    previous;

    public HeapNode()
    {
        this(null, null, null, null, null, null);
    }

    public HeapNode(T data, HeapNode<T> parent)
    {
        this(data, parent, null, null, null, null);
    }

    public HeapNode(T data,                 HeapNode<T> parent,
                    HeapNode<T> leftChild,  HeapNode<T> rightChild,
                    HeapNode<T> next,       HeapNode<T> previous)
    {
        setData(data);
        setParent(parent);
        setLeftChild(leftChild);
        setRightChild(rightChild);
        setNextNode(next);
        setPrevNode(previous);
    }

    public void setData(T data)
    {
        this.data       = data;
    }

    public void setNextNode(HeapNode<T> next)
    {
        this.next       = next;
    }

    public void setPrevNode(HeapNode<T> previous)
    {
        this.previous   = previous;
    }

    public void setParent(HeapNode<T> parent)
    {
        this.parent     = parent;
    }

    public void setLeftChild(HeapNode<T> child)
    {
        this.leftChild  = child;
    }

    public void setRightChild(HeapNode<T> child)
    {
        this.rightChild = child;
    }

    public T getData()
    {
        return (data);
    }

    public HeapNode<T> getNextNode()
    {
        return (next);
    }

    public HeapNode<T> getPrevNode()
    {
        return (previous);
    }

    public int getWeight()
    {
        return (data.getWeight());
    }

    public HeapNode<T> getParent()
    {
        return (parent);
    }

    public HeapNode<T> getLeftChild()
    {
        return (leftChild);
    }

    public HeapNode<T> getRightChild()
    {
        return (rightChild);
    }

    public void swapChild(HeapNode<T> oldChild, HeapNode<T> newChild)
    {
        if (leftChild == oldChild)
            leftChild   = newChild;

        else if (rightChild == oldChild)
            rightChild  = newChild;
    }

    public void inheritParent(HeapNode<T> newParent, HeapNode<T> oldChild)
    {
        setParent(newParent);

        if (newParent != null)
            newParent.swapChild(oldChild, this);
    }

    public void inheritChildren(HeapNode<T> newLeftChild,
                                HeapNode<T> newRightChild)
    {
        setLeftChild(newLeftChild);
        setRightChild(newRightChild);

        if ((newLeftChild != null) && (newLeftChild != this))
            newLeftChild.setParent(this);
        
        if ((newRightChild != null) && (newRightChild != this))
            newRightChild.setParent(this);
    }

    public void inheritLinks(HeapNode<T> newNext,
                             HeapNode<T> newPrev)
    {
        if (newNext != this)
        {
            setNextNode(newNext);

            if (newNext != null)
                newNext.setPrevNode(this);
        }

        if (newPrev != this)
        {
            setPrevNode(newPrev);

            if (newPrev != null)
                newPrev.setNextNode(this);
        }
    }
}