/*
 * Arc.java
 *
 * Version:
 *    $Id: Arc.java,v 1.4 2005/06/20 01:54:31 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: Arc.java,v $
 *    Revision 1.4  2005/06/20 01:54:31  gap7472
 *    -Package changed from SuffixTree -> util.suffixtree.
 *    -Fix added to insert() to prevent an exception on very short data
 *     (apparently, what RPJ said should never happen in the loop
 *      actually does happen; may need more checking).
 *
 *    Revision 1.2  2005/03/16 07:26:19  gap7472
 *    Fixed implementation of the search so that it now pertains
 *    to provided specifications.
 *
 *    Revision 1.1  2005/03/16 05:57:55  gap7472
 *    Initial revision
 *
 *    Revision 1.2  2005/03/09 07:09:28  gap7472
 *    Minor comment changes.
 *
 *    Revision 1.1  2005/03/09 07:01:48  gap7472
 *    Initial revision
 *
 */

package util.suffixtree;

import java.util.*;

/**
 * Arcs store a label and link to either InternalNodes or LeafNodes.
 *
 * @author  Guy Paddock
 */
class Arc extends AbstractArc
{
    /**
     * The label stored in this Arc -- usually a suffix or part of a suffix.
     */
    private String  label;

    /**
     * The "head" of this arc, either an InternalNode or LeafNode.
     */
    private Node    head;

    /**
     * Creates an Arc that contains the specified label and whose head is a
     * new LeafNode that stores the specified suffix position.
     *
     * @param   label   label to store in this arc
     * @param   pos     position where the suffix occurs in the original string
     */
    public Arc(String label, int pos)
    {
        setLabel(label);
        setHead(new LeafNode(pos));
    }

    /**
     * Creates an Arc that contains the specified label and whose head
     * points to the specified node.
     *
     * @param   label   label to store in this Arc
     * @param   head    the node to become the head of this Arc
     */
    public Arc(String label, Node head)
    {
        setLabel(label);
        setHead(head);
    }

    /**
     * Accessor for the label stored in this Arc.
     *
     * @return  the label stored in this Arc
     */
    public String getLabel()
    {
        return (label);
    }

    /**
     * Accessor for the head node of this Arc.
     *
     * @return  head node instance of this Arc.
     */
    public Node getHead()
    {
        return (head);
    }

    /**
     * Sets the head node of this Arc.
     *
     * @param   head    the node to become the head of this Arc
     */
    public void setHead(Node head)
    {
        // Set the head node of this Arc
        this.head   = head;
    }

    /**
     * Sets the label stored in this Arc.
     *
     * @param   label   the label to store in this Arc
     */
    public void setLabel(String label)
    {
        // Set the label on our Arc to what we're given
        this.label  = label;
    }

    /**
     * Inserts a specified suffix value into this Arc. If the suffix occurs
     * in the middle of the Arc, then this Arc will be split up into two arcs.
     *
     * @param   suffix  suffix to insert into this Arc
     * @param   pos     position where the suffix occurs in the original string
     *
     * @return          an Arc with the specified suffix linked in
     */
    public Arc insert(String suffix, int pos)
    {
        int             suffixPos;
        InternalNode    iNodeHead;
        String          newSuffix;
        AbstractArc     subArc;

        // Bugfix on April 18, 2005 -- Handle empty lines being passed in
        //                             as data by doing nothing
        if (suffix.length() == 0)
            return (this);

        // We compare the specified suffix against the letters of this
        // arc's label; we split this arc when we encounter a letter that
        // is different between this arc and the specified suffix.
        for (suffixPos = 0; ((suffixPos < label.length()) &&
                            (suffixPos < suffix.length())); suffixPos++)
        {
            // Look for different letters
            if (label.charAt(suffixPos) != suffix.charAt(suffixPos))
            {
                // Split this arc and return the result
                return (splitArc(suffixPos, suffix, pos));
            }
        }

        // If we reached the end of our label but not the end of our
        // suffix, the rest of the match might be in our child arcs
        if ((suffixPos < (suffix.length())) &&
            (head instanceof InternalNode))
        {
            iNodeHead   = (InternalNode)head;
            newSuffix   = suffix.substring(suffixPos);
            subArc      = iNodeHead.getArc(newSuffix.charAt(0));

            // Have our arc insert the suffix and then replace itself with
            // the resulting arc
            subArc      = subArc.insert(newSuffix, pos);

            // Add the new arc into the internal node, replacing any
            // arc already stored for that character
            iNodeHead.addArc(newSuffix.charAt(0), subArc);
        }
        
        // Return ourselves -- we weren't replaced with a new arc
        return (this);
    }

    /**
     * This method will split this arc into three arcs:
     * -The first will be the "top" arc and will contain  all the
     *  letters that the suffix and our original label have in common.
     * -The second will contain all the letters that the original
     *  label has that the suffix doesn't.
     * -The third one will contain the letters that the suffix has
     *  that the original label doesn't.
     *
     * @param   splitPos    the position at which to split the arc
     *                      label and the suffix
     * @param   suffix      the suffix that is being inserted by
     *                      this split
     * @param   pos         position where the suffix occurs in the
     *                      original string
     */
    public Arc splitArc(int splitPos, String suffix, int pos)
    {
        String          firstPart   = label.substring(0, splitPos),
                        secondPart  = label.substring(splitPos),
                        newPart     = suffix.substring(splitPos);
        Node            origHead    = head;
        InternalNode    newNode;
        Arc             firstArc,
                        secondArc,
                        newArc;

        // Create arcs to hold the parts that differ between our
        // original label and the string we've been given
        secondArc   = new Arc(secondPart, origHead);
        newArc      = new Arc(newPart, pos);

        // Create an internal node that will link to our two new
        // diverging arcs
        newNode     = new InternalNode();

        // Add the two arcs to a new InternalNode
        newNode.addArc(label.charAt(splitPos), secondArc);
        newNode.addArc(suffix.charAt(splitPos), newArc);

        // Create the top Arc
        firstArc    = new Arc(firstPart, newNode);

        // Return the new top arc
        return (firstArc);
    }

    /**
     * Attempts to find and return the location of a
     * specified target string in the suffix tree.
     *
     * @param   target  the target string suffix to find
     * @return          a tree map of matches for the specified
     *                  suffix sorted by position, or null if
     *                  the suffix was not found
     *
     * @throws  ContainmentException  if the specified target has
     *                                multiple possible paths
     */
    public TreeMap<Integer, String> findTarget(String target, String curPath)
    throws ContainmentException
    {
        int                         targetPos,
                                    matchLen;
        String                      newTarget;
        InternalNode                iNodeHead;
        LeafNode                    leafHead;
        TreeMap<Integer, String>    matches;

        //System.out.printf("Target is %s; searching %s\n", target, label);

        matchLen    = curPath.length();

        // Loop through and compare the specified target against
        // the letters of this arc's label
        for (targetPos = 0; ((targetPos < label.length()) &&
                             (targetPos < target.length())); targetPos++)
        {
            //System.out.printf("Comparing %c to %c\n", label.charAt(targetPos), target.charAt(targetPos));

            // Compare letters, return no match if any are different
            if (label.charAt(targetPos) != target.charAt(targetPos))
            {
                if (matchLen == 0 || label.charAt(targetPos) != '$')
                {
                    //System.out.println("Returning null.\n");
                    return (null);
                }
            }

            else
                matchLen++;
        }

        // The suffix is contained if it is shorter than the label
        // on this arc
        if (target.length() < (label.length()))
            throw new ContainmentException();

        // Add to the current path text
        curPath += label;

        // If our head is an InternalNode, pass along the search
        if (head instanceof InternalNode)
        {
            iNodeHead   = (InternalNode)head;
            newTarget   = target.substring(targetPos);

            //System.out.println("Passing on search of \"" + newTarget + "\" to head iNode.\n");
            return (iNodeHead.findTarget(newTarget, curPath));
        }

        // If our head is a LeafNode, we've hit the end of an arc
        // path and matched a suffix, so return the information
        // about the suffix position
        else
        {
            leafHead    = (LeafNode)head;
            matches     = new TreeMap<Integer, String>();

            // Add the LeafNode's position to a 1-element TreeMap
            matches.put(leafHead.getPos(), curPath);

            return (matches);
        }
    }
}