/*
 * InternalNode.java
 *
 * Version:
 *    $Id: InternalNode.java,v 1.4 2005/06/20 01:54:31 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: InternalNode.java,v $
 *    Revision 1.4  2005/06/20 01:54:31  gap7472
 *    -Package changed from SuffixTree -> util.suffixtree.
 *
 *    Revision 1.2  2005/03/16 07:26:19  gap7472
 *    Fixed implementation of the search so that it now pertains
 *    to provided specifications.
 *
 *    Revision 1.1  2005/03/16 05:57:55  gap7472
 *    Initial revision
 *
 *    Revision 1.2  2005/03/09 07:09:28  gap7472
 *    Minor comment changes.
 *
 *    Revision 1.1  2005/03/09 07:01:48  gap7472
 *    Initial revision
 *
 */

package util.suffixtree;

import java.util.*;

/**
 * InternalNodes are used to link together arc paths. Each
 * InternalNode branches out to two or more arcs.
 *
 * @author  Guy Paddock
 */
class InternalNode extends Node
{
    /**
     * The arcs of this InternalNode
     */
    private LinkedHashMap<Character, AbstractArc>   theArcs;

    /**
     * Constructor of a new InternalNode.
     */
    public InternalNode()
    {
        theArcs     = new LinkedHashMap<Character, AbstractArc>();
    }

    /**
     * Retrieves the arc associated with the specified character.
     *
     * @param   c   The letter under which the arc is stored;
     *              usually the first letter of the label
     *              stored in the arc
     * @return      the arc stored under the specified character,
     *              or an empty arc if it cannot be found.
     */
    public AbstractArc getArc(char c)
    {
        AbstractArc retVal  =   theArcs.get(c);

        // If there is nothing for the specified letter,
        // a new, empty arc is created and returned
        return ((retVal != null) ? retVal : new EmptyArc());
    }

    /**
     * Used to access the LinkedHashMap that this class uses to
     * store arcs.
     *
     * @return  a LinkedHashMap of this internal node's arcs
     */
    public LinkedHashMap<Character, AbstractArc> getArcs()
    {
        return (theArcs);
    }

    /**
     * Adds an arc as a branch under this node.
     *
     * @param   c   the letter under which the arc is to be found
     *              in this node; usually the first letter of the
     *              arc's label
     * @param   arc the arc to add
     */
    public void addArc(char c, AbstractArc arc)
    {
        theArcs.put(c, arc);
    }

    /**
     * Attempts to find and return the location of a
     * specified target string in the suffix tree.
     *
     * @param   target  the target string suffix to find
     * @param   curPath string representing the text of the labels on 
     *                  arcs up to the current point
     *
     * @return          a tree map of matches for the specified suffix sorted
     *                  by position, or null if the suffix was not found
     *
     * @throws  ContainmentException  if the specified target has
     *                                multiple possible paths
     */
    public TreeMap<Integer, String> findTarget(String target, String curPath)
    throws ContainmentException
    {
        TreeMap<Integer, String>        retVal      = new TreeMap<Integer, String>(),
                                        locations;

        // Loop through our child arcs, asking each to match the target string
        for (AbstractArc curArc : theArcs.values())
        {
            locations   = curArc.findTarget(target, curPath);

            // Add the findings to our return value
            if (locations != null)
                retVal.putAll(locations);
        }

        // Return no match if we have nothing to return
        return ((retVal.size() != 0) ? retVal : null);
    }
}