/*
 * SuffixTree.java
 *
 * Version:
 *    $Id: SuffixTree.java,v 1.9 2005/06/20 01:54:31 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: SuffixTree.java,v $
 *    Revision 1.9  2005/06/20 01:54:31  gap7472
 *    -Package changed from SuffixTree -> util.suffixtree.
 *
 *    Revision 1.8  2005/06/08 03:19:44  gap7472
 *    -Fixed a bug in findTarget() that was lingering around from
 *     early testing that forced the target of the search to lowercase,
 *     causing the search to fail in uppercase data.
 *
 *    Revision 1.7  2005/03/16 07:26:19  gap7472
 *    Fixed implementation of the search so that it now pertains
 *    to provided specifications.
 *
 *    Revision 1.6  2005/03/16 05:57:55  gap7472
 *    Sync-ed with MultiWordSuffixTree, then added code to locate
 *    multiple matches for a given search phrase.
 *
 *    Revision 1.2  2005/03/09 07:09:28  gap7472
 *    Minor comment changes.
 *
 *    Revision 1.1  2005/03/09 07:01:48  gap7472
 *    Initial revision
 *
 */

package util.suffixtree;

import java.util.*;

/**
 * This class implements a Suffix Tree.
 *
 * @author  Guy Paddock
 */
public class SuffixTree
{
    /**
     * The top-level internal node that links to all of the arcs
     * of this tree.
     */
    private InternalNode    rootNode;
    private String          theWord;

    /**
     * Constructor for SuffixTree.
     *
     * @param   word    The word whose suffixes this tree will
     *                  represent
     */
    public SuffixTree(String word)
    {
        rootNode    = new InternalNode();
        theWord     = word;

        // Grow the tree
        growTree();
    }

    /**
     * Accessor for rootNode.
     *
     * @return  the InternalNode that links to the top-most
     *          arcs of this tree.
     */
    public InternalNode getRootNode()
    {
        return (rootNode);
    }

    /**
     * "Grows" the arcs of the suffix tree.
     */
	public void growTree()
	{
        String      suffix;
        AbstractArc startArc;

        // Add termination character
        theWord = theWord + "$";

        // Loop through the word, inserting progressively
        // shorter suffixes
		for (int i = 0; i < theWord.length(); i++)
		{
			suffix	    = theWord.substring(i);
            startArc    = rootNode.getArc(theWord.charAt(i));

            // Insert will return a new arc with our suffix inserted
            startArc    = startArc.insert(suffix, i);

            // Link the newly-formed arc into the node,
            // replacing any previous arc that was linked
            // to this character
            rootNode.addArc(theWord.charAt(i), startArc);
        }
	}

    /**
     * Attempts to find and return the location of a
     * specified target string in the suffix tree.
     *
     * @param   target  the target string suffix to find
     * @return          a tree map of matches for the specified
     *                  suffix sorted by position, or null if
     *                  the suffix was not found
     *
     * @throws  ContainmentException  if the specified target has
     *                                multiple possible paths
     */
    public TreeMap<Integer, String> findTarget(String target)
    throws ContainmentException
    {
        // Pass it on to the root node
        return (rootNode.findTarget(target, new String()));
    }
}