/*
 * ParamInfo.java
 *
 * Version:
 *    $Id: ParamInfo.java,v 1.1 2005/06/19 21:18:10 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ParamInfo.java,v $
 *    Revision 1.1  2005/06/19 21:18:10  gap7472
 *    Initial revision
 *
 *
 */

/**
 * A class to encapsulate the large number of parameters that are
 * passed from ProfilerGUI to ProfilerCore and SummaryDialog.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ParamInfo
{
    enum InputMode
    {
        IM_GENERATE_DATA,
        IM_SUPPLIED_DATA;
    };

    InputMode               mode;

    /**
     * The length of data to generate.
     */
    int                     dataLength;

    /**
     * The text of a user-supplied file.
     */
    StringBuffer            suppliedData;

    /**
     * Information about the distribution of nucleotide bases
     * in the generated data.
     */
    NucleotideDistInfo      baseInfo;

    /**
     * Information describing the number and size of short, medium,
     * and large repeats.
     */
    ValueInfo[]             repeatInfo;

    /**
     * Information describing the number and size of fragments.
     */
    ValueInfo               fragmentInfo;

    /**
     * Construct for ParamInfo.
     *
     * Creates a new instance of ParamInfo that stores the specified
     * values.
     *
     * @param   dataLength      The length of data to generate
     * @param   baseInfo        Information about the distribution
     *                          of nucleotide bases in the generated
     *                          data
     * @param   repeatInfo      Information describing the number and
     *                          size of short, medium, and large
     *                          repeats
     * @param   fragmentInfo    Information describing the number and
     *                          size of fragments
     */
    public ParamInfo(int                dataLength,
                     NucleotideDistInfo baseInfo,
                     ValueInfo[]        repeatInfo,
                     ValueInfo          fragmentInfo)
    {
        this.dataLength     = dataLength;
        this.baseInfo       = baseInfo;
        this.repeatInfo     = repeatInfo;
        this.fragmentInfo   = fragmentInfo;
        this.mode           = InputMode.IM_GENERATE_DATA;

        this.suppliedData   = null;
    }

    public ParamInfo(StringBuffer   suppliedData,
                     ValueInfo[]    repeatInfo,
                     ValueInfo      fragmentInfo)
    {
        this.suppliedData   = suppliedData;
        this.dataLength     = suppliedData.length();
        this.repeatInfo     = repeatInfo;
        this.fragmentInfo   = fragmentInfo;
        this.mode           = InputMode.IM_SUPPLIED_DATA;

        this.baseInfo       = null;
    }

    public InputMode getMode()
    {
        return (mode);
    }

    public StringBuffer getSuppliedData()
    {
        return (suppliedData);
    }

    /**
     * Accessor for the length of data to generate.
     *
     * @return  The length of data to generate
     */
    public int getDataLength()
    {
        return (dataLength);
    }

    /**
     * Accessor for information about the distribution of nucleotide
     * bases in the generated data.
     *
     * @return  An instance of NucleotideDistInfo that describes
     *          how the nucleotide bases are to be distributed in the
     *          generated data.
     */
    public NucleotideDistInfo getNucleotideInfo()
    {
        return (baseInfo);
    }

    /**
     * Accessor for information describing the number and size of
     * short, medium, and large repeats.
     *
     * @return  An array of three instances of ValueInfo that
     *          describe the number and size of short, medium, and
     *          large repeats respectively.
     */
    public ValueInfo[] getRepeatInfo()
    {
        return (repeatInfo);
    }

    /**
     * Accessor for information describing the number and size of
     * fragments.
     *
     * @return  An instance of ValueInfo that describes the number
     *          and size of fragments.
     */
    public ValueInfo getFragmentInfo()
    {
        return (fragmentInfo);
    }
}

/**
 * Class to describe how nucleotide bases are to be distributed in
 * generated data.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
class NucleotideDistInfo
{
    /**
     * The maximum value of a random number in the range 0-100
     * that will generate the nucletotide base A.
     */
    private int maxA,

    /**
     * The maximum value of a random number in the range 0-100
     * that will generate the nucletotide base C.
     */
                maxC,

    /**
     * The maximum value of a random number in the range 0-100
     * that will generate the nucletotide base G.
     */
                maxG;

    /**
     * Constructor for NucleotideDistInfo.
     *
     * Creates a new instance of NucleotideDistInfo that stores
     * the specified values.
     *
     * @param   maxA    The maximum value of a random number in the
     *                  range 0-100 that will generate the
     *                  nucletotide base A.
     * @param   maxC    The maximum value of a random number in the
     *                  range 0-100 that will generate the
     *                  nucletotide base C.
     * @param   maxA    The maximum value of a random number in the
     *                  range 0-100 that will generate the
     *                  nucletotide base G.
     */
    public NucleotideDistInfo(int maxA, int maxC, int maxG)
    {
        this.maxA   = maxA;
        this.maxC   = maxC;
        this.maxG   = maxG;
    }

    /**
     * Accessor for the maximum value of a random number in the
     * range 0-100 that will generate the nucletotide base A.
     *
     * @return  The maximum value that will produce A.
     */
    public int getMaxA()
    {
        return (maxA);
    }

    /**
     * Accessor for the maximum value of a random number in the
     * range 0-100 that will generate the nucletotide base C.
     *
     * @return  The maximum value that will produce C.
     */
    public int getMaxC()
    {
        return (maxC);
    }

    /**
     * Accessor for the maximum value of a random number in the
     * range 0-100 that will generate the nucletotide base G.
     *
     * @return  The maximum value that will produce G.
     */
    public int getMaxG()
    {
        return (maxG);
    }
}

/**
 * Class to describe the number and min and max size of a particular
 * thing.
 */
class ValueInfo
{
    /**
     * The number of elements of the particular thing to generate.
     */
    private int howMany,

    /**
     * The minimum length of the particular thing to generate.
     */
                minLen,
    /**
     * The maximum length of the particular thing to generate.
     */
                maxLen;

    /**
     * Constructor for ValueInfo.
     *
     * Creates a new instance of ValueInfo that stores
     * the specified values.
     *
     * @param   howMany The number of elements of the particular
     *                  thing to generate.
     * @param   minLen  The minimum length of the particular thing
     *                  to generate.
     * @param   maxLen  The maximum length of the particular thing
     *                  to generate.
     */
    public ValueInfo(int howMany, int minLen, int maxLen)
    {
        this.howMany    = howMany;
        this.minLen     = minLen;
        this.maxLen     = maxLen;
    }

    /**
     * Accessor for the number of elements of the particular thing
     * to generate.
     *
     * @return  How many to generate
     */
    public int getHowMany()
    {
        return (howMany);
    }
    
    /**
     * Accessor for the minimum length of the particular thing to
     * generate.
     *
     * @return  The minimum length to generate
     */
    public int getMinLength()
    {
        return (minLen);
    }
    
     /**
     * Accessor for the maximum length of the particular thing to
     * generate.
     *
     * @return  The maximum length to generate
     */
    public int getMaxLength()
    {
        return (maxLen);
    }
}