/*
 * ProfilerGUI.java
 *
 * Version:
 *    $Id: ProfilerGUI.java,v 1.6 2005/06/20 20:34:36 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ProfilerGUI.java,v $
 *    Revision 1.6  2005/06/20 20:34:36  gap7472
 *    -Data length slider reduced from a maximum of 1,000,000 characters
 *     to 900,000.
 *    -Changed layouts of all tabs from using left and right panels with
 *     BorderLayout to using GridBagLayout.
 *    -Fixed minor GUI bugs.
 *
 *    Revision 1.5  2005/06/20 00:58:07  gap7472
 *    -File renamed from GreedyGUI.java -> ProfilerGUI.java.
 *    -Added code to show and hide the new ProgessDialog.
 *    -Removed code that changes "Run" button into "Stop"; Stop button
 *     moved to ProgressDialog.
 *    -Added code to show summary dialog.
 *    -Divided controls into groups using the new GroupBox control.
 *    -Modified the way the GUI passes parameters into the core by
 *     encapsulating them into new ParamInfo class.
 *    -Added code to display appropriate error messages when problems
 *     occur.
 *    -Changed "Number of Repeats" and "Repeat Size" sliders into three
 *     pairs of sliders, allowing the user to choose the size and number of
 *     three sizes of repeats (short, medium, and large).
 *    -Split controls into multiple tabs to keep window size small.
 *    -Added option for either generated data or text file input as the
 *     initial data source.
 *    -Added "Coverage" calculation to Data and Fragments tabs.
 *    -Added "Reset to Defaults" button to reset all profiler parameters.
 *
 *    Revision 1.4  2005/06/08 02:47:27  gap7472
 *    Changes to RCS comments.
 *
 *    Revision 1.3  2005/06/08 02:38:46  gap7472
 *    -Rewrote javadocs and commenting to be much clearer and more
 *     accurate.
 *    -Refomatted code lines to be no longer than 70-80 characters,
 *     as per the Sun Coding standard.
 *    -Changed the four sliders controlling the amounts of the DNA
 *     nucleotide bases into one multislider with three thumbs.
 *    -Changed the fragment size and repeat size sliders to be
 *     multisliders that set a min and max size instead of taking
 *     an average size.
 *    -Updated the max and default values for all sliders to be more
 *     reasonable.
 *    -Re-organized the addComponents() method so that it is much easier
 *     to read.
 *    -Added code so that the Run button becomes a stop button while the
 *     algorithm is running, allowing the user to stop the algorithm while
 *     it is running.
 *    -Began adding ToolTips to controls.
 *
 *    Revision 1.2  2005/05/09 07:24:13  gap7472
 *    Modified to allow user to choose average fragment size and average
 *    repeat size
 *
 *    Revision 1.1  2005/05/04 17:36:52  gap7472
 *    Initial revision
 *
 */

import gui.*;
import util.*;

import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

/**
 * This class graphically demonstrates and profiles the
 * Greedy algorithm.
 *
 * @author  Guy Paddock
 */
public class ProfilerGUI extends JFrame
{
    /**
     * The "stage director" for this frame -- it ensures that
     * the right methods are called when user-triggered
     * events occur (button clicks, slider adjustments, etc).
     */
    EventDispatcher     eventManager;
    ProfilerProgress    progress;
    StringBuffer        inputFileData;

    ProgressDialog      progressDlg;

    /*
     * The elements that make up this frame.
     */
    JTabbedPane         tabPanel;
    RadioGroupBox       inputTypeRadioBox;
    JPanel              inputPanel;
    JLabel              labelSelectedFile,
                        labelFileLength;
    JTextField          coverageField1,
                        coverageField2;
    MultiSlider         sliderDistribution,
                        sliderFragmentSize,
                        sliderShortRepSize,
                        sliderMedRepSize,
                        sliderLongRepSize;
    JSlider             sliderLenInit,
                        sliderFragments,
                        sliderShortReps,
                        sliderMedReps,
                        sliderLongReps;
    JButton             loadDataBtn,
                        runBtn,
                        defaultsBtn,
                        quitBtn;

    /**
     * Constructor for the GUI.
     */
    public ProfilerGUI()
    {
        // Setup our event manager
        eventManager    = new EventDispatcher(this);

        setTitle("Greedy Algorithm Profiler");
        setPreferredSize(new Dimension(475, 350));
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLayout(new BorderLayout());
        setResizable(false);

        addComponents();
        resetToDefaults();

        // Center window on screen
        setLocationRelativeTo(null);
    }

    /**
     * This adds the components to the form of the JFrame.
     */
    private void addComponents()
    {
        JPanel  mainPanel   = new JPanel(),
                buttonPanel = new JPanel();

        tabPanel    = new JTabbedPane();

        tabPanel.setBorder(new EmptyBorder(5, 5, 5, 5));

        // Tabs
        setupDataTab();
        setupFragmentsTab();
        setupRepeatsTab();

        // Buttons
        runBtn      = new JButton("Run...");
        defaultsBtn = new JButton("Reset to Defaults");
        quitBtn     = new JButton("Quit");

        eventManager.addActionHandler(runBtn,      "runBtnClick");
        eventManager.addActionHandler(defaultsBtn, "defaultsBtnClick");
        eventManager.addActionHandler(quitBtn,     "quitBtnClick");

        buttonPanel.setBorder(new BevelBorder(BevelBorder.RAISED));

        buttonPanel.add(runBtn);
        buttonPanel.add(defaultsBtn);
        buttonPanel.add(quitBtn);

        mainPanel.setLayout(new BorderLayout());
        mainPanel.add(tabPanel, BorderLayout.CENTER);

        this.add(mainPanel,     BorderLayout.CENTER);
        this.add(buttonPanel,   BorderLayout.SOUTH);

        this.pack();
    }

    /**
     * This method creates and adds all of the components that make
     * up the "Data" tab.
     */
    private void setupDataTab()
    {
        JLabel              labelDistribution,
                            labelLenInit,
                            labelCoverage;
        GroupBox            generatedInputBox,
                            fileInputBox,
                            coverageBox;
        JPanel              tab;
        GridBagConstraints  c;

        tab         = new JPanel();
        inputPanel  = new JPanel();

        tab.setLayout(new BorderLayout());

        inputPanel.setLayout(new CardLayout());
        inputPanel.setBorder(new EmptyBorder(0, 5, 0, 5));

        // Data Source (Radio Button)
        inputTypeRadioBox   = new RadioGroupBox("Data Source");

        inputTypeRadioBox.setOuterBorder(new EmptyBorder(0, 5, 0, 5));

        inputTypeRadioBox.addRadioBtn("Generate Random Data", true);
        inputTypeRadioBox.addRadioBtn("Use Text File",        false);

        eventManager.addActionHandler(inputTypeRadioBox,
                                      "inputTypeRadioBoxChanged");

        /* Option 1 - Generate Random Data */
        generatedInputBox   = new GroupBox("Data Generation");
        c                   = new GridBagConstraints();

        generatedInputBox.setInnerBorder(new EmptyBorder(5, 5, 5, 5));
        generatedInputBox.setLayout(new GridBagLayout());

        c.fill              = GridBagConstraints.BOTH;
        c.weightx           = 0.75;
        c.gridx             = 0;
        c.gridy             = 0;
        c.insets            = new Insets(0, 0, 5, 0);

        // DNA Nucleotide Distribution (Slider)
        labelDistribution   = createLabel("DNA Nucleotide Distribution");
        sliderDistribution  = createDistributionSlider();

        generatedInputBox.add(labelDistribution,  c);
        c.gridy++;
        generatedInputBox.add(sliderDistribution, c);

        // Length of Data (Slider)
        labelLenInit        = createLabel("Length of Data");
        sliderLenInit       = createSlider(0,       // Min Value
                                            900000,  // Max Value
                                            30000,   // Min Spacing
                                            300000); // Max Spacing

        c.gridy             = 0;
        c.gridx             = 1;
        c.weightx           = 1;

        generatedInputBox.add(labelLenInit,       c);
        c.gridy++;
        generatedInputBox.add(sliderLenInit,      c);

        eventManager.addChangeHandler(sliderLenInit, "updateCoverage");

        /* Option 2 - Use Text File */
        fileInputBox        = new GroupBox("File Input");
        c                   = new GridBagConstraints();
        
        fileInputBox.setLayout(new GridBagLayout());

        // Selected File (Label)
        labelSelectedFile   = new JLabel("<html><i>(Click the button " +
                                         "below to select a file)</i>" +
                                         "</html>");

        c.gridy             = 0;

        fileInputBox.add(labelSelectedFile, c);

        // File Length (Label)
        labelFileLength     = new JLabel();

        c.anchor            = GridBagConstraints.LAST_LINE_START;
        c.gridy             = 1;
        c.insets            = new Insets(5, 11, 0, 0);

        fileInputBox.add(labelFileLength, c);

        // Load Text File (Button)
        loadDataBtn         = new JButton("Load Text File...");

        c.anchor            = GridBagConstraints.CENTER;
        c.gridy             = 2;
        c.insets            = new Insets(20, 0, 0, 0);

        fileInputBox.add(loadDataBtn, c);

        eventManager.addActionHandler(loadDataBtn, "loadDataBtnClicked");

        // Coverage (Box)
        coverageField1      = new JTextField();
        coverageBox         = createCoverageBox(coverageField1);

        inputPanel.add(generatedInputBox, "0");
        inputPanel.add(fileInputBox,      "1");
        
        tab.add(inputTypeRadioBox, BorderLayout.NORTH);
        tab.add(inputPanel,        BorderLayout.CENTER);
        tab.add(coverageBox,       BorderLayout.SOUTH);

        tabPanel.addTab("Data", tab);
    }

    /**
     * This method creates and adds all of the components that make
     * up the "Fragments" tab.
     */
    private void setupFragmentsTab()
    {
        GroupBox            fragGenBox,
                            coverageBox;
        JPanel              tab;
        JLabel              labelFragments,
                            labelFragmentSize;
        GridBagConstraints  c;

        tab                 = new JPanel();
        fragGenBox          = new GroupBox("Fragment Generation");
        c                   = new GridBagConstraints();

        tab.setLayout(new BorderLayout());

        fragGenBox.setLayout(new GridBagLayout());
        fragGenBox.setOuterBorder(new EmptyBorder(0, 5, 0, 5));
        fragGenBox.setInnerBorder(new EmptyBorder(5, 5, 5, 5));

        // Number of Fragments (Slider)
        labelFragments      = createLabel("Number of Fragments");
        sliderFragments     = createSlider(0,               // Min Value
                                           10000,           // Max Value
                                           250,             // Min Spacing
                                           2500);           // Max Spacing

        c.fill              = GridBagConstraints.BOTH;
        c.weightx           = 0.45;
        c.gridx             = 0;
        c.gridy             = 0;
        c.insets            = new Insets(0, 0, 5, 5);

        fragGenBox.add(labelFragments,  c);
        c.gridy++;
        fragGenBox.add(sliderFragments, c);

        // Fragment Size (Slider)
        labelFragmentSize   = createLabel("Fragment Size");
        sliderFragmentSize  = createMultiSlider(0,          // Min Value
                                                1000,       // Max Value
                                                25,         // Min Spacing
                                                250,        // Max Spacing
                                                2);         // Number of Thumbs

        c.gridy             = 0;
        c.gridx             = 1;
        c.weightx           = 0.55;
        c.insets            = new Insets(0, 5, 5, 0);

        fragGenBox.add(labelFragmentSize,   c);
        c.gridy++;
        fragGenBox.add(sliderFragmentSize,  c);

        eventManager.addChangeHandler(sliderFragments,    "updateCoverage");
        eventManager.addChangeHandler(sliderFragmentSize, "updateCoverage");

        // Coverage (Box)
        coverageField2      = new JTextField();
        coverageBox         = createCoverageBox(coverageField2);

        tab.add(fragGenBox,  BorderLayout.CENTER);
        tab.add(coverageBox, BorderLayout.SOUTH);

        tabPanel.addTab("Fragments", tab);
    }

    /**
     * This method creates and adds all of the components that make
     * up the "Repeats" tab.
     */
    private void setupRepeatsTab()
    {
        GroupBox            repGenBox;
        JPanel              tab;
        JLabel              labelShortReps,
                            labelMedReps,
                            labelLongReps,
                            labelShortRepSize,
                            labelMedRepSize,
                            labelLongRepSize;
        GridBagConstraints  c;

        tab                 = new JPanel();
        repGenBox           = new GroupBox("Repeat Generation");
        c                   = new GridBagConstraints();

        tab.setLayout(new BorderLayout());

        repGenBox.setLayout(new GridBagLayout());
        repGenBox.setOuterBorder(new EmptyBorder(0, 5, 5, 5));
        repGenBox.setInnerBorder(new EmptyBorder(5, 5, 5, 5));

        // Number of Short Repeats (Slider)
        labelShortReps      = createLabel("Number of Short Repeats");
        sliderShortReps     = createSlider(0,           // Min Value
                                           10000,       // Max Value
                                           500,         // Min Spacing
                                           2500);       // Max Spacing

        c.fill              = GridBagConstraints.BOTH;
        c.weightx           = 0.40;
        c.gridx             = 0;
        c.gridy             = 0;
        c.insets            = new Insets(0, 0, 5, 15);

        repGenBox.add(labelShortReps, c);

        c.insets            = new Insets(0, 0, 10, 15);
        c.gridy             = 1;

        repGenBox.add(sliderShortReps, c);

        // Number of Medium Repeats (Slider)
        labelMedReps        = createLabel("Number of Medium Repeats");
        sliderMedReps       = createSlider(0,           // Min Value
                                           1000,        // Max Value
                                           50,          // Min Spacing
                                           250);        // Max Spacing

        c.insets            = new Insets(0, 0, 5, 15);
        c.gridy             = 2;

        repGenBox.add(labelMedReps, c);

        c.insets            = new Insets(0, 0, 10, 15);
        c.gridy             = 3;

        repGenBox.add(sliderMedReps, c);

        // Number of Long Repeats (Slider)
        labelLongReps       = createLabel("Number of Long Repeats");
        sliderLongReps      = createSlider(0,           // Min Value
                                           200,         // Max Value
                                           10,          // Min Spacing
                                           50);         // Max Spacing

        c.insets            = new Insets(0, 0, 5, 15);
        c.gridy             = 4;

        repGenBox.add(labelLongReps, c);

        c.gridy             = 5;

        repGenBox.add(sliderLongReps, c);

        // Short Repeat Size (Slider)
        labelShortRepSize   = createLabel("Short Repeat Size");
        sliderShortRepSize  = createMultiSlider(0,      // Min Value
                                                20,     // Max Value
                                                1,      // Min Spacing
                                                5,      // Max Spacing
                                                2);     // Number of Thumbs

        c.gridy             = 0;
        c.gridx             = 1;
        c.weightx           = 0.60;

        repGenBox.add(labelShortRepSize, c);

        c.insets            = new Insets(0, 15, 10, 0);
        c.gridy             = 1;

        repGenBox.add(sliderShortRepSize, c);

        // Medium Repeat Size (Slider)
        labelMedRepSize     = createLabel("Medium Repeat Size");
        sliderMedRepSize    = createMultiSlider(20,     // Min Value
                                                100,    // Max Value
                                                4,      // Min Spacing
                                                20,     // Max Spacing
                                                2);     // Number of Thumbs

        c.insets            = new Insets(0, 15, 5, 0);
        c.gridy             = 2;

        repGenBox.add(labelMedRepSize, c);

        c.insets            = new Insets(0, 15, 10, 0);
        c.gridy             = 3;

        repGenBox.add(sliderMedRepSize, c);

        // Long Repeat Size (Slider)
        labelLongRepSize    = createLabel("Long Repeat Size");
        sliderLongRepSize   = createMultiSlider(100,    // Min Value
                                                1000,   // Max Value
                                                45,     // Min Spacing
                                                225,    // Max Spacing
                                                2);     // Number of Thumbs

        c.insets            = new Insets(0, 15, 5, 0);
        c.gridy             = 4;

        repGenBox.add(labelLongRepSize, c);

        c.gridy             = 5;

        repGenBox.add(sliderLongRepSize, c);

        tab.add(repGenBox, BorderLayout.CENTER);

        tabPanel.addTab("Repeats", tab);
    }

    /**
     * This simplifies the process of creating multiple centered
     * labels by reducing redundant code.
     *
     * @param   text    The text of the new label
     * @return          A new center-aligned JLabel with the text
     *                  specified
     */
    private static JLabel createLabel(String text)
    {
        JLabel  label   = new JLabel(text, JLabel.CENTER);

        label.setAlignmentX(Component.CENTER_ALIGNMENT);

        return label;
    }

    /**
     * Creates the distribution slider for adjusting what percentages
     * each nucleotide base constitutes of the generated data.
     *
     * @return  The newly-created nucleotide distribution MultiSlider,
     *          with appropriate labeling.
     */
    private static MultiSlider createDistributionSlider()
    {
        MultiSlider                     slider;
        Hashtable<Integer, Component>   labels;

        slider  = createMultiSlider(0,      // Min Value
                                    100,    // Max Value
                                    5,      // Min Spacing
                                    25,     // Max Spacing
                                    3);     // Number of Thumbs
        labels  = new Hashtable<Integer, Component>();

        // Labels for the "regions" of the distribution slider
        labels.put(new Integer(14),     new JLabel("A"));
        labels.put(new Integer(39),     new JLabel("C"));
        labels.put(new Integer(64),     new JLabel("G"));
        labels.put(new Integer(87),     new JLabel("T"));

        slider.setLabelTable(labels);
        slider.setToolTipText("<html><font size='2'><p style='width: 150px'>" +
                              "Each slider adjusts the distribution of two "  +
                              "bases. Drag a slider to the left to reduce "   +
                              "the frequency of the base to its left and "    +
                              "increase the frequency of the base to its "    +
                              "right. Drag it to the right to do the "        +
                              "opposite.</font></html>");

        // Fill the whole track because technically all of it
        // is the active range
        ((MultiSliderUI)slider.getUI()).setFillWholeTrack(true);

        return slider;
    }

    /**
     * This simplifies the process of creating multiple MultiSliders
     * by reducing redundant code.
     *
     * @param   min             The slider's minimum value.
     * @param   max             The slider's maximum value.
     * @param   minorSpacing    The spacing between minor ticks.
     * @param   majorSpacing    The spacing between major ticks.
     * @param   defaultVals     The default values of the slider.
     * @return                  A new MultiSlider with the specified
     *                          properties
     */
    private static MultiSlider createMultiSlider(int min, int max,
                                                 int minorSpacing,
                                                 int majorSpacing,
                                                 int numberOfThumbs)
    {
        MultiSlider newSlider;
        int[]       thumbVals   = new int[numberOfThumbs];

        for (int i = 0; i < numberOfThumbs; i++)
        {
            thumbVals[i]    = min;
        }
        
        newSlider   = new MultiSlider(JSlider.HORIZONTAL,
                                      min, max, thumbVals);

        newSlider.setMinorTickSpacing(minorSpacing);
        newSlider.setMajorTickSpacing(majorSpacing);
        newSlider.setPaintTicks(true);
        newSlider.setPaintLabels(true);
        newSlider.setSnapToTicks(true);
        newSlider.setBounded(true);

        return newSlider;
    }

    /**
     * This simplifies the process of creating multiple JSliders
     * by reducing redundant code.
     *
     * @param   min             The slider's minimum value.
     * @param   max             The slider's maximum value.
     * @param   minorSpacing    The spacing between minor ticks.
     * @param   majorSpacing    The spacing between major ticks.
     * @param   defaultVal      The default value of the slider.
     * @return                  A new JSlider with the specified
     *                          properties
     */
    private static JSlider createSlider(int min, int max,
                                        int minorSpacing,
                                        int majorSpacing)
    {
        JSlider newSlider;
        
        newSlider   = new JSlider(min, max);

        newSlider.setMinorTickSpacing(minorSpacing);
        newSlider.setMajorTickSpacing(majorSpacing);
        newSlider.setPaintTicks(true);
        newSlider.setPaintLabels(true);
        newSlider.setSnapToTicks(true);

        return newSlider;
    }

    /**
     * This method generates a coverage group box, containing the
     * specified field as the coverage field. This method
     * simplifies the creation of multiple coverage boxes for
     * different tabs.
     *
     * @param   coverageField   A pre-created JTextField instance
     *                          that will become a coverage textbox
     * @return                  A new GroupBox instance containing
     *                          the specified field as the coverage
     *                          textbox, with proper labeling.
     * 
     */
    public GroupBox createCoverageBox(JTextField coverageField)
    {
        GroupBox            coverageBox;
        JLabel              labelCoverage;
        GridBagConstraints  c;

        coverageBox     = new GroupBox("Coverage");
        labelCoverage   = new JLabel("Estimated Coverage:");
        c               = new GridBagConstraints();

        // The box is read-only, white in color, and center-aligned.
        coverageField.setEditable(false);
        coverageField.setPreferredSize(new Dimension(60, 20));
        coverageField.setBackground(Color.WHITE);
        coverageField.setHorizontalAlignment(JTextField.CENTER);

        coverageBox.setLayout(new GridBagLayout());
        coverageBox.setInnerBorder(new EmptyBorder(0, 0, 7, 0));
        coverageBox.setOuterBorder(new EmptyBorder(0, 5, 5, 5));

        c.fill          = GridBagConstraints.NONE;
        c.anchor        = GridBagConstraints.CENTER;

        coverageBox.add(labelCoverage, c);

        c.gridx         = 1;
        c.insets        = new Insets(0, 6, 0, 0);

        coverageBox.add(coverageField, c);

        return (coverageBox);
    }

    /**
     * This method is called by the Event Manager when the user
     * adjusts a slider that changes the coverage calculation.
     * It is also called manually at initialization and at times
     * when coverage needs to be recalculated.
     *
     * @param   e   The ChangeEvent the caused this method to be
     *              invoked (unusued).
     */
    public void updateCoverage(ChangeEvent e)
    {
        int     numFragments,
                avgFragSize,
                dataLen;
        float   coverage;
        
        numFragments    = sliderFragments.getValue();
        avgFragSize     = ((sliderFragmentSize.getValueAt(0) + 
                            sliderFragmentSize.getValueAt(1)) / 2);
       
        if (inputTypeRadioBox.getSelectedIndex() == 0)
            dataLen = sliderLenInit.getValue();

        else
            dataLen = inputFileData.length();

        if (dataLen != 0)
        {
            coverage    = (numFragments * avgFragSize) / (float)dataLen;

            coverageField1.setText(String.format("%01.2f", coverage) + "%");
            coverageField2.setText(String.format("%01.2f", coverage) + "%");
        }

        else
        {
            coverageField1.setText(null);
            coverageField2.setText(null);
        }
    }

    /**
     * This method is called by the Event Manager when the selected
     * input source type radio button is changed.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void inputTypeRadioBoxChanged(ActionEvent e)
    {
        CardLayout      cardLayout;
        int             selectedIdx;

        cardLayout  = (CardLayout)inputPanel.getLayout();
        selectedIdx = inputTypeRadioBox.getSelectedIndex();

        cardLayout.show(inputPanel, Integer.toString(selectedIdx));

        // If the user selected "Use Text File", disable the "Run"
        // button and clear the coverage boxes until the user
        // specifies a file.
        if ((selectedIdx == 1) && (inputFileData == null))
        {
            runBtn.setEnabled(false);

            coverageField1.setText(null);
            coverageField2.setText(null);
        }

        else
        {
            runBtn.setEnabled(true);

            updateCoverage(null);
        }
    }

    /**
     * This method is called by the Event Manager when the "Load Text
     * File..." button is clicked.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void loadDataBtnClicked(ActionEvent e)
    {
        File            inputFile;
        JFileChooser    loadDialog  = new JFileChooser();

        loadDialog.setFileFilter(new TextFileFilter());

        // Ask the user for the file to load data from
        if (loadDialog.showOpenDialog(this) == JFileChooser.APPROVE_OPTION)
        {
            inputFile   = loadDialog.getSelectedFile();
        
            try
            {
                inputFileData   = readFile(inputFile);

                labelSelectedFile.setText(
                    "<html>Selected File: <span style='font-weight: normal'>" +
                    inputFile.getName() + "</span></html>");

                labelFileLength.setText(
                    "<html>File Length: <span style='font-weight: normal'>" +
                    inputFileData.length() + "</span></html>");

                runBtn.setEnabled(true);

                updateCoverage(null);
            }

            catch (IOException ioe)
            {
                GUIErrors.errorReadingInput(this);
            }
        }
    }

    /**
     * This method is called by the Event Manager when the "Reset
     * to Defaults" button is clicked.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void defaultsBtnClick(ActionEvent e)
    {
        resetToDefaults();
    }

    /**
     * This method is called by the Event Manager when the "Run"
     * button is clicked.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void runBtnClick(ActionEvent e)
    {
        // Prevent the user from modifying values while the
        // algorithm is running
        disableControls();

        // Show that we're busy
        setCursor(new Cursor(Cursor.WAIT_CURSOR));

        // Create a separate thread in which to run the algorithm
        (new Thread()
        {
            public void run()
            {
                ParamInfo   execParams  = generateParamInfo();

                progress    = new ProfilerProgress();

                displayProgress(true);

                try
                {
                    ProfilerCore.run(execParams, progress);
                }

                catch (OutOfMemoryError e)
                {
                    GUIErrors.errorOutOfMemory(ProfilerGUI.this);
                }

                finally
                {
                    displayProgress(false);
                    displaySummary(execParams);

                    // Re-enable all controls
                    enableControls();

                    // Reset the cursor
                    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                }
            }
        }).start();
    }

    /**
     * This method is called by the Event Manager when the "Quit"
     * button is clicked.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void quitBtnClick(ActionEvent e)
    {
        System.exit(0);
    }
    
    /**
     * This method will read the specified file into a StringBuffer.
     *
     * @param       inputFile   The file that will be read.
     * @return                  A new StringBuffer containing the
     *                          contents of the specified file.
     * @exception   IOException Thrown if a read error occurs.
     */
    private static StringBuffer readFile(File inputFile)
    throws IOException
    {
        StringBuffer    data;
        BufferedReader  reader;
        int             curChar;
        
        data        = new StringBuffer();
        reader      = new BufferedReader(new FileReader(inputFile));

        while ((curChar = reader.read()) != -1)
        {
            data.append((char)curChar);
        }

        return (data);
    }

    /**
     * This method resets all sliders to their default values.
     */
    public void resetToDefaults()
    {
        inputTypeRadioBox.setSelectedIndex(0);
        inputTypeRadioBoxChanged(null);

        sliderDistribution.setValues(new int[] {25, 50, 75});
        sliderLenInit.setValue(95000);

        sliderFragments.setValue(2500);
        sliderFragmentSize.setValues(new int[] {250, 350});

        sliderShortReps.setValue(0);
        sliderShortRepSize.setValues(new int[] {4, 11});

        sliderMedReps.setValue(0);
        sliderMedRepSize.setValues(new int[] {30, 70});

        sliderLongReps.setValue(0);
        sliderLongRepSize.setValues(new int[] {300, 700});
    }

    /**
     * This method pulls all of the parameters the user has
     * specfied out of the JSliders and MultiSliders and into a
     * new instance of ParamInfo, allowing the parameters to be
     * easily passed between classes.
     *
     * @return  A new instance of ParamInfo that contains the user's
     *          desired parameters for executing the greedy
     *          algorithm.
     */
    public ParamInfo generateParamInfo()
    {
        int                 dataLength;
        NucleotideDistInfo  baseInfo;
        ValueInfo[]         repeatInfo      = new ValueInfo[3];
        ValueInfo           fragmentInfo;

        // Short Repeats
        repeatInfo[0]   = new ValueInfo(
                                sliderShortReps.getValue(),
                                sliderShortRepSize.getValueAt(0),
                                sliderShortRepSize.getValueAt(1));

        // Medium Repeats
        repeatInfo[1]   = new ValueInfo(
                                sliderMedReps.getValue(),
                                sliderMedRepSize.getValueAt(0),
                                sliderMedRepSize.getValueAt(1));

        // Long Repeats
        repeatInfo[2]   = new ValueInfo(
                                sliderLongReps.getValue(),
                                sliderLongRepSize.getValueAt(0),
                                sliderLongRepSize.getValueAt(1));

        // Fragments
        fragmentInfo    = new ValueInfo(
                                sliderFragments.getValue(),
                                sliderFragmentSize.getValueAt(0),
                                sliderFragmentSize.getValueAt(1));

        if (inputTypeRadioBox.getSelectedIndex() == 0)
        {            
            // Data Length
            dataLength  = sliderLenInit.getValue();

            // Nucleotide Distribution
            baseInfo    = new NucleotideDistInfo(
                                sliderDistribution.getValueAt(0),
                                sliderDistribution.getValueAt(1),
                                sliderDistribution.getValueAt(2));

            return (new ParamInfo(dataLength, baseInfo,
                                  repeatInfo, fragmentInfo));
        }

        else
        {
            return (new ParamInfo(inputFileData, repeatInfo,
                                  fragmentInfo));
        }
    }

    /**
     * Disables all the controls, including all sliders.
     */
    private void disableControls()
    {
        inputTypeRadioBox.setEnabled(false);

        sliderDistribution.setEnabled(false);
        sliderLenInit.setEnabled(false);
        sliderFragments.setEnabled(false);
        sliderFragmentSize.setEnabled(false);
        sliderShortReps.setEnabled(false);
        sliderShortRepSize.setEnabled(false);
        sliderMedReps.setEnabled(false);
        sliderMedRepSize.setEnabled(false);
        sliderLongReps.setEnabled(false);
        sliderLongRepSize.setEnabled(false);

        loadDataBtn.setEnabled(false);
        runBtn.setEnabled(false);
        defaultsBtn.setEnabled(false);
        quitBtn.setEnabled(false);
    }

    /**
     * Re-enables all the controls after disableControls() has been
     * called.
     */
    private void enableControls()
    {
        inputTypeRadioBox.setEnabled(true);

        sliderDistribution.setEnabled(true);
        sliderLenInit.setEnabled(true);
        sliderFragments.setEnabled(true);
        sliderFragmentSize.setEnabled(true);
        sliderShortReps.setEnabled(true);
        sliderShortRepSize.setEnabled(true);
        sliderMedReps.setEnabled(true);
        sliderMedRepSize.setEnabled(true);
        sliderLongReps.setEnabled(true);
        sliderLongRepSize.setEnabled(true);

        loadDataBtn.setEnabled(true);
        runBtn.setEnabled(true);
        defaultsBtn.setEnabled(true);
        quitBtn.setEnabled(true);
    }

    /**
     * This method either creates an instance of the progress dialog
     * and tells it to show itself or hides and destroys a previous
     * instance of the progress dialog.
     *
     * @param   display True to create and show the progress dialog,
     *                  false to hide and destroy it.
     */
    public void displayProgress(boolean display)
    {
        if (display)
        {
            progressDlg = new ProgressDialog(this, progress);

            progressDlg.showProgress();
        }

        else
        {
            progressDlg.dispose();

            progressDlg = null;
        }
    }

    /**
     * This method will display the summary dialog after the greedy
     * algorithm has finished running.
     */
    public void displaySummary(ParamInfo execParams)
    {
        SummaryDialog   summaryDlg  = new SummaryDialog(this);
        StringBuffer    inputData   = ProfilerCore.getData();
        String          contigs[]   = ProfilerCore.getContigs();
        MemoryWatcher   memWatch    = progress.getMemWatcher();

        summaryDlg.setInputData(inputData.toString());
        summaryDlg.setContigs(contigs);
        summaryDlg.setStats(execParams,
                            inputData.length(),
                            progress.getNumFragmentsContained(),
                            contigs.length,
                            memWatch.getAvgUsage(),
                            memWatch.getMaxUsage(),
                            progress.getTotalTime());

        summaryDlg.setVisible(true);
    }

    /**
     * This method creates and shows the GUI in the event dispatch
     * thread. It is invoked by the call to invokeLater in the
     * main method.
     */
    private static void createAndShowGUI()
    {
        ProfilerGUI mainWindow  = new ProfilerGUI();

        try
        {
            // Attempt to set look and feel
            UIManager.setLookAndFeel(
                UIManager.getCrossPlatformLookAndFeelClassName());

            JFrame.setDefaultLookAndFeelDecorated(true);
        }

        catch (Exception unused)
        {
            // Do nothing if we can't set look and feel
        }

        mainWindow.setVisible(true);
    }

    /**
     * The main method of the program.
     *
     * @param   args    The command-line arguments (unused)
     */
    public static void main(String args[])
    {
        SwingUtilities.invokeLater(new Runnable()
        {
            public void run()
            {
                createAndShowGUI();
            }
        });
    }
}