/*
 * ProfilerProgress.java
 *
 * Version:
 *    $Id: ProfilerProgress.java,v 1.1 2005/06/20 00:59:26 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: ProfilerProgress.java,v $
 *    Revision 1.1  2005/06/20 00:59:26  gap7472
 *    Initial revision
 *
 */

import alg.greedy.*;
import util.*;

import java.util.*;

/**
 * A custom version of ProgressManager that not only monitors the
 * progress of the greedy algorithm, but also monitors initial
 * steps that run before the algorithm. In addition, it also keeps
 * track of runtime and memory used while the algorithm is running.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class ProfilerProgress extends ProgressManager
{
    /**
     * Time constants used to clarify the timeToStr() method.
     */
    private static final int        SECS_PER_MIN    = 60,
                                    SECS_PER_HOUR   = SECS_PER_MIN * 60,
                                    SECS_PER_DAY    = SECS_PER_HOUR * 24;

    /**
     * The number of repeats that the algorithm has generated.
     * This is automatically incremented by calls to
     * repeatGenerated().
     *
     * @see ProfilerProgress#repeatGenerated
     */
    private int                     numRepeatsGenerated,

    /**
     * The total number of repeats that the algorithm will need to
     * generate. This is set by a call to setNumNeededRepeats().
     *
     * @see ProfilerProgress#setNumRepeatsNeeded
     */
                                    numRepeatsTotal;
    /**
     * The starting time for the algorithm, used for calculating
     * runtime. This is set by a call to startMonitors(). Elapsed
     * time since the starting time can be obtained by a call to
     * getTotalTime().
     *
     * @see ProfilerProgress#startMonitors
     * @see ProfilerProgress#getTotalTime
     */
    private Date                    startTime,

    /**
     * The ending time for the algorithm, used for calculating total
     * runtime. This is set by a call to stopMonitors(). The total
     * runtime can be obtained by a call to getTotalTime().
     *
     * @see ProfilerProgress#stopMonitors
     * @see ProfilerProgress#getTotalTime
     */
                                    endTime;

    /**
     * The memory watcher is a special thread that will monitor
     * memory usage and keep track of the min, max, and average
     * usage values. It is started in a call to startMonitors()
     * and stopped in a call to stopMonitors().
     *
     * @see ProfilerProgress#startMonitors
     * @see ProfilerProgress#stopMonitors
     */
    private MemoryWatcher           memWatcher;

    /**
     * This indicates whether or a not a previous call to
     * startMonitors() has been made to start runtime and memory
     * usage monitoring. A call to stopMonitors() will set this
     * to false.
     *
     * @see ProfilerProgress#startMonitors
     * @see ProfilerProgress#stopMonitors
     */
    private boolean                 monitoring;

    /**
     * Constructor for ProfilerProgress.
     *
     * Initializes total progress and all counters to zero.
     */
    public ProfilerProgress()
    {
        // This will automatically call resetProgress()
        super();

        numRepeatsTotal     = 0;
        memWatcher          = new MemoryWatcher();
        monitoring          = false;
    }

    /**
     * Accessor for the memory watcher for callers that need to
     * directly obtain memory usage values.
     *
     * @return  The instance of MemoryWatcher used during monitoring
     *          to keep track of memory usage.
     */
    public MemoryWatcher getMemWatcher()
    {
        return (memWatcher);
    }

    /**
     * Sets the number of total repeats that will need to be
     * generated in the data. This is used in conjunction with calls
     * to repeatGenerated() to properly display progress while 
     * repeats are being generated.
     *
     * @see     ProfilerProgress#repeatGenerated
     * @param   numRepeats  Total number of repeats that will be
     *                      generated
     */
    public void setNumRepeatsNeeded(int numRepeats)
    {
        numRepeatsTotal = numRepeats;
    }

    /**
     * Used to signal the completion of a repeat in the data.
     * This automatically increments the counter of generated repeats
     * and updates the status string and percent complete values
     * accordingly.
     */
    public void repeatGenerated()
    {
        // Increment counter
        numRepeatsGenerated++;

        // Update status string
        setStatusString(String.format("Generating repeats (%d/%d)...",
                                      numRepeatsGenerated,
                                      numRepeatsTotal));

        // Update overall progress
        updatePercentage(UpdateType.UT_INITIAL,
                         (numRepeatsGenerated/(float)numRepeatsTotal) * 100);
    }

    /**
     * This method is used to indicate that all initial steps are
     * complete. This will update the overall progress appropriately
     * to reflect this.
     */
    public void initialStepsComplete()
    {
        updatePercentage(UpdateType.UT_INITIAL, 100);
    }

    /**
     * Resets all counters to zero without touching the total number
     * of repeats to be generated, total number of fragments, or
     * total number of arc candidates.
     *
     * @see alg.greedy.ProgressManager#resetProgress
     */
    public void resetProgress()
    {
        super.resetProgress();

        numRepeatsGenerated = 0;
        monitoring          = false;
        startTime           = null;
        endTime             = null;
    }

    /**
     * This starts monitoring runtime and memory usage. Monitoring
     * can be stopped by a subsequent call to stopMonitors().
     *
     * Once monitoring has started, the elapsed runtime can be
     * obtained by calling getTotalTime(). To obtain the exact
     * starting time of monitoring, call getStartTime().
     *
     * @see ProfilerProgress#stopMonitors
     * @see ProfilerProgress#getTotalTime
     * @see ProfilerProgress#getStartTime
     */
    public void startMonitors()
    {
        startTime       = new Date();
        endTime         = null;
        monitoring      = true;

        memWatcher.start();
    }

    /**
     * This stops monitoring runtime and memory usage, if a previous
     * call to startMonitors() started monitoring.
     *
     * The runtime of the monitoring period can be obtained by
     * calling getTotalTime().
     *
     * @see ProfilerProgress#startMonitors
     * @see ProfilerProgress#getTotalTime
     */
    public void stopMonitors()
    {
        // See if we're even monitoring at all
        if (monitoring)
        {
            memWatcher.stopWatch();

            endTime     = new Date();
            monitoring  = false;
        }
    }

    /**
     * This is used to determine if monitoring of runtime and memory
     * usage is currently in progress.
     *
     * Calling startMonitors() will cause this to be true. Calling
     * stopMonitors() will reset this to false.
     *
     * @see     ProfilerProgress#startMonitors
     * @see     ProfilerProgress#stopMonitors
     * @return  True if monitoring of runtime and memory usage is
     *          active, false otherwise.
     */
    public boolean isMonitoring()
    {
        return (monitoring);
    }

    /**
     * Returns the starting time of monitoring, if a previous call to
     * startMonitors() was made.
     *
     * @see     ProfilerProgress#startMonitors
     * @return  The starting time of monitoring.
     */
    public Date getStartTime()
    {
        return (startTime);
    }

    /**
     * If startMonitors() has been called, but stopMonitors()
     * has not, this will return the elapsed time since monitoring
     * started. Once stopMonitors() has been called, this will 
     * return the total runtime.
     *
     * If startMonitors() has not been called before calling this
     * method, it will return 0 to indicate no runtime.
     *
     * @see     ProfilerProgress#startMonitors
     * @see     ProfilerProgress#stopMonitors
     * @return  The elapsed time since monitoring started, the total
     *          runtime, or 0 if monitoring was never started.
     *          
     */
    public long getTotalTime()
    {
        Date curTime;

        // Return 0 if we were never asked to start monitoring
        // runtime
        if (startTime == null)
            return 0;

        // If we have no end time, return the elapsed runtime
        if (endTime == null)
        {
            curTime = new Date();

            return (curTime.getTime() - startTime.getTime());
        }

        // Otherwise, return total runtime
        else
            return (endTime.getTime() - startTime.getTime());
    }

    /**
     * Converts a specified interval of milliseconds into a string of the
     * following format:
     *
     *      (Days):(Hours):(Minutes):(Seconds)
     *
     * @param   millisecs   The interval of milliseconds to be converted
     * @return              The string resulting from the conversion
     */
    public static String timeToStr(long millisecs)
    {
        int numDays,
            numHours,
            numMins,
            numSecs;

        numSecs     = (int)(millisecs / 1000);

        numDays     = (int)(numSecs / SECS_PER_DAY);
        numSecs    -= numDays * SECS_PER_DAY;

        numHours    = (int)(numSecs / SECS_PER_HOUR);
        numSecs    -= numHours * SECS_PER_HOUR;

        numMins     = (int)(numSecs / SECS_PER_MIN);
        numSecs    -= numMins * SECS_PER_MIN;

        return (String.format("%02d:%02d:%02d:%02d",
                              numDays, numHours, numMins, numSecs));
    }
}