/*
 * SummaryDialog.java
 *
 * Version:
 *    $Id: SummaryDialog.java,v 1.1 2005/06/20 00:59:26 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: SummaryDialog.java,v $
 *    Revision 1.1  2005/06/20 00:59:26  gap7472
 *    Initial revision
 *
 */

import gui.*;

import java.text.*;
import java.io.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.text.*;

/**
 * This dialog box is displayed to the user after the greedy
 * algorithm has either completed or been stopped. It
 * displays the original data that was broken up into fragments,
 * the contig output from the algorithm, and statistical
 * information from the execution of the algorithm.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class SummaryDialog extends JDialog
{
    /**
     * The "stage director" for this dialog -- it ensures that
     * the right methods are called when user-triggered
     * events occur (button clicks, slider adjustments, etc).
     */
    EventDispatcher     eventManager;

    /*
     * The elements that make up this dialog.
     */
    JPanel              buttonPanel;
    JSplitPane          dataPanel;
    GroupBox            inputBox,
                        outputBox,
                        statBox;
    JTabbedPane         outputTabPane;
    JTextArea           inputText,
                        statLeftText,
                        statRightText;
    JScrollPane         inputScroller;
    JButton             saveBtn,
                        copyBtn,
                        closeBtn;
    
    /**
     * Constructor for SummaryDialog.
     *
     * The dialog is created as a modal child of the specified
     * parent.
     *
     * @param   parent  The main window for which this dialog
     *                  will be a child
     */
    public SummaryDialog(JFrame parent)
    {
        super(parent, true);

        // Setup our event manager
        eventManager    = new EventDispatcher(this);
        
        setTitle("Algorithm Execution Summary");
        setPreferredSize(new Dimension(640, 480));
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        setLayout(new BorderLayout());

        addComponents();

        // Center on our parent
        setLocationRelativeTo(parent);
    }

    /**
     * This adds the components to the form of the dialog.
     */
    private void addComponents()
    {
        // LEFT SIDE (Input / Output Data)
        dataPanel       = new JSplitPane(JSplitPane.VERTICAL_SPLIT, true);

        dataPanel.setBorder(new EmptyBorder(5, 5, 0, 5));
        dataPanel.setDividerLocation(175);
        dataPanel.setOneTouchExpandable(true);

        //  - Input Data
        inputBox        = new GroupBox("Input Data");
        inputText       = createTextArea();

        inputBox.setLayout(new BorderLayout());
        inputBox.setMinimumSize(new Dimension(370, 100));

        inputBox.add(setupScrollPane(inputText), BorderLayout.CENTER);

        //  - Algorithm Output
        outputBox       = new GroupBox("Algorithm Output");
        outputTabPane   = new JTabbedPane();

        outputBox.setLayout(new BorderLayout());
        outputBox.setMinimumSize(new Dimension(370, 150));

        outputBox.add(outputTabPane, BorderLayout.CENTER);

        dataPanel.setTopComponent(inputBox);
        dataPanel.setBottomComponent(outputBox);

        // RIGHT SIDE (Statistics)
        statBox         = new GroupBox("Execution Statistics");
        statLeftText    = createMultilineLabel();
        statRightText   = createMultilineLabel();

        statRightText.setPreferredSize(new Dimension(65, 370));

        statBox.setPreferredSize(new Dimension(230, 370));
        statBox.setLayout(new BorderLayout());
        statBox.setBorders(new EmptyBorder(5, 0, 0, 5),
                           new EmptyBorder(0, 4, 4, 4));

        statBox.add(statLeftText,   BorderLayout.CENTER);
        statBox.add(statRightText,  BorderLayout.EAST);

        // BOTTOM (Buttons)
        buttonPanel     = new JPanel();
        saveBtn         = new JButton("Save Results to Text File...");
        copyBtn         = new JButton("Copy Results to Clipboard");
        closeBtn        = new JButton("Close");

        eventManager.addActionHandler(saveBtn,  "saveBtnClick");
        eventManager.addActionHandler(copyBtn,  "copyBtnClick");
        eventManager.addActionHandler(closeBtn, "closeBtnClick");

        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.setBorder(new EmptyBorder(0, 5, 0, 5));

        buttonPanel.add(saveBtn);
        buttonPanel.add(copyBtn);
        buttonPanel.add(closeBtn);

        this.add(dataPanel,         BorderLayout.CENTER);
        this.add(statBox,           BorderLayout.EAST);
        this.add(buttonPanel,       BorderLayout.SOUTH);

        pack();
    }

    /**
     * This method allows for easy creation of a read-only textarea
     * with line wrapping and 10pt font.
     * 
     * @return  A new instance of JTextArea
     */
    private JTextArea createTextArea()
    {
        JTextArea   textArea    = new JTextArea();

        textArea.setEditable(false);
        textArea.setLineWrap(true);
        textArea.setFont(textArea.getFont().deriveFont(10.0f));

        return (textArea);
    }

    /**
     * This method is used to quickly setup a JScrollPane to wrap
     * around a JTextArea. The JScrollPane is initialized to have
     * an inset bevelled border around the text area.
     *
     * @param   textArea    The JTextArea for which a JScrollPane
     *                      is to be made
     * @return              A new instance of JScrollPane, wrapped
     *                      around the JTextArea
     */
    private JScrollPane setupScrollPane(JTextArea textArea)
    {
        JScrollPane scrollPane  = new JScrollPane(textArea);

        /*
         * By default, a JTextArea has text unnaturally close to the
         * border of the control. This adds a 3px padding around the
         * JTextArea to fix that.
         */
        scrollPane.setViewportBorder(new MatteBorder(3, 3, 3, 3,
                                                     Color.WHITE));

        /**
         * This creates a 2px empty border around the outside of the
         * JScrollPane and an inset beveled border around the inside
         * of the JScrollPane. The inner bevel color is modified
         * to keep the color of the bevel from blending with the 
         * color of the text area.
         */
        scrollPane.setBorder(new CompoundBorder(
                                new EmptyBorder(0, 2, 2, 2),
                                new BevelBorder(
                                      BevelBorder.LOWERED,
                                      Color.WHITE,
                                      Color.LIGHT_GRAY,
                                      new Color(116, 116, 116),
                                      new Color(166, 166, 166))));

        return (scrollPane);
    }

    /**
     * This method is used to create a JTextArea that behaves and
     * looks like a JLabel, but supports multiple lines.
     *
     * @return  A new instance of JTextArea that behaves like a
     *          JLabel
     */
    public JTextArea createMultilineLabel()
    {
        JTextArea   newLabel    = new JTextArea();

        newLabel.setFont(newLabel.getFont().deriveFont(11.0f));
        newLabel.setBackground(UIManager.getColor("Label.background"));
        newLabel.setEditable(false);

        return (newLabel);
    }

    /**
     * This method is used to specify the data that the greedy
     * algorithm was to reconstruct from fragments.
     *
     * @param   inputData   The data from which the fragments the
     *                      algorithm processed were generated
     */
    public void setInputData(String inputData)
    {
        inputText.setText(inputData + "\n");
    }

    /**
     * This method is used to specify the contig(s) that the
     * algorithm built out of the fragments it was provided with.
     *
     * @param   contigs The contigs that the algorithm produced
     */
    public void setContigs(String contigs[])
    {
        JTextArea   contigText;
        JScrollPane contigScroll;

        if (contigs.length > 0)
        {
            // For each contig, create a new text box and tab and add
            // the text box to the new tab.
            for (int i = 0; i < contigs.length; i++)
            {
                contigText      = createTextArea();
                contigScroll    = setupScrollPane(contigText);

                contigText.setText(contigs[i] + "\n");

                outputTabPane.addTab("Contig " + (i + 1), contigScroll);
            }
        }

        else
        {
            outputBox.setVisible(false);
            dataPanel.setDividerSize(0);
        }
    }

    /**
     * This method sets the statistical information that is displayed
     * in the right box of the dialog.
     *
     * @param   params          The initial parameters that the user provided
     * @param   initLen         The initial length of the data
     * @param   finalLen        The length of the data with repeats
     * @param   numContained    The number of fragments that were found to be
     *                          contained within other fragments
     * @param   numContigs      The number of contigs that the algorithm
     *                          generated
     * @param   avgMem          The average heap memory usage
     * @param   maxMem          The maximum heap memory usage
     * @param   runTime         The total runtime of the algorithm, in
     *                          milliseconds
     */
    public void setStats(ParamInfo  params,
                         int        finalLen,
                         int        numContained,
                         int        numContigs,
                         long       avgMem,
                         long       maxMem,
                         long       runTime)
    {
        int             initLen     = params.getDataLength();
        ValueInfo[]     repInfo     = params.getRepeatInfo();
        ValueInfo       fragInfo    = params.getFragmentInfo();
        NumberFormat    f           = NumberFormat.getInstance();
        String[]        repeatTypes = {"Short", "Medium", "Long"};
        String          curType;

        // Format all numbers so that digits are grouped for readability
        if (f instanceof DecimalFormat)
            ((DecimalFormat)f).setGroupingSize(3);

        statLeftText.append("Initial Data Length:\n");
        statRightText.append(f.format(initLen) + "\n");

        if (finalLen > initLen)
        {
            statLeftText.append("Data Length with Repeats:\n");
            statRightText.append(f.format(finalLen) + "\n");
        }

        statLeftText.append("\n");
        statRightText.append("\n");

        statLeftText.append("Number of Fragments:\n");
        statRightText.append(f.format(fragInfo.getHowMany()) + "\n");

        statLeftText.append("Minimum Fragment Length:\n");
        statRightText.append(f.format(fragInfo.getMinLength()) + "\n");

        statLeftText.append("Maximum Fragment Length:\n");
        statRightText.append(f.format(fragInfo.getMaxLength()) + "\n");

        statLeftText.append("\n");
        statRightText.append("\n");

        for (int i = 0; i < 3; i++)
        {
            if (repInfo[i].getHowMany() == 0)
                continue;

            curType = repeatTypes[i];

            statLeftText.append("Number of " + curType + " Repeats:\n");
            statRightText.append(f.format(repInfo[i].getHowMany()) + "\n");

            statLeftText.append("Min. " + curType + " Repeat Length:\n");
            statRightText.append(f.format(repInfo[i].getMinLength()) + "\n");

            statLeftText.append("Max. " + curType + " Repeat Length:\n");
            statRightText.append(f.format(repInfo[i].getMaxLength()) + "\n");

            statLeftText.append("\n");
            statRightText.append("\n");
        }

        statLeftText.append("Fragments Contained:\n");
        statRightText.append(f.format(numContained) + "\n");

        statLeftText.append("Contigs Generated:\n");
        statRightText.append(f.format(numContigs) + "\n");

        statLeftText.append("\n");
        statRightText.append("\n");

        statLeftText.append("Average Memory Usage:\n");
        statRightText.append(f.format(avgMem / 1024) + " KB\n");

        statLeftText.append("Maximum Memory Usage:\n");
        statRightText.append(f.format(maxMem / 1024) + " KB\n");

        statLeftText.append("\n");
        statRightText.append("\n");

        statLeftText.append("Total Runtime:\n");
        statRightText.append(ProfilerProgress.timeToStr(runTime) + "\n");
    }

    /**
     * This method returns the statistics text as an array of
     * Strings, with each element representing one line of the text.
     *
     * @return  A string array of the lines of the statistics text
     */
    public String[] getStats()
    {
        String[]    leftLines     = textAreaToArray(statLeftText),
                    rightLines    = textAreaToArray(statRightText),
                    retLines      = new String[leftLines.length];

        for (int i = 0; i < retLines.length; i++)
        {
            retLines[i] = leftLines[i] + " " + rightLines[i];
        }

        return (retLines);
    }

    /**
     * This method is called by the EventManager when the user clicks
     * the "Save Results to Text File" button in the dialog box.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void saveBtnClick(ActionEvent e)
    {
        JFileChooser    saveDialog  = new JFileChooser();

        saveDialog.setFileFilter(new TextFileFilter());

        // Ask the user where to save the results
        if (saveDialog.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                ProfilerCore.saveResults(saveDialog.getSelectedFile(),
                                         getStats());
            }

            // Catch errors while trying to save
            catch (IOException ioe)
            {
                // Inform the user of the problem
                GUIErrors.errorSavingResults(this);
            }
        }
    }

    /**
     * This method is called by the EventManager when the user clicks
     * the "Copy Results to Clipboard" button in the dialog box.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void copyBtnClick(ActionEvent e)
    {
        ProfilerCore.copyResults(getStats());
    }

    /**
     * This method is called by the EventManager when the user clicks
     * the "Close" button in the dialog box.
     *
     * @param   e   The action event that caused this method to be
     *              invoked.
     */
    public void closeBtnClick(ActionEvent e)
    {
        this.dispose();
    }

    /**
     * This method will convert the lines of the specified JTextArea
     * into an array of Strings.
     *
     * @param   textArea    The JTextArea whose lines to return in an
     *                      array of strings
     * @return              An array of strings representing the
     *                      lines of the specified JTextArea.
     */
    private static String[] textAreaToArray(JTextArea textArea)
    {
        int             rangeStart,
                        rangeEnd,
                        curLine;
        String[]        lines;

        curLine         = 0;
        lines           = new String[textArea.getLineCount()];

        for (int i = 0; i < lines.length; i++)
        {
            try
            {
                rangeStart  = textArea.getLineStartOffset(i);
                rangeEnd    = textArea.getLineEndOffset(i);
                lines[i]    = textArea.getText(rangeStart,
                                               rangeEnd - rangeStart);

                // Strip newlines
                lines[i]    = lines[i].replaceAll("\n", "");
            }

            catch (BadLocationException ex)
            {
                // SHOULD NEVER HAPPEN
            }
        }

        return (lines);
    }
}