/*
 * GraphNode.java
 *
 * Version:
 *    $Id: GraphNode.java,v 1.3 2005/06/20 01:32:02 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: GraphNode.java,v $
 *    Revision 1.3  2005/06/20 01:32:02  gap7472
 *    Minor comment changes.
 *
 *    Revision 1.2  2005/06/08 02:52:50  gap7472
 *    Minor changes.
 *
 *    Revision 1.1  2005/04/06 05:54:17  gap7472
 *    Initial revision
 *
 */

package alg.greedy;

import util.suffixtree.*;

import java.util.*;

/**
 * This class represents one node or fragment that is to be connected
 * to other nodes via the greedy algorithm.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class GraphNode
{
    /**
     * The label of this node.
     */
    private String              label;

    /**
     * The associate suffix tree for the label
     * on this node.
     */
    private SuffixTree          labelTree;

    /**
     * The graph arcs that come in and go out
     * of this node.
     */
    private GraphArc            inArc,
                                outArc;

    /**
     * Used to reduce the amount of overhead involved
     * in determining if a node that has arcs linking
     * to it has been disqualified after the arcs
     * were created.
     */
    private boolean             disqualified;

    /**
     * Constructor for GraphNode.
     *
     * @param   the label of this node
     */
    public GraphNode(String label)
    {
        this.label          = label;
        this.labelTree      = new SuffixTree(label);

        this.inArc          = null;
        this.outArc         = null;

        this.disqualified   = false;
    }

    /**
     * Accessor for label.
     *
     * @return  the label of this node.
     */
    public String getLabel()
    {
        return (label);
    }

    /**
     * Modifier for label.
     *
     * @param   label   the new label for this node
     */
    public void setLabel(String label)
    {
        this.label = label;
    }

    /**
     * Accessor for inArc.
     *
     * @return  the graph arc pointing to this node, or null
     *          if this node does not currently have one
     */
    public GraphArc getIn()
    {
        return (inArc);
    }

    /**
     * Modifier for inArc.
     *
     * @param   inArc   the new arc that points to this node
     */
    public void setIn(GraphArc inArc)
    {
        this.inArc  = inArc;
    }

    /**
     * Accessor for outArc.
     *
     * @return  the graph arc pointing out from this node,
     *          or null if this node does not currently have
     *          one
     */
    public GraphArc getOut()
    {
        return (outArc);
    }

    /**
     * Modifier for outArc.
     *
     * @param   outArc   the new arc that points out of this node
     */
    public void setOut(GraphArc outArc)
    {
        this.outArc = outArc;
    }

    /**
     * Accessor for disqualified.
     *
     * @return  the value of disqualified
     */
    public boolean isDisqualified()
    {
        return (disqualified);
    }

    /**
     * Modifier for disqualified.
     *
     * @param   disqualified   the new disqualified status;
     *                         true if this node has been
     *                         disqualified, false otherwise.
     */
    public void setDisqualified(boolean disqualified)
    {
        this.disqualified   = disqualified;
    }

    /**
     * Method to find the longest matching prefix that is a
     * suffix of this node's label (i.e. the overlap between
     * the end of this node's label and the beginning of another's)
     *
     * @param       otherLabel              the label with which to
     *                                      attempt to find overlap
     * @return                              information about the
     *                                      overlap between the
     *                                      labels
     *
     * @exception   ContainmentException    if the specified label is contained
     *                                      within this node's label.
     */
    public OverlapInfo findOverlap(String otherLabel) throws ContainmentException
    {
        TreeMap<Integer, String>    resultMap   = labelTree.findTarget(otherLabel);
        int                         overlapPos;
        String                      overlapStr;

        if (resultMap == null)
            return (null);

        // The first element in the map should be the longest overlap;
        // the tree map that the results come from is sorted ascendingly
        // by the position of each match in the original string
        overlapPos  = resultMap.firstKey();
        overlapStr  = resultMap.get(overlapPos);

        // Trim off the dollar-sign from the end of the overlap
        overlapStr  = overlapStr.substring(0, overlapStr.length() - 1);

        return new OverlapInfo(overlapStr, overlapPos);
    }
}