/* 
 * MultiSliderUI.java
 *
 * Version:
 *    $Id: MultiSliderUI.java,v 1.2 2005/06/20 01:26:30 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: MultiSliderUI.java,v $
 *    Revision 1.2  2005/06/20 01:26:30  gap7472
 *    -Fixed bug in paint() that did not properly set thumbRect when
 *     component was first initialized, causing the track to be drawn
 *     oddly when the MultiSlider was used in a JTabbedPane.
 *
 *    Revision 1.1  2005/06/08 03:18:39  gap7472
 *    Initial revision
 */

/*
 * Licensed under the GNU Lesser General Public License from:
 * -------------------------------------------------------------------
 * GeoVISTA Center (Penn State, Dept. of Geography)
 *
 * Java source file for the class MultiSliderUI.
 *
 * Copyright (c), 2002, Masahiro Takatsuka & GeoVISTA Center
 * All Rights Reserved.
 *
 * This is a copy from jh9gpz.ui.slider.MtMultiSliderUI.java
 * originally written by Masahiro Takatsuka.
 * -------------------------------------------------------------------
 */

/*
 * Modifications from GeoVISTA's Original
 * --------------------------------------
 * -Package changed to gui
 * -Reformatted code to match style of other files in this project
 * -Modified paintTrack() using Sun's code from MetalSliderUI so that
 *  the track is more logically painted to reflect the range between
 *  sliders
 * -Added fillWholeTrack and respective mutex, and modified
 *  paintTrack() to allow the slider to be painted as if the entire
 *  track is the active "range". This is really only incredibly
 *  useful for multiple thumbs where each thumb does not represent a
 *  value but instead the division betweeen ranges. Thus, dragging
 *  the slider one way increases the range on one side of the slider
 *  and decreases the range on the other and vice-versa.
 * -Fixed bug in paint() that did not properly set thumbRect when
 *  component was first initialized, causing the track to be drawn
 *  oddly when the MultiSlider was used in a JTabbedPane.
 */

package gui;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.basic.BasicSliderUI.*;
import javax.swing.plaf.metal.*;

/**
 * A Basic L&F implementation of SliderUI.
 *
 * @version $Revision: 1.2 $
 * @see javax.swing.plaf.metal.MetalSliderUI
 * @author Masahiro Takatsuka   (masa@psu.edu)
 * @author Guy Paddock          (gap7472@rit.edu)
 */
public class MultiSliderUI extends MetalSliderUI
{
    transient private int       currentIndex    = 0;
    transient private boolean   isDragging;
    transient private int[]     minmaxIndices   = new int[2];

    private Rectangle[]         thumbRects      = null;
    private int                 thumbCount;
    private boolean             fillWholeTrack;

    /**
     * ComponentUI Interface Implementation methods
     */
    public static ComponentUI createUI(JComponent b)
    {
        return new MultiSliderUI();
    }

    /**
     * Construct a new MultiSliderUI object.
     */
    public MultiSliderUI()
    {
        super();

        fillWholeTrack  = false;
    }

    int getTrackBuffer()
    {
        return this.trackBuffer;
    }

    /**
     * Sets the number of Thumbs.
     */
    public void setThumbCount(int count)
    {
        this.thumbCount = count;
    }

    /**
     * Turns on or off filling the entire slider track.
     * This is usually only useful if the slider has
     * more than 2 thumbs and it doesn't make sense
     * to fill only the region between two thumbs.
     *
     * @param   doFill  The new value for fillWholeTrack --
     *                  true if the whole track should be
     *                  filled, false if only the track
     *                  between thumbs should be filled.
     */
    public void setFillWholeTrack(boolean doFill)
    {
        fillWholeTrack  = doFill;
    }

    /**
     * Returns the index number of the thumb currently operated.
     */
    protected int getCurrentIndex()
    {
        return this.currentIndex;
    }

    public void installUI(JComponent c)
    {
        this.thumbRects     = new Rectangle[this.thumbCount];

        for (int i = 0; i < this.thumbCount; i++)
        {
            this.thumbRects[i]  = new Rectangle();
        }

        this.currentIndex   = 0;

        if (this.thumbCount > 0)
            thumbRect           = this.thumbRects[this.currentIndex];

        super.installUI(c);
    }

    public void uninstallUI(JComponent c)
    {
        super.uninstallUI(c);

        for (int i = 0; i < this.thumbCount; i++)
        {
            this.thumbRects[i]  = null;
        }

        this.thumbRects = null;
    }

    protected void installListeners(JSlider slider)
    {
        slider.addMouseListener(trackListener);
        slider.addMouseMotionListener(trackListener);
        slider.addFocusListener(focusListener);
        slider.addComponentListener(componentListener);
        slider.addPropertyChangeListener(propertyChangeListener);

        for (int i = 0; i < this.thumbCount; i++)
        {
            ((MultiSlider)slider).getModelAt(i).addChangeListener(changeListener);
        }
    }

    protected void uninstallListeners(JSlider slider)
    {
        slider.removeMouseListener(trackListener);
        slider.removeMouseMotionListener(trackListener);
        slider.removeFocusListener(focusListener);
        slider.removeComponentListener(componentListener);
        slider.removePropertyChangeListener(propertyChangeListener);

        for (int i = 0; i < this.thumbCount; i++)
        {
            BoundedRangeModel   model   = ((MultiSlider)slider).getModelAt(i);

            if (model != null)
            {
                model.removeChangeListener(changeListener);
            }
        }
    }

    protected void calculateThumbSize()
    {
        Dimension   size    = getThumbSize();

        for (int i = 0; i < this.thumbCount; i++)
        {
            this.thumbRects[i].setSize(size.width, size.height);
        }

        thumbRect.setSize(size.width, size.height);
    }

    protected void calculateThumbLocation()
    {
        MultiSlider slider              = (MultiSlider) this.slider;
        int         majorTickSpacing    = slider.getMajorTickSpacing(),
                    minorTickSpacing    = slider.getMinorTickSpacing(),
                    tickSpacing         = 0;

        if (minorTickSpacing > 0)
            tickSpacing = minorTickSpacing;
        
        else if (majorTickSpacing > 0)
            tickSpacing = majorTickSpacing;

        for (int i = 0; i < this.thumbCount; i++)
        {
//            if (slider.isBounded())
//            {
//                int neighbours[] = new int[2];
//
//                neighbours[0]   = ((i - 1) < 0) ? -1 : (i - 1);
//                neighbours[1]   = ((i + 1) >= this.thumbCount) ? -1 : (i + 1);
//
//                setThumbBounds(neighbours);
//            }
                  
            if (slider.getSnapToTicks())
            {
                int sliderValue     = slider.getValueAt(i),
                    snappedValue    = sliderValue;

                if (tickSpacing != 0)
                {
                    // If it's not on a tick, change the value
                    if (((sliderValue - slider.getMinimum()) % tickSpacing) != 0)
                    {
                        float   temp        = (float)(sliderValue - slider.getMinimum()) / (float)tickSpacing;
                        int     whichTick   = Math.round(temp);

                        snappedValue        = slider.getMinimum() + (whichTick * tickSpacing);
                    }

                    if (snappedValue != sliderValue)
                        slider.setValueAt(i, snappedValue);
                }
            }

            if (slider.getOrientation() == JSlider.HORIZONTAL)
            {
                int valuePosition       = xPositionForValue(slider.getValueAt(i));

                this.thumbRects[i].x    = valuePosition - (this.thumbRects[i].width / 2);
                this.thumbRects[i].y    = trackRect.y;
            }
            
            else
            {
                int valuePosition       = yPositionForValue(slider.getValueAt(i));

                this.thumbRects[i].x    = trackRect.x;
                this.thumbRects[i].y    = valuePosition - (this.thumbRects[i].height / 2);
            }
        }
    }

    public void paint(Graphics g, JComponent c)
    {
        Rectangle   clip;

        recalculateIfInsetsChanged();
        recalculateIfOrientationChanged();
        
        clip        = g.getClipBounds();

        // FIX ON 06/19/2005: Track was not drawn properly if the MultiSlider
        //                    was used in a JTabbedPane
        thumbRect   = this.thumbRects[this.currentIndex];

        if (slider.getPaintTrack() && clip.intersects(trackRect))
            paintTrack(g);

        if (slider.getPaintTicks() && clip.intersects(tickRect))
            paintTicks(g);

        if (slider.getPaintLabels() && clip.intersects(labelRect))
            paintLabels(g);

        if (slider.hasFocus() && clip.intersects(focusRect))
            paintFocus(g);

        // first paint unfocused thumbs.
        for (int i = 0; i < this.thumbCount; i++)
        {
            if (i != this.currentIndex)
            {
                if (clip.intersects(this.thumbRects[i]))
                {
                    thumbRect   = this.thumbRects[i];
                    paintThumb(g);
                }
            }
        }

        // then paint currently focused thumb.
        if (clip.intersects(this.thumbRects[this.currentIndex]))
        {
            thumbRect   = this.thumbRects[this.currentIndex];
            paintThumb(g);
        }
    }

    public void paintThumb(Graphics g)
    {
        super.paintThumb(g);
    }

    // Copied from Sun's Revision 1.34 of javax.swing.plaf.metal.MetalSliderUI
    private Color chooseColor(boolean chooseFirst, Color c1, Color c2)
    {
        if (chooseFirst)
            return c2;
        
        else
            return c1;
    }

    // Copied from Sun's Revision 1.34 of javax.swing.plaf.metal.MetalSliderUI
    /**
     * Returns a rectangle enclosing the track that will be painted.
     */
    private Rectangle getPaintTrackRect()
    {
        int         trackLeft   = 0,
                    trackRight  = 0,
                    trackTop    = 0,
                    trackBottom = 0;
        boolean     leftToRight = slider.getComponentOrientation().isLeftToRight();

        if (slider.getOrientation() == JSlider.HORIZONTAL)
        {
            trackBottom = (trackRect.height - 1) - getThumbOverhang();
            trackTop    = trackBottom - (getTrackWidth() - 1);
            trackRight  = trackRect.width - 1;
        }

        else
        {
	    
            if (leftToRight)
            {
	            trackLeft   = (trackRect.width - getThumbOverhang()) - 
		                                         getTrackWidth();
		        trackRight  = (trackRect.width - getThumbOverhang()) - 1;
            }
	    
            else
            {
	            trackLeft   = getThumbOverhang();
		        trackRight  = getThumbOverhang() + getTrackWidth() - 1;
	        }

            trackBottom = trackRect.height - 1;
        }

        return new Rectangle(trackRect.x + trackLeft, trackRect.y + trackTop,
                             trackRight - trackLeft, trackBottom - trackTop);
    }

    // Modified from Sun's Revision 1.34 of javax.swing.plaf.metal.MetalSliderUI
    public void paintTrack(Graphics g)
    {
        int         w,
                    h,
                    nextThumb           = (this.currentIndex == (this.thumbCount - 1)) ?
                                          this.currentIndex - 1                        :
                                          this.currentIndex + 1;
	    boolean     leftToRight         = slider.getComponentOrientation().isLeftToRight(),
                    drawInverted        = drawInverted();
        Color       sliderAltTrackColor = UIManager.getColor("Slider.altTrackColor");
        Rectangle   paintRect;

        // Translate to the origin of the painting rectangle
        paintRect   = getPaintTrackRect();

        g.translate(paintRect.x, paintRect.y);

        // Width and height of the painting rectangle.
        w           = paintRect.width;
        h           = paintRect.height;

        if (!slider.isEnabled())
        {
            g.setColor(MetalLookAndFeel.getControlShadow());
            g.drawRect(0, 0, w - 1, h - 1);
        }
        
        else if (slider.getOrientation() == JSlider.HORIZONTAL)
        {
            int fillMinX,
                fillMaxX,
                middleOfNextThumb   = thumbRects[nextThumb].x +
                                      (thumbRects[nextThumb].width / 2) -
                                      paintRect.x,
                middleOfActiveThumb = thumbRect.x + (thumbRect.width / 2) -
                                      paintRect.x;

            if (middleOfActiveThumb > 0)
            {
                g.setColor(chooseColor(drawInverted,
                                       MetalLookAndFeel.getPrimaryControlDarkShadow(),
                                       MetalLookAndFeel.getControlDarkShadow()));
                g.drawRect(0, 0, middleOfActiveThumb - 1, h - 1);
            }

            if (middleOfActiveThumb < w)
            {
                g.setColor(chooseColor(drawInverted,
                                       MetalLookAndFeel.getControlDarkShadow(),
                                       MetalLookAndFeel.getPrimaryControlDarkShadow()));
                g.drawRect(middleOfActiveThumb, 0, w - middleOfActiveThumb - 1, h - 1);
            }

            g.setColor(MetalLookAndFeel.getPrimaryControlShadow());

            if (this.fillWholeTrack)
            {
                fillMinX    = 1;
                fillMaxX    = w - 2;

                g.drawLine(1, 1, w - 1, 1);
            }

            else if (this.thumbCount > 1)
            {
                if (nextThumb > this.currentIndex)
                {
                    fillMinX    = middleOfActiveThumb;
                    fillMaxX    = middleOfNextThumb;
                }

                else
                {
                    fillMinX    = middleOfNextThumb;
                    fillMaxX    = middleOfActiveThumb;
                }

                if (drawInverted)
                    g.drawLine(1, 1, middleOfActiveThumb, 1);

                else
                    g.drawLine(middleOfActiveThumb, 1, w - 1, 1);
            }

            else
            {
                if (drawInverted)
                {
                    fillMinX    = middleOfActiveThumb;
                    fillMaxX    = w - 2;

                    g.drawLine(1, 1, middleOfActiveThumb, 1);
                }

                else
                {
                    fillMinX    = 1;
                    fillMaxX    = middleOfActiveThumb;

                    g.drawLine(middleOfActiveThumb, 1, w - 1, 1);
                }
            }

            if (h == 6)
            {
                g.setColor(MetalLookAndFeel.getWhite());
                g.drawLine(fillMinX, 1, fillMaxX, 1);

                g.setColor(sliderAltTrackColor);
                g.drawLine(fillMinX, 2, fillMaxX, 2);

                g.setColor(MetalLookAndFeel.getControlShadow());
                g.drawLine(fillMinX, 3, fillMaxX, 3);

                g.setColor(MetalLookAndFeel.getPrimaryControlShadow());
                g.drawLine(fillMinX, 4, fillMaxX, 4);
            }
        }

        else
        {
            int fillMinY,
                fillMaxY,
                middleOfNextThumb   = thumbRects[nextThumb].y +
                                      (thumbRects[nextThumb].height / 2) -
                                      paintRect.y,
                middleOfActiveThumb = thumbRect.y + (thumbRect.height / 2) -
                                      paintRect.y;

            if (middleOfActiveThumb > 0)
            {
                g.setColor(chooseColor(drawInverted,
                                       MetalLookAndFeel.getControlDarkShadow(),
                                       MetalLookAndFeel.getPrimaryControlDarkShadow()));
                g.drawRect(0, 0, w - 1, middleOfActiveThumb - 1);
            }

            if (middleOfActiveThumb < h)
            {
                g.setColor(chooseColor(drawInverted,
                                       MetalLookAndFeel.getPrimaryControlDarkShadow(),
                                       MetalLookAndFeel.getControlDarkShadow()));
                g.drawRect(0, middleOfActiveThumb, w - 1, h - middleOfActiveThumb - 1);
            }

            g.setColor(MetalLookAndFeel.getPrimaryControlShadow());

            if (this.fillWholeTrack)
            {
                fillMinY    = 1;
                fillMaxY    = h - 2;

                g.drawLine(1, 1, 1, h - 1);
            }

            else if (this.thumbCount > 1)
            {
                if (nextThumb > this.currentIndex)
                {
                    fillMinY    = middleOfActiveThumb;
                    fillMaxY    = middleOfNextThumb;
                }

                else
                {
                    fillMinY    = middleOfNextThumb;
                    fillMaxY    = middleOfActiveThumb;
                }

                if (drawInverted)
                {
                    if (leftToRight)
                        g.drawLine(1, middleOfActiveThumb, 1, h - 1);

                    else
                        g.drawLine(w - 2, middleOfActiveThumb, w - 2, h - 1);
                }

                else
                {
                    if (leftToRight)
                        g.drawLine(1, 1, 1, middleOfActiveThumb);

                    else
                        g.drawLine(w - 2, 1, w - 2, middleOfActiveThumb);
                }
            }

            else
            {
                if (drawInverted)
                {
                    fillMinY    = 1;
                    fillMaxY    = middleOfActiveThumb;

                    if (leftToRight)
                        g.drawLine(1, middleOfActiveThumb, 1, h - 1);

                    else
                        g.drawLine(w - 2, middleOfActiveThumb, w - 2, h - 1);
                }

                else
                {
                    fillMinY    = middleOfActiveThumb;
                    fillMaxY    = h - 2;

                    if (leftToRight)
                        g.drawLine(1, 1, 1, middleOfActiveThumb);

                    else
                        g.drawLine(w - 2, 1, w - 2, middleOfActiveThumb);
                }
            }

            if (w == 6)
            {
                g.setColor(chooseColor(!leftToRight,
                                       MetalLookAndFeel.getWhite(),
                                       MetalLookAndFeel.getPrimaryControlShadow()));

                g.drawLine(1, fillMinY, 1, fillMaxY);
                g.setColor(chooseColor(!leftToRight, sliderAltTrackColor,
                                       MetalLookAndFeel.getControlShadow()));

                g.drawLine(2, fillMinY, 2, fillMaxY);
                g.setColor(chooseColor(!leftToRight,
                                       MetalLookAndFeel.getControlShadow(),
                                       sliderAltTrackColor));

                g.drawLine(3, fillMinY, 3, fillMaxY);
                g.setColor(chooseColor(!leftToRight,
                                       MetalLookAndFeel.getPrimaryControlShadow(),
                                       MetalLookAndFeel.getWhite()));

                g.drawLine(4, fillMinY, 4, fillMaxY);
            }
        }

        g.translate(-paintRect.x, -paintRect.y);
    }

    public void scrollByBlock(int direction)
    {
        synchronized(slider)
        {
            int oldValue        = ((MultiSlider)slider).getValueAt(this.currentIndex),
                blockIncrement  = slider.getMaximum() / 10,
                delta           = blockIncrement * ((direction > 0) ? POSITIVE_SCROLL : NEGATIVE_SCROLL);

            ((MultiSlider)slider).setValueAt(this.currentIndex, oldValue + delta);
        }
    }

    public void scrollByUnit(int direction)
    {
        synchronized(slider)
        {
            int oldValue    = ((MultiSlider)slider).getValueAt(this.currentIndex),
                delta       = 1 * ((direction > 0) ? POSITIVE_SCROLL : NEGATIVE_SCROLL);

            ((MultiSlider)slider).setValueAt(this.currentIndex, oldValue + delta);
        }
    }

    protected TrackListener createTrackListener(JSlider slider)
    {
        return new MultiTrackListener();
    }

    /**
     * Track Listener Class tracks mouse movements.
     */
    class MultiTrackListener extends BasicSliderUI.TrackListener
    {
        int                     _trackTop,
                                _trackBottom,
                                _trackLeft,
                                _trackRight;
        transient private int[] firstXY         = new int[2];

        /**
         * If the mouse is pressed above the "thumb" component
         * then reduce the scrollbars value by one page ("page up"),
         * otherwise increase it by one page.  If there is no
         * thumb then page up if the mouse is in the upper half
         * of the track.
         */
        public void mousePressed(MouseEvent e)
        {
            int[]   neighbours  = new int[2];
            boolean bounded     = ((MultiSlider)slider).isBounded();

            if (!slider.isEnabled())
                return;

            currentMouseX       = e.getX();
            currentMouseY       = e.getY();
            firstXY[0]          = currentMouseX;
            firstXY[1]          = currentMouseY;

            slider.requestFocus();

            // Clicked in the Thumb area?
            minmaxIndices[0]    = -1;
            minmaxIndices[1]    = -1;

            for (int i = 0; i < MultiSliderUI.this.thumbCount; i++)
            {
                if (MultiSliderUI.this.thumbRects[i].contains(currentMouseX, currentMouseY))
                {
                    if (minmaxIndices[0] == -1)
                    {
                        minmaxIndices[0]                = i;
                        MultiSliderUI.this.currentIndex = i;
                    }

                    if (minmaxIndices[1] < i)
                        minmaxIndices[1]                = i;

                    switch (slider.getOrientation())
                    {
                        case JSlider.VERTICAL:
                            offset  = currentMouseY - MultiSliderUI.this.thumbRects[i].y;
                            break;

                        case JSlider.HORIZONTAL:
                            offset  = currentMouseX - MultiSliderUI.this.thumbRects[i].x;
                            break;
                    }

                    MultiSliderUI.this.isDragging   = true;
                    thumbRect                       = MultiSliderUI.this.thumbRects[i];

                    if (bounded)
                    {
                        neighbours[0]   = ((i - 1) < 0) ? -1 : (i - 1);
                        neighbours[1]   = ((i + 1) >= MultiSliderUI.this.thumbCount) ? -1 : (i + 1);
                    }

                    else
                    {
                        MultiSliderUI.this.currentIndex = i;

                        ((MultiSlider)slider).setValueIsAdjustingAt(i, true);
                        
                        neighbours[0]                   = -1;
                        neighbours[1]                   = -1;
                    }

                    setThumbBounds(neighbours);
                }
            }
            if (minmaxIndices[0] > -1)
                return;

            MultiSliderUI.this.currentIndex = findClosest(currentMouseX, currentMouseY, neighbours, -1);
            
            thumbRect                       = MultiSliderUI.this.thumbRects[MultiSliderUI.this.currentIndex];
            MultiSliderUI.this.isDragging   = false;

            ((MultiSlider)slider).setValueIsAdjustingAt(MultiSliderUI.this.currentIndex, true);

            Dimension sbSize = slider.getSize();
            int direction = POSITIVE_SCROLL;

            switch (slider.getOrientation())
            {
                case JSlider.VERTICAL:
                    if (thumbRect.isEmpty())
                    {
                        int scrollbarCenter = sbSize.height / 2;
                        
                        if (!drawInverted())
                            direction   = (currentMouseY < scrollbarCenter) ? POSITIVE_SCROLL : NEGATIVE_SCROLL;

                        else
                            direction   = (currentMouseY < scrollbarCenter) ? NEGATIVE_SCROLL : POSITIVE_SCROLL;
                    }

                    else
                    {
                        int thumbY          = thumbRect.y;

                        if (!drawInverted())
                            direction   = (currentMouseY < thumbY) ? POSITIVE_SCROLL : NEGATIVE_SCROLL;

                        else
                            direction   = (currentMouseY < thumbY) ? NEGATIVE_SCROLL : POSITIVE_SCROLL;
                    }

                    break;

                case JSlider.HORIZONTAL:
                    if (thumbRect.isEmpty())
                    {
                        int scrollbarCenter = sbSize.width / 2;

                        if (!drawInverted())
                            direction   = (currentMouseX < scrollbarCenter) ? NEGATIVE_SCROLL : POSITIVE_SCROLL;
                        else
                            direction   = (currentMouseX < scrollbarCenter) ? POSITIVE_SCROLL : NEGATIVE_SCROLL;
                    }
                    
                    else
                    {
                        int thumbX  = thumbRect.x;

                        if (!drawInverted())
                            direction = (currentMouseX < thumbX) ? NEGATIVE_SCROLL : POSITIVE_SCROLL;

                        else
                            direction = (currentMouseX < thumbX) ? POSITIVE_SCROLL : NEGATIVE_SCROLL;
                    }
                    break;
            }

            scrollDueToClickInTrack(direction);

            Rectangle   r   = thumbRect;

            if (!r.contains(currentMouseX, currentMouseY))
            {
                if (shouldScroll(direction))
                {
                    scrollTimer.stop();
                    scrollListener.setDirection(direction);
                    scrollTimer.start();
                }
            }
        }

        /**
         * Sets a track bound for the thumb currently operated.
         */
        private void setThumbBounds(int[] neighbours)
        {
            int halfThumbWidth  = thumbRect.width / 2,
                halfThumbHeight = thumbRect.height / 2;

            switch (slider.getOrientation())
            {
                case JSlider.VERTICAL:
                    _trackTop       = (neighbours[1] == -1) ? trackRect.y : MultiSliderUI.this.thumbRects[neighbours[1]].y + halfThumbHeight;
                    _trackBottom    = (neighbours[0] == -1) ? trackRect.y + (trackRect.height - 1) : MultiSliderUI.this.thumbRects[neighbours[0]].y + halfThumbHeight;
                    break;

                case JSlider.HORIZONTAL:
                    _trackLeft      = (neighbours[0] == -1) ? trackRect.x : MultiSliderUI.this.thumbRects[neighbours[0]].x + halfThumbWidth;
                    _trackRight     = (neighbours[1] == -1) ? trackRect.x + (trackRect.width - 1) : MultiSliderUI.this.thumbRects[neighbours[1]].x + halfThumbWidth;
                    break;
            }
        }

        /*
         * This is a very lazy way to find the closest.  One might want to
         * implement a much faster algorithm.
         */
        private int findClosest(int x, int y, int[] neighbours, int excluded)
        {
            int orientation = slider.getOrientation(),
                rightmin    = Integer.MAX_VALUE,    // for dxw, dy
                leftmin     = -Integer.MAX_VALUE,   // for dx, dyh
                dx = 0,
                dxw = 0,
                dy = 0,
                dyh = 0;

            neighbours[0]   = -1;	// left
            neighbours[1]   = -1;	// right

            for (int i = 0; i < MultiSliderUI.this.thumbCount; i++)
            {
                if (i == excluded)
                    continue;

                switch (orientation)
                {
                    case JSlider.VERTICAL:
                        dy  = MultiSliderUI.this.thumbRects[i].y - y;
                        dyh = (MultiSliderUI.this.thumbRects[i].y + MultiSliderUI.this.thumbRects[i].height) - y;

                        if (dyh <= 0)
                        {
                            // has to be > and not >=
                            if (dyh > leftmin)
                            {
                                leftmin         = dyh;
                                neighbours[0]   = i;
                            }
                        }

                        if (dy >= 0)
                        {
                            if (dy <= rightmin)
                            {
                                rightmin        = dy;
                                neighbours[1]   = i;
                            }
                        }

                        break;

                    case JSlider.HORIZONTAL:
                        dx  = MultiSliderUI.this.thumbRects[i].x - x;
                        dxw = (MultiSliderUI.this.thumbRects[i].x + MultiSliderUI.this.thumbRects[i].width) - x;

                        if (dxw <= 0)
                        {
                            if (dxw >= leftmin)
                            {
                                leftmin = dxw;
                                neighbours[0] = i;
                            }
                        }

                        if (dx >= 0)
                        {
                            // has to be < and not <=
                            if (dx < rightmin)
                            {
                                rightmin = dx;
                                neighbours[1] = i;
                            }
                        }
                        break;
                }
            }

            int closest = (Math.abs(leftmin) <= Math.abs(rightmin)) ? neighbours[0] : neighbours[1];

            return (closest == -1) ? 0 : closest;
        }

        /**
         * Set the models value to the position of the top/left
         * of the thumb relative to the origin of the track.
         */
        public void mouseDragged(MouseEvent e)
        {
            int         thumbMiddle = 0;
            boolean     bounded     = ((MultiSlider)slider).isBounded();
            MultiSlider _slider     = (MultiSlider)MultiSliderUI.this.slider;

            _slider.setValueBeforeStateChange(_slider.getValueAt(MultiSliderUI.this.currentIndex));

            if (!slider.isEnabled())
                return;

            currentMouseX   = e.getX();
            currentMouseY   = e.getY();

            if (!MultiSliderUI.this.isDragging)
            {
                return;
            }

            switch (slider.getOrientation())
            {
                case JSlider.VERTICAL:
                    int halfThumbHeight     = thumbRect.height / 2,
                        thumbTop            = e.getY() - offset;

                    if (bounded)
                    {
                        int[]   neighbours      = new int[2];
                        int     idx             = -1,
                                diff            = e.getY() - firstXY[1];

                        //System.out.println("diff = " + diff);

                        if (e.getY() - firstXY[1] > 0)
                            idx = minmaxIndices[0];

                        else
                            idx = minmaxIndices[1];
                        
                        minmaxIndices[0]        =
                            minmaxIndices[1]    = idx;

                        //System.out.println("idx = " + idx);
                        if (idx == -1)
                            break;

                        //System.out.println("thumbTop = " + thumbTop);
                        neighbours[0]           = ((idx - 1) < 0) ? -1 : (idx - 1);
                        neighbours[1]           = ((idx + 1) >= MultiSliderUI.this.thumbCount) ? -1 : (idx + 1);

                        thumbRect               = MultiSliderUI.this.thumbRects[idx];

                        MultiSliderUI.this.currentIndex = idx;

                        ((MultiSlider)slider).setValueIsAdjustingAt(idx, true);
                        setThumbBounds(neighbours);
                    }

                    thumbTop    = Math.max(thumbTop, _trackTop - halfThumbHeight);
                    thumbTop    = Math.min(thumbTop, _trackBottom - halfThumbHeight);

                    setThumbLocation(thumbRect.x, thumbTop);

                    thumbMiddle = thumbTop + halfThumbHeight;

                    ((MultiSlider)slider).setValueAt(MultiSliderUI.this.currentIndex,
                                                     valueForYPosition(thumbMiddle));
                    break;

                case JSlider.HORIZONTAL:
                    int halfThumbWidth  = thumbRect.width / 2,
                        thumbLeft       = e.getX() - offset;

                    if (bounded)
                    {
                        int[]   neighbours  = new int[2];
                        int     idx         = -1;

                        if (e.getX() - firstXY[0] <= 0)
                            idx = minmaxIndices[0];
                        else
                            idx = minmaxIndices[1];

                        minmaxIndices[0]        =
                            minmaxIndices[1]    = idx;

                        //System.out.println("idx = " + idx);

                        if (idx == -1)
                            break;

                        //System.out.println("thumbLeft = " + thumbLeft);
                        neighbours[0]   = ((idx - 1) < 0) ? -1 : (idx - 1);
                        neighbours[1]   = ((idx + 1) >= MultiSliderUI.this.thumbCount) ? -1 : (idx + 1);

                        thumbRect       = MultiSliderUI.this.thumbRects[idx];

                        MultiSliderUI.this.currentIndex = idx;

                        ((MultiSlider)slider).setValueIsAdjustingAt(idx, true);

                        setThumbBounds(neighbours);
                    }

                    thumbLeft   = Math.max(thumbLeft, _trackLeft - halfThumbWidth);
                    thumbLeft   = Math.min(thumbLeft, _trackRight - halfThumbWidth);

                    setThumbLocation(thumbLeft, thumbRect.y);

                    thumbMiddle = thumbLeft + halfThumbWidth;

                    ((MultiSlider)slider).setValueAt(MultiSliderUI.this.currentIndex,
                                                     valueForXPosition(thumbMiddle));
                    break;

                default:
                    return;
            }
        }

        public void mouseReleased(MouseEvent e)
        {
            if (!slider.isEnabled())
                return;

            offset  = 0;
            scrollTimer.stop();

            if (slider.getSnapToTicks())
            {
                MultiSliderUI.this.isDragging   = false;

                ((MultiSlider)slider).setValueIsAdjustingAt(MultiSliderUI.this.currentIndex, false);
            }
            
            else
            {
                ((MultiSlider)slider).setValueIsAdjustingAt(MultiSliderUI.this.currentIndex, false);

                MultiSliderUI.this.isDragging = false;
            }

            slider.repaint();
        }
    }

    /**
     * A static version of the above.
     */
    static class SharedActionScroller extends AbstractAction
    {
        int     _dir;
        boolean _block;

        public SharedActionScroller(int dir, boolean block)
        {
            _dir = dir;
            _block = block;
        }

        public void actionPerformed(ActionEvent e)
        {
            JSlider         slider  = (JSlider)e.getSource();
            MultiSliderUI   ui      = (MultiSliderUI)slider.getUI();

            if (_dir == NEGATIVE_SCROLL || _dir == POSITIVE_SCROLL)
            {
                int realDir = _dir;

                if (slider.getInverted())
                    realDir = _dir == NEGATIVE_SCROLL ? POSITIVE_SCROLL : NEGATIVE_SCROLL;

                if (_block)
                    ui.scrollByBlock(realDir);

                else
                    ui.scrollByUnit(realDir);
            }
            
            else
            {
                if (slider.getInverted())
                {
                    if (_dir == MIN_SCROLL)
                    {
                        ((MultiSlider)slider).setValueAt(ui.currentIndex,
                                                         slider.getMaximum());
                    }
                    
                    else if (_dir == MAX_SCROLL)
                    {
                        ((MultiSlider)slider).setValueAt(ui.currentIndex,
                                                         slider.getMinimum());
                    }
                }
                
                else
                {
                    if (_dir == MIN_SCROLL)
                    {
                        ((MultiSlider)slider).setValueAt(ui.currentIndex,
                                                         slider.getMinimum());
                    }
                    
                    else if (_dir == MAX_SCROLL)
                    {
                        ((MultiSlider)slider).setValueAt(ui.currentIndex,
                                                         slider.getMaximum());
                    }
                }
            }
        }
    }
}