/*
 * RadioGroupBox.java
 *
 * Version:
 *    $Id: RadioGroupBox.java,v 1.1 2005/06/20 01:27:48 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: RadioGroupBox.java,v $
 *    Revision 1.1  2005/06/20 01:27:48  gap7472
 *    Initial revision
 *
 */

package gui;

import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * A descendent of GroupBox that emulates a "RadioGroup" from the
 * Windows GUI.
 *
 * A RadioGroup is similar in appearance to a GroupBox, but is more
 * powerful when working with two or more radio buttons that belong
 * to the same button group.
 * 
 * Instead of having to create each individual radio button, add it
 * to a group box, position it, and add it to a radio group,
 * callers can simply create a RadioGroupBox and tell it to add radio
 * buttons with the required text. The RadioGroupBox will take care
 * of placing and grouping the radio buttons, and can easily 
 * inform callers when the user changes the selected radio button.
 */
public class RadioGroupBox extends GroupBox implements ActionListener
{
    LinkedList<ActionListener>  listeners;
    ArrayList<JRadioButton>     radioBtns;
    ButtonGroup                 radioGroup;
    boolean                     enabled;

    /**
     * Constructs a new RadioGroupBox without a title and a default
     * of one column per radio button, in a single row.
     */
    public RadioGroupBox()
    {
        this(null, 0);
    }

    /**
     * Constructs a new RadioGroupBox without a title and the
     * specified number of columns of radio buttons.
     * 
     * @see     RadioGroupBox#setColumns
     * @param   cols    The number of desired radio button columns
     */
    public RadioGroupBox(int cols)
    {
        this(null, cols);
    }

    /**
     * Constructs a new RadioGroupBox with the specified title and a
     * default of one column per radio button, in a single row.
     *
     * @see     GroupBox#setText
     * @param   title   The title to display for the new
     *                  RadioGroupBox
     */
    public RadioGroupBox(String title)
    {
        this(title, 0);
    }

    /**
     * Constructs a new RadioGroupBox with the specified title and
     * the specified number of radio button columns.
     *
     * @see     RadioGroupBox#setColumns
     * @see     GroupBox#setText
     * @param   title   The title to display for the new
     *                  RadioGroupBox
     * @param   cols    The number of desired radio button columns
     */
    public RadioGroupBox(String title, int cols)
    {
        super(title);

        setColumns(cols);

        listeners   = new LinkedList<ActionListener>();
        radioBtns   = new ArrayList<JRadioButton>();
        radioGroup  = new ButtonGroup();
    }

    /**
     * Sets the number of radio group columns. Specifying zero for
     * the number of columns indicates that the default of one
     * column per radio button in a single row should be used.
     *
     * @param   cols    The desired number of radio button columns,
     *                  or 0 for the default placement.
     */
    public void setColumns(int cols)
    {
        if (cols != 0)
            super.setLayout(new GridLayout(0, cols));

        else
            super.setLayout(new GridLayout());
    }

    /**
     * This method is used to determine the number of radio group
     * columns.
     *
     * @see     RadioGroupBox#setColumns
     * @return  The number of radio group columns
     */
    public int getColumns()
    {
        return (((GridLayout)getLayout()).getColumns());
    }

    /**
     * This method is unsupported for the RadioGroupBox.
     * RadioGroupBoxes always use GridLayout to arrange radio
     * buttons.
     */
    public void setLayout(LayoutManager mgr)
    {
        // Unsupported; can't throw exception because our grandfather
        // JPanel will call this method from the constructor to set a
        // default layout manager of FlowLayout
    }

    public void setEnabled(boolean enabled)
    {
        this.enabled    = enabled;

        for (int i = 0; i < radioBtns.size(); i++)
        {
            radioBtns.get(i).setEnabled(enabled);
        }        
    }

    public boolean isEnabled()
    {
        return (enabled);
    }

    /**
     * This method is used to determine the index of the selected
     * radio button.
     *
     * @see     RadioGroupBox#setSelectedIndex
     * @return  The index of the selected radio button, or -1 if
     *          no button is selected
     */
    public int getSelectedIndex()
    {
        for (int i = 0; i < radioBtns.size(); i++)
        {
            if (radioBtns.get(i).isSelected())
                return (i);
        }

        return (-1);
    }

    /**
     * This method will set the selected radio button to that
     * with the specified index. Note that this method does not
     * trigger an ActionEvent.
     *
     * @see     RadioGroupBox#getSelectedIndex
     * @param   index   The index of the radio button to set
     *                  as selected
     */
    public void setSelectedIndex(int index)
    {
        radioBtns.get(index).setSelected(true);
    }

    /**
     * This method is used to determine if the radio button
     * with the specified index is selected or not.
     *
     * @see     RadioGroupBox#getSelectedIndex
     * @see     RadioGroupBox#setSelectedIndex
     * @param   index   The index of the radio button whose selection
     *                  status is to be determined
     * @return          True if the specified radio button is
     *                  selected, false otherwise
     */
    public boolean isSelected(int index)
    {
        return (radioBtns.get(index).isSelected());
    }

    /**
     * This method is used to determine the index of the radio button
     * with the specified text.
     *
     * In the case of two radio buttons with the same text, this
     * will return the radio button that was added first.
     *
     * @param   text    The text of the radio button whose index will
     *                  be returned
     * @return          The index of the radio button with the
     *                  specified text, or -1 if a radio button with
     *                  the specified text cannot be found
     */
    public int indexOf(String text)
    {
        for (int i = 0; i < radioBtns.size(); i++)
        {
            if (radioBtns.get(i).getText().equals(text))
                return (i);
        }

        return (-1);
    }

    /**
     * This method will return the JRadioButton with the specified
     * index.
     *
     * @param   index   The index of the radio button to return
     * @return          The radio button with the specified index.
     */
    public JRadioButton get(int index)
    {
        return (radioBtns.get(index));
    }

    /**
     * This method will add a pre-existing radio button to the
     * RadioGroupBox. The radio button is automatically entered
     * into the same group as radio buttons already in the
     * RadioGroupBox.
     */
    public void addRadioBtn(JRadioButton newRadio)
    {
        // Make us aware of changes in selection
        newRadio.addActionListener(this);

        // Add to our list of radio buttons, the radio group, and
        // our own container
        radioBtns.add(newRadio);
        radioGroup.add(newRadio);
        this.add(newRadio);

        // Force a redraw of our own container
        validate();
    }

    /**
     * This method will add a new radio button with the specified
     * text to the RadioGroupBox. The new radio button is
     * automatically entered into the same group as radio buttons
     * already in the RadioGroupBox.
     *
     * The index of the new radio button is returned.
     *
     * @param   text        The text of the new radio button.
     * @param   selected    A boolean value to indicate whether or
     *                      not the new radio button should be the
     *                      selected button of the group.
     * @return              The index of the new radio button.
     */
    public int addRadioBtn(String text, boolean selected)
    {
        JRadioButton    newRadio    = new JRadioButton(text);

        newRadio.setSelected(selected);

        addRadioBtn(newRadio);

        return (radioBtns.size() - 1);
    }

    /**
     * This method will remove the radio button with the specified
     * index from the RadioGroupBox. The removed radio button is
     * returned.
     *
     * @param   index   The index of the radio button to remove.
     * @return          The removed radio button.
     */
    public JRadioButton removeRadioBtn(int index)
    {
        JRadioButton    removeRadio = radioBtns.remove(index);

        radioGroup.remove(removeRadio);
        remove(removeRadio);

        return (removeRadio);
    }

    /**
     * Adds an ActionListener to the RadioGroupBox. The listener will
     * be notified when any radio button is selected.
     *
     * @param   listener    The listener to be added.
     */
    public void addActionListener(ActionListener listener)
    {
        listeners.add(listener);
    }

    /**
     * Removes an ActionListener from the RadioGroupBox.
     *
     * @param   listener    The listener to be removed.
     */
    public void removeActionListener(ActionListener listener)
    {
        listeners.remove(listener);
    }

    /**
     * This method is called by a radio button in the RadioGroupBox
     * when it is selected. This method will notify all attached 
     * ActionListener(s) of the action, if any are attached.
     *
     * @param   e   An event describing the action that caused this
     *              method to be invoked.
     */
    public void actionPerformed(ActionEvent e)
    {
        ActionEvent newEvent;
        
        newEvent    = new ActionEvent(this,
                                      e.getID(),
                                      e.getActionCommand(),
                                      e.getWhen(),
                                      e.getModifiers());

        for (ActionListener curListener : listeners)
        {
            curListener.actionPerformed(newEvent);
        }
    }
}