/*
 * LinkManager.java
 *
 * Version:
 *    $Id: LinkManager.java,v 1.5 2005/06/20 01:43:23 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: LinkManager.java,v $
 *    Revision 1.5  2005/06/20 01:43:23  gap7472
 *    -Package changed from GreedyAlg -> util.
 *
 *    Revision 1.4  2005/06/08 03:13:57  gap7472
 *    -Added an additional constructor that takes an initial capacity for
 *     the Heap, so that it can initialize its internal HashMap to
 *     expect a certain number of items, thereby increasing performance.
 *
 *    Revision 1.3  2005/04/18 06:19:08  gap7472
 *    Renamed file to be simpler.
 *
 *    Revision 1.2  2005/04/13 06:17:36  gap7472
 *    Modified to use HashMap instead of Hashtable.
 *
 *    Revision 1.1  2005/04/13 06:07:53  gap7472
 *    Initial revision
 *
 */

package util;

import java.util.*;

/**
 * This class keeps track of the linking between objects
 * by using a map and groups of HashSets.
 *
 * @author  Guy Paddock
 */
public class LinkManager<T>
{
    /**
     * The map that links an object to the hash set
     * in which the object and the objects it's connected
     * to are stored.
     */
    private HashMap<T, HashSet<T>> setMap;

    /**
     * Constructor for the LinkManager.
     */
    public LinkManager()
    {
        setMap  = new HashMap<T, HashSet<T>>();
    }

    /**
     * Constructor for the LinkManager, with an additional parameter
     * to specify the number of elements that the LinkManager should
     * expect.
     *
     * @param   initialCapacity The initial number of elements that
     *                          the LinkManager should expect.
     */
    public LinkManager(int initialCapacity)
    {
        setMap  = new HashMap<T, HashSet<T>>(initialCapacity);
    }

    /**
     * This method is used to tell if two objects are already linked
     * or not.
     *
     * @param   first   The first of the two objects.
     * @param   second  The second of the two objects.
     * @return          True if the objects are linked, false otherwise.
     */
    public boolean areConnected(T first, T second)
    {
        // We can't say that they're connected if we
        // don't have any record of them
        if (!setMap.containsKey(first) ||
            !setMap.containsKey(second))
            return false;

        // If the set under the key of either object contains the other
        // object, then the two are connected
        if (setMap.get(first) == setMap.get(second))
            return true;

        // They aren't linked
        else
            return false;
    }

    /**
     * This method adds a link between two objects.
     *
     * @param   first   The first of the two objects to link.
     * @param   second  The second of the two objects to link.
     */
    public void addLinked(T first, T second)
    {
        HashSet<T>  firstSet    = null,
                    secondSet   = null,
                    newSet      = null;

        // No sense in adding things we already have
        if (areConnected(first, second))
            return;

        // If we have the first object already in the map,
        // grab the instance of its set
        firstSet    = setMap.get(first);

        // Now try to do the same for the second object
        secondSet   = setMap.get(second);

        // We might have two distinct sets on our hands
        if ((firstSet != null) && (secondSet != null))
        {
            // Take the larger of the two sets as our
            // shared instance, and integrate the
            // members of the smaller set
            if (firstSet.size() > secondSet.size())
            {
                newSet  = firstSet;

                newSet.addAll(secondSet);

                // If the set instances are not equal, we have orphaned set
                // instances, which is bad. Go through and make all the
                // members of the two sets have the same instance of the
                // shared set.
                // 
                // This is the worst case scenario of O(n) and only occurs when
                // sets are being linked together, which is somewhat rare.
                if (firstSet != secondSet)
                {
                    // Fix orphans from the second set
                    for (T curObject : secondSet)
                        setMap.put(curObject, newSet);
                }
            }

            // Handle second set being larger than the first
            else
            {
                newSet = secondSet;

                newSet.addAll(firstSet);

                // See note above
                if (firstSet != secondSet)
                {
                    // Fix orphans from the first set
                    for (T curObject : firstSet)
                        setMap.put(curObject, newSet);
                }
            }
        }

        // Handle when we only have a set for the first object
        else if (firstSet != null)
        {
            newSet  = firstSet;

            newSet.add(second);
        }

        // Handle when we only have a set for the second object
        else if (secondSet != null)
        {
            newSet  = secondSet;

            newSet.add(first);
        }

        else
        {
            // If we don't yet have a shared instance, then we
            // don't yet know about either object, so create a
            // new set instance to contain them. They have no
            // connection to anything else at this point.
            if (newSet == null)
                newSet  = new HashSet<T>();

            newSet.add(first);
            newSet.add(second);
        }

        // Store the shared set instance under both objects' keys
        setMap.put(first, newSet);
        setMap.put(second, newSet);
    }

    /**
     * This returns the internal hashtable -- used only for
     * debugging.
     *
     * @return  the value stored in setMap
     */
    private final HashMap<T, HashSet<T>> getMap()
    {
        return setMap;
    }

    /**
     * The main method. Used for debugging.
     * 
     * @param   args    The command-line arguments (not used).
     */
    public static void main(String args[])
    {
        LinkManager<String> testManager = new LinkManager<String>();

        System.out.println("Strings: 1 & 2");
        System.out.println("Are Connected?: " + testManager.areConnected("1", "2"));

        testManager.addLinked("1", "2");
        System.out.println("Connected 1 & 2");
        System.out.println("Are Connected?: " + testManager.areConnected("1", "2"));
        
        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println();

        System.out.println("Connecting 2 & 3");
        testManager.addLinked("2", "3");
        System.out.println("Are 2 & 3 Connected?: " + testManager.areConnected("2", "3"));
        System.out.println("Are 1 & 3 Connected?: " + testManager.areConnected("1", "3"));

        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println(testManager.getMap().get("3") + " " + testManager.getMap().get("3").hashCode());
        System.out.println();

        System.out.println("Connecting 1 & 4");
        testManager.addLinked("1", "4");
        System.out.println("Are 1 & 4 Connected?: " + testManager.areConnected("1", "4"));
        System.out.println("Are 2 & 4 Connected?: " + testManager.areConnected("2", "4"));
        System.out.println("Are 3 & 4 Connected?: " + testManager.areConnected("3", "4"));

        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println(testManager.getMap().get("3") + " " + testManager.getMap().get("3").hashCode());
        System.out.println(testManager.getMap().get("4") + " " + testManager.getMap().get("4").hashCode());
        System.out.println();

        System.out.println("Connecting 3 & 7");
        testManager.addLinked("3", "7");
        System.out.println("Are 1 & 7 Connected?: " + testManager.areConnected("1", "7"));
        System.out.println("Are 2 & 7 Connected?: " + testManager.areConnected("2", "7"));
        System.out.println("Are 3 & 7 Connected?: " + testManager.areConnected("3", "7"));
        System.out.println("Are 4 & 7 Connected?: " + testManager.areConnected("4", "7"));

        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println(testManager.getMap().get("3") + " " + testManager.getMap().get("3").hashCode());
        System.out.println(testManager.getMap().get("4") + " " + testManager.getMap().get("4").hashCode());
        System.out.println(testManager.getMap().get("7") + " " + testManager.getMap().get("7").hashCode());
        System.out.println();

        System.out.println("Adding 5 & 6");
        testManager.addLinked("5", "6");

        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println(testManager.getMap().get("3") + " " + testManager.getMap().get("3").hashCode());
        System.out.println(testManager.getMap().get("4") + " " + testManager.getMap().get("4").hashCode());
        System.out.println(testManager.getMap().get("5") + " " + testManager.getMap().get("5").hashCode());
        System.out.println(testManager.getMap().get("6") + " " + testManager.getMap().get("6").hashCode());
        System.out.println(testManager.getMap().get("7") + " " + testManager.getMap().get("7").hashCode());
        System.out.println();

        System.out.println("Connecting 6 & 7");
        testManager.addLinked("6", "7");

        System.out.println();
        System.out.println(testManager.getMap().get("1") + " " + testManager.getMap().get("1").hashCode());
        System.out.println(testManager.getMap().get("2") + " " + testManager.getMap().get("2").hashCode());
        System.out.println(testManager.getMap().get("3") + " " + testManager.getMap().get("3").hashCode());
        System.out.println(testManager.getMap().get("4") + " " + testManager.getMap().get("4").hashCode());
        System.out.println(testManager.getMap().get("5") + " " + testManager.getMap().get("5").hashCode());
        System.out.println(testManager.getMap().get("6") + " " + testManager.getMap().get("6").hashCode());
        System.out.println(testManager.getMap().get("7") + " " + testManager.getMap().get("7").hashCode());
        System.out.println();
    }
}