/*
 * MemoryWatcher.java
 *
 * Version:
 *    $Id: MemoryWatcher.java,v 1.1 2005/06/20 01:43:23 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: MemoryWatcher.java,v $
 *    Revision 1.1  2005/06/20 01:43:23  gap7472
 *    Initial revision
 *
 */

package util;

/**
 * This class implements a thread that tracks heap memory usage
 * over a period of time.
 *
 * @author  Guy Paddock (gap7472@rit.edu)
 */
public class MemoryWatcher extends SleepThread
{
    /**
     * The instance of the current Java Runtime Environment. This is
     * needed to obtain the current heap size.
     */
    private Runtime runtime;

    /**
     * The current heap memory usage. This value is obtained by a
     * call to getCurUsage().
     *
     * @see MemoryWatcher#getCurUsage
     */
    private long    curMem,

    /**
     * The minimum heap memory usage over the monitoring period. This
     * value is obtained by a call to getMinUsage().
     *
     * @see MemoryWatcher#getMinUsage
     */
                    minMem,

    /**
     * The maximum heap memory usage over the monitoring period. This
     * value is obtained by a call to getMaxUsage().
     *
     * @see MemoryWatcher#getMaxUsage
     */
                    maxMem,

    /**
     * The number of samples of memory usage that have been taken.
     * This value is incremented by the thread each time it samples
     * memory usage. This is then used to calculate the average
     * memory usage, which can be obtained by a call to
     * getAvgUsage().
     *
     * @see MemoryWatcher#getAvgUsage
     */
                    numSamples,

    /**
     * The sum of memory usage samples. The thread adds on to this
     * value each time it samples memory usage. This is then used to
     * calculate the average memory usage, which can be obtained by
     * a call to getAvgUsage().
     *
     * @see MemoryWatcher#getAvgUsage
     */
                    samples;
    /**
     * This is set to true while the thread is running. A call to
     * stopWatch() will set this to false to stop the thread.
     */
    private boolean keepRunning;


    /**
     * Constructor for MemoryWatcher.
     */
    public MemoryWatcher()
    {
        runtime     = Runtime.getRuntime();

        reset();
    }

    /**
     * Used to obtain the current heap memory usage in bytes.
     *
     * @return  The current heap size in bytes, or 0 if the thread
     *          has never run.
     */
    public long getCurUsage()
    {
        return (curMem);
    }

    /**
     * Used to obtain the minimum heap memory usage during the
     * entire time the thread has run.
     *
     * @return  The minimum heap size in bytes, or 0 if the thread
     *          has never run.
     */
    public long getMinUsage()
    {
        return (minMem);
    }

    /**
     * Used to obtain the maximum heap memory usage during the
     * entire time the thread has run.
     *
     * @return  The maximum heap size in bytes, or 0 if the thread
     *          has never run.
     */
    public long getMaxUsage()
    {
        return (maxMem);
    }

    /**
     * Used to obtain the average heap memory usage during the
     * entire time the thread has run.
     *
     * @return  The average heap size in bytes, or 0 if the thread
     *          has never run.
     */
    public long getAvgUsage()
    {
        if (numSamples > 0)
            return (samples / numSamples);

        // No data to average
        else
            return 0;
    }

    /**
     * The actual heart of the thread. This runs in a loop, taking
     * samples of memory usage every 500ms until stopWatch()
     * is called.
     *
     * @see MemoryWatcher#stopWatch
     */
    public void run()
    {
        keepRunning = true;

        minMem      = runtime.totalMemory() - runtime.freeMemory();
        maxMem      = minMem;
        samples     = minMem;
        numSamples  = 1;

        // Keep running until stopWatch() is called
        while (keepRunning)
        {
            curMem  = runtime.totalMemory() - runtime.freeMemory();

            // Check for a new maximum
            if (curMem > maxMem)
                maxMem  = curMem;

            // Check for a new minimum
            if (curMem < minMem)
                minMem  = curMem;

            samples += curMem;
            
            numSamples++;

            // Sleep to prevent excessive updating
            sleep(500);
        }
    }

    /**
     * This causes the thread to stop watching memory usage and
     * terminate.
     */
    public void stopWatch()
    {
        keepRunning = false;
    }

    /**
     * This resets all of the counters of the watcher to zero.
     */
    public void reset()
    {
        minMem      = 0;
        maxMem      = 0;
        samples     = 0;
        numSamples  = 0;
    }
}