/*
 * SuffixTest.java
 *
 * Version:
 *    $Id: SuffixTest.java,v 1.9 2005/06/20 01:54:31 gap7472 Exp gap7472 $
 *
 * Revisions:
 *    $Log: SuffixTest.java,v $
 *    Revision 1.9  2005/06/20 01:54:31  gap7472
 *    -Package changed from SuffixTree -> util.suffixtree.
 *
 *    Revision 1.7  2005/03/16 07:26:19  gap7472
 *    Fixed implementation of the search so that it now pertains
 *    to provided specifications.
 *
 *    Revision 1.6  2005/03/16 05:57:55  gap7472
 *    Sync-ed with MultiWordSuffixTree, then added code to locate
 *    multiple matches for a given search phrase.
 *
 *    Revision 1.2  2005/03/09 07:09:28  gap7472
 *    Minor comment changes.
 *
 *    Revision 1.1  2005/03/09 07:01:48  gap7472
 *    Initial revision
 *
 */

package util.suffixtree;

import java.util.*;

/**
 * This class tests a SuffixTree.
 *
 * @author  Guy Paddock
 */
public class SuffixTest
{
    private static SuffixTree  myTree;

    /**
     * The main method.
     *
     * @param   args    Command-line arguments
     */
    public static void main(String args[])
    {
        if ((args.length != 1) && (args.length != 2))
        {
            System.err.println("Usage: SuffixTest <Word> [Suffix to Find]");
            System.exit(-1);
        }

        // Create the tree
        myTree      = new SuffixTree(args[0]);

        // Print the tree
        printTree();

        // Test findSuffix
        if (args.length == 2)
            testFindTarget(args[1]);
    }

    /**
     * This method prints a textual depiction of the
     * suffix tree to the screen.
     */
    public static void printTree()
    {
        // Recursively trace down the arcs, printing
        // out the tree
        followInternalNode(myTree.getRootNode(), "|");
    }

    /**
     * This method tests the suffix tree's ability
     * to locate target strings.
     *
     * @param   suffix  The target to attempt to locate
     */
    public static void testFindTarget(String target)
    {
        TreeMap<Integer, String>    matches;
        Iterator<Integer>           keyIterator;
        Integer                     curKey;

        // Print a blank line
        System.out.println();

        try
        {
            // Attempt to get a mapping of position->target matches
            matches = myTree.findTarget(target);

            // Print out the position if we matched the target
            if (matches != null)
            {
                keyIterator = matches.keySet().iterator();

                System.out.println("\"" + target + "\" matched the following suffixes:");

                while (keyIterator.hasNext())
                {
                    curKey  = keyIterator.next();

                    System.out.println(matches.get(curKey) + " at " + curKey);
                }
            }

            else
                System.out.println("\"" + target + "\" was not located.");
        }

        catch (ContainmentException e)
        {
            System.out.println("\"" + target + "\" was contained.");
        }
    }

    /**
     * This method recursively traces down the SuffixTree,
     * printing the values it encounters into a textual-tree.
     *
     * @param   iNode       the internal node from which to trace
     * @param   indentStr   used for proper formatting of the tree; it is 
     *                      applied to the front of the output and is built
     *                      recursively, so with each deeper level of recursion,
     *                      the output also gets deeper
     */
    public static void followInternalNode(InternalNode iNode, String indentStr)
    {
        Arc             curCastArc;
        String          curArcLabel;
        Node            curArcHead;
        LeafNode        curLeafHead;
        InternalNode    curInodeHead;

        for (AbstractArc curArc : iNode.getArcs().values())
        {
            // Ensure that the current arc is not empty
            if (!(curArc instanceof EmptyArc))
            {
                curCastArc  = (Arc)curArc;
                curArcHead  = curCastArc.getHead();
                curArcLabel = curCastArc.getLabel();

                // Format the output using the indentStr, then print the
                // current arc's label
                System.out.println(indentStr);
                System.out.println(indentStr + "-+" + curArcLabel);
                System.out.println(indentStr + " |");

                // If we encounter a LeafNode, simply print its
                // stored position value
                if (curArcHead instanceof LeafNode)
                {
                    curLeafHead     = (LeafNode)curArcHead;

                    System.out.printf("%s |-@%s\n", indentStr, curLeafHead.getPos());
                }

                // If we encounter another InternalNode, recursively call
                // this method to deal with it
                else
                {
                    curInodeHead    = (InternalNode)curArcHead;

                    followInternalNode(curInodeHead, indentStr + " |");
                }
            }
        }
    }
}